﻿/*
* ============================================================================
*  Name        : Service.as
*  Part of     : Nokia Flash Lite
*  Description : Defines the class which helps the contents to request for a 
* service and pass input and output parameters.
*                
*  Version     :
*
*  Copyright © 2008-2009 Nokia. All rights reserved.
*  This material, including documentation and any related 
*  computer programs, is protected by copyright controlled by 
*  Nokia. All rights are reserved. Copying, including 
*  reproducing, storing, adapting or translating, any 
*  or all of this material requires the prior written consent of 
*  Nokia. This material also contains confidential 
*  information which may not be disclosed to others without the 
*  prior written consent of Nokia
* ============================================================================
*/

import com.nokia.impl.CUtils;
import com.nokia.lib.ErrorValue;
import com.nokia.impl.ServiceProxy;
import com.nokia.impl.ServiceUtilsProxy;
import com.nokia.impl.Callback;

class com.nokia.lib.Service {
	/*
	 Function: Constructor
	
	 Initializes the service parameters and creates the service 
	 instance.
	
	 Parameters:
	 aServiceName - The name of Service.
	 aInterfaceName - The name of the interface 
	 aInitialParam - The initial param to be passed during the
	  service instantiation
	
	 Returns:
	 Throws execption if service or interface name is null.
	*/
	function Service(aServiceName:String, aInterfaceName:String, aVersion:Object, aInitialParam:Object) {
		iError = ErrorValue.EErrorNone;
		this.iServiceName = aServiceName;
		this.iInterfaceName = aInterfaceName;

		if ((null == aServiceName || null == aInterfaceName)) {
			iError = ErrorValue.EUndefinedDataType;
			return;
		}
		
		if ((undefined == aServiceName || undefined == aInterfaceName)) {
			iError = ErrorValue.EUndefinedDataType;
			return;
		}
		
		iServiceProxy = new ServiceProxy();
		if ((iServiceProxy != null) &&(iServiceProxy != undefined)) {
			CreateServiceReq(aVersion,aInitialParam);
			if(iError){
				iServiceProxy = null;//throw new RaiseException(ErrorValue.EServiceNotFound); //error loading service provider
			}
			else{
				iCallback = new Callback();
				iServiceProxy.addListener(iCallback);
			}

		}
		else{
			iError = ErrorValue.EServiceNotFound;
			iServiceProxy = null;
		} //service provider loading failed

		this.__resolve = function(methodName) {
			return function () {
			if ((null != this.iInterfaceName) && (undefined != this.iInterfaceName)) {
				if (0 == arguments.length) {
					var outputObj:Object = null;
					var inputObj:Object = null;
					outputObj = iServiceProxy.ExecuteSyncReq(methodName, inputObj);
					return outputObj;			
				
				}else if (arguments.length == 1) {					
					return iServiceProxy.ExecuteSyncReq(methodName, arguments[0]);		
				
				} else if ((arguments.length >= 2) && (arguments.length <= 3 ) ){
	
					var aTransID:Number = CUtils.GetTransactionID();
					if((arguments[0] == null) ||(arguments[0] == undefined) ){
						var errOutput:Object ={ErrorCode:ErrorValue.EUndefinedDataType};
						return errOutput;
					}

					if( (typeof(arguments[1]) != "object") &&(typeof(arguments[1]) != "function") ){
						var errVar = {ErrorCode:ErrorValue.EInvalidInputError};
						return errVar;							
						}						
					CUtils.Store(aTransID, arguments[1]);		
					
					var callbackConf:Object;
					if((arguments[2] == null) ||(arguments[2] == undefined) ){						
						callbackConf = null;
					}
					else{
						callbackConf = arguments[2];						
					}					
					return iServiceProxy.ExecuteAsyncReq(methodName, aTransID, arguments[0],callbackConf);
				}
				else //in case input variable count exceeds 3
				{
				var errVar = {ErrorCode:ErrorValue.EInvalidInputError};
				return errVar;
				}
			}
			
			};
			// return function
		};
		//resolve function
	}
	

	/*
	 Function: Cancel
	
	 Cancels the specified asynchronous service.
	
	 Parameters:
	 aTransID - The Transaction ID for the asynchronous service that has to be cancelled.
	
	 Returns:
	 Returns true if successful.
	*/
	function Cancel(aTransID:Number):Boolean {
		var output:Object = iServiceProxy.CancelAsyncReq(aTransID);
		if(output.ErrorCode == 0){
			CUtils.Clean(aTransID); //cleanup callback
			return true;
		}
		return false;
	}
	/*
	 Function: UnloadService
	
	 Unloads the specified Service Provider.
	
	 Returns:
	 Returns true if successful.
	*/
	function UnloadService():Boolean {
		if(!iServiceProxy){
			return false;
		}
		iServiceProxy.UnloadService();
		iServiceName = null;
		iInterfaceName = null;
		
		delete iServiceProxy;
		iServiceProxy = null;
		
		return true;
	}	
	/*
	 Function: Name
	
	 Returns the requested service name.
	
	 Parameters:
	 None.
	
	 Returns:
	 Returns the requested service name.
	*/
	function Name():String {
		return this.iServiceName;
	}
	
	/*
	 Function: InterfaceName
	
	 Returns the requested interface name.
	
	 Parameters:
	 None.
	
	 Returns:
	 Returns the requested interface name.
	*/
	function InterfaceName():String {
		return this.iInterfaceName;
	}

	/*
	 Function: CreateServiceReq
	
	 Creates the specified service instance.
	
	 Parameters:
	 aVersion - Version Information of service Provider 
	 aInitialParam - The initial parameter to be passed during service
	  instantiation.
	
	 Returns:
	 None.
	*/
	private function CreateServiceReq(aVersion:Object, aInitialParam:Object){

		if (aInitialParam) {
			if (aVersion) {
				iError = iServiceProxy.Load(iServiceName, iInterfaceName, aVersion, aInitialParam);
			} else {
				iError = iServiceProxy.Load(iServiceName, iInterfaceName, null, aInitialParam);
			}
		} else {
			if (aVersion) {
				iError = iServiceProxy.Load(iServiceName, iInterfaceName, aVersion, null);
			} else {
				iError = iServiceProxy.Load(iServiceName, iInterfaceName, null, null);
			}
		}		
		
	}
	
	

	// The name of the service to be instantiated.
	private var iServiceName:String = null;
	// The name of the interface to be instantiated.
	private var iInterfaceName:String = null;
	// Stores the error value.
	var iError:Number = 0 ;
	//Service Proxy Instance 
	private var iServiceProxy = null;
	private var iCallback = null;
}