/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the BindingHelper class. The Binding Helper class provides helper function to convert
 *                data from Action Script to LIW specific data type and Vice versa.                 
 
 *
*/


#ifndef __BINDING_HELPER_H__
#define __BINDING_HELPER_H__

#include <liwservicehandler.h>
#include <liwvariant.h>
#include <liwvarianttype.hrh>
#include <liwgenericparam.h>
#include <liwcommon.hrh>
#include <e32base.h> 

#include "flashliteExtensionInterfaces.h"

//forward Declaration
class ServiceProxy;
class ExtensionInterface;

class CFlvVarHandle : public CBase
    {
public:
    /**
     Constructs a new CFlvVarHandle object.
     
     @param aEI The ExtensionInterface pointer which will be used for cleaning up of mamber variable iVarHandle.
     @return a new CFlvVarHandle object and puts it on to Cleanup Stack
     */
    static CFlvVarHandle* NewLC(ExtensionInterface* aEI);

    /**
     Returns Handle to FL_VAR_HANDLE.
     
     @return a Hanlde to variable created on core player side of type FL_VAR_HANDLE.
     */
    FL_VAR_HANDLE Handle();

    /**
     Destructor
     */
    ~CFlvVarHandle();

private:

    /**
     Constructor.
     
     @param pointer to ExtensionInterface which will be used to clenup of iVarHandle.
     */
    CFlvVarHandle(ExtensionInterface* aEI) :
        iExtInterface(aEI)
        {
        }
    ;

    /**
     Second Phase constructor.      
     */
    void ConstructL();

private:
    //members

    FL_VAR_HANDLE iVarHandle;

    ExtensionInterface* iExtInterface;
    };

class BindingHelper
    {
public:

    /**
     Converts the input params in Object passed by flash content to the
     LIW specific data types.

     @param aFnInfo The FnInfo pointer of Caller function.
     @param aObject The flash specific data in an Object.
     @param inps On return contains the input param converted to LIW specific data type.
     */
    static void ConstructInputParamsL(FnInfo* aFnInfo,
            FL_OBJECT_HANDLE aObject, CLiwGenericParamList& aInps);

    /**
     Converts the output params from LIW specific data type to Action Script specific Object
     data type.
     
     @param aExtInterface ExtensionManager reference to be used to create/set/get values from Core player
     @param aOutParamList The LIW specific data returned from SAPI.
     @param aOutObject On return contains Action Script specific output.
     */
    static void
            ConstructOutputParamL(ExtensionInterface* aExtInterface,
                    CLiwGenericParamList& aOutParamList,
                    FL_OBJECT_HANDLE* aOutObject);

    /**
     Converts the output params from LIW specific data type to Action Script specific Object
     data type.
     
     @param aExtInterface ExtensionManager reference to be used to create/set/get values from Core player
     @param aParamVar The LIW specific data.
     @param aVar On return contains the Action Script specific output 
     */
    static void ConstructOutputParamL(ExtensionInterface* aExtInterface,
            TLiwVariant& aParamVar, FL_VAR_HANDLE* aVar);

    /**
     Converts the output params from LIW specific data type to Action Script specific Object
     data type.
     
     @param aExtInterface ExtensionManager reference to be used to create/set/get values from Core player
     @param aParamVar The LIW specific data.
     @param aVar On return contains the Action Script specific output 
     */
    static void HandleOutPutObjectL(ExtensionInterface* aExtInterface,
            TLiwVariant& aParamVar, FL_VAR_HANDLE* aVar);

    /**
     Sets the version meta data options for the criteria item.

     @param aExtInterface ExtensionManager Interface Refernece.
     @param aCrit The criteria for which the list of service providers is needed.
     @param aVerParams Version information.
     @return Result of the function invocation.
     */
    static TBool SetMetaDataForCriteriaItemL(
            ExtensionInterface* aExtInterface, CLiwCriteriaItem* aCrit,
            FL_OBJECT_HANDLE aVerParams);

    /**
     Gets the min and max version number for which the list of providers
     is queried.
     
     @param aExtInterface ExtensionManager Interface Refernece.    
     @param aMinVersion The minimum version number.
     @param aMaxVersion The maximum version number.
     @param aVerParams The Version parameter Object.
     @return Result of the function invocation.
     */
    static TBool GetMinAndMaxVersionL(ExtensionInterface* aExtInterface,
            TReal& aMinVersion, TReal& aMaxVersion,
            FL_OBJECT_HANDLE aVerParams);

    static HBufC* StringToDescriptorL(const char* aString);

    static char* Convert8BitDescritptorToStringL(const TDesC8& aDes);
    
    static ArgInfo* GetArgInfoLC(TInt aArgCnt, FL_VAR_HANDLE* aArgList, char* aName );
    
    static void PointerArrayCleanup(TAny* aArray);


private:

    static TBool IsInternalProperty(TDesC8& aPropName);
    /**
     Converts the List/Array input data to the TLiwVariant data type of LIW.
     
     @param aListVar On return contains the List/Array  converted to LIW data specific type.
     @param aObject The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleListL(FnInfo* aFnInfo, TLiwVariant& aListVar,
            FL_OBJECT_HANDLE aObject);

    /**
     Converts the Object input data to the TLiwVariant data type of LIW.
     
     @param aObjectVar On return contains the Object converted to LIW data specific type.
     @param aVar The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleObjectL(FnInfo* aFnInfo, TLiwVariant& aObjectVar,
            FL_VAR_HANDLE aVar);

    /**
     Converts the String input data to the TLiwVariant data type of LIW.
     
     @param aStrVar On return contains the String value converted to LIW data specific type.
     @param aVar The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleStringL(FnInfo* aFnInfo, TLiwVariant& aStrVar,
            FL_VAR_HANDLE aVar);

    /**
     Converts the float input data to the TLiwVariant data type of LIW.

     @param aFloatVar On return contains the float value converted to LIW data specific type.
     @param aVar The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleFloat(FnInfo* aFnInfo, TLiwVariant& aFloatVar,
            FL_VAR_HANDLE aVar);

    /**
     Converts the bool input data to the TLiwVariant data type of LIW.

     @param aBoolVar On return contains the bool value converted to LIW data specific type.
     @param aVar The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleBool(FnInfo* aFnInfo, TLiwVariant& aBoolVar,
            FL_VAR_HANDLE aVar);

    /**
     Converts the Date input data to the TLiwVariant data type of LIW.

     @param aDateVar On return contains the Date value converted to LIW data specific type.
     @param aVar The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleDateL(FnInfo* aFnInfo, TLiwVariant& aDateVar,
            FL_OBJECT_HANDLE aObject);

    /**
     Converts the Int input data to the TLiwVariant data type of LIW.

     @param aIntVar On return contains the Int value converted to LIW data specific type.
     @param aVar The Action Script variable contaning the input value to be converted to LIW specific
     data type.
     */
    static void HandleInt(FnInfo* aFnInfo, TLiwVariant& aIntVar,
            FL_VAR_HANDLE aVar);

    /**
     This function converts the value stored in the Action Script Variable to the TLiwVariant, which
     is a LIW specific data type.
     
     @param aFnInfo The FnInfo pointer of Caller function.
     @param inpVariant On return contains the LIW specific data type.
     @param node Node whose data is to converted to LIW specific data type.
     */
    static void ConstructInputParamFromNodeL(FnInfo* aFnInfo,
            TLiwVariant& aInpVariant, FL_VAR_HANDLE aVar, TBool &aIsVartypeNull);

    /**
     Converts the TGenericParamId passed by the service provider
     to string, as we can pass only string values to flash content.

     @param aParamID The TGenericParamId value.
     @return The string value for the corresponding TGenericParamId.
     */
    static const TDesC8& GetParamAsString(TInt aParamID);

    };

#endif //_BINDING_HELPER_H_
