/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginIMEControl class implementation
*
*/


// INCLUDE FILES
#include "saplayer_build.h"
#include "platformbuild.h"
#include "platformclib.h"

#include <bldvariant.hrh>
#include <fepitfr.h>                // For MFepInlineTextFormatRetriever
#include <coeaui.h>
#include <aknappui.h>
#include <aknedsts.h>
#include <AknUtils.h>
#include <avkon.rsg>
#include <aknenv.h>
#include <AknSettingCache.h>
#include <featmgr.h>
#include "flashpluginimecontrol.h"
#include "flashplugincontrol.h"
#include "flash_engine.h"
#include "commondef.h"			    // For FI_Text
#include "amp_mtextnavigation.h"
#include "amp_returnstring.h"

extern void NPPDebug( TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... );

// CONSTANTS
const TUint KDocMaxLength = 0x7FFFFFFFL;
const TUint KLineFeed     = 0x0A;
const TUint KCharCode1    = 0x2029;
const TUint KCharCode2    = 0x2028;
const TUint KCharCode3    = 0x21b2;
const TUint KCharCode4    = 35;
const TUint KSingleChar   = 1;

#ifdef FEATURE_INLINE_TEXT_ENTRY


////////////////////////////////////////////////////////////////////////////////
// InputModeSelector
////////////////////////////////////////////////////////////////////////////////

InputModeSelector* InputModeSelector::NewL(AMP_MTextNavigation* pNav, 
        								   MFlashImeClientIf* aPluginControl)
{
	InputModeSelector* self = new (ELeave) InputModeSelector(pNav, aPluginControl);
	
	CleanupStack::PushL(self);
	self->ConstructL(pNav);
	CleanupStack::Pop();
	
	return self;
}

void InputModeSelector::ConstructL(AMP_MTextNavigation* pNav)
{
	long len       = pNav->GetTextLength();
	long maxLength = pNav->GetMaxChars();

	AMP_ReturnString ampString(FI_UTF16EncodingForm);
	FI_ReturnString* fiReturnString = ampString.Wrapper();
	FI_Result res =  pNav->GetText(0, len, fiReturnString);

	if(res == FI_ResultOK)
	{
		const FI_UTF16Char* chars = ampString.GetTemporaryUTF16String();
		if(chars)
		{
		    mpInitialString = new (ELeave) FI_UTF16Char[len+1];
			if(mpInitialString)
			{
				FlashMemCpy(mpInitialString, chars,(len+1) * sizeof(FI_UTF16Char));
				mLen=len;
			}
		}
	}
}

InputModeSelector::InputModeSelector(AMP_MTextNavigation* /*pNav*/, 
                                     MFlashImeClientIf* aPluginControl)
                                     : mpInitialString(0), 
                                       mLen(0),
                                       iPluginControl(aPluginControl)
{
}

InputModeSelector::~InputModeSelector()
{
	if (mpInitialString)
		delete [ ] mpInitialString;
}


TBool InputModeSelector::IsInComplexLanguageMode()
{
	if ((iPluginControl->CurrentInputLanguage() == ELangThai) ||
		(iPluginControl->CurrentInputLanguage() == ELangArabic) ||
		(iPluginControl->CurrentInputLanguage() == ELangFarsi) ||
		(iPluginControl->CurrentInputLanguage() == ELangHebrew) ||
		(iPluginControl->CurrentInputLanguage() == ELangHindi) ||
		(iPluginControl->CurrentInputLanguage() == ELangUrdu) 	|| 
		(iPluginControl->CurrentInputLanguage() == ELangVietnamese)) 
		{
		return ETrue;
		}

	return EFalse;		
}

TBool InputModeSelector::IsNoninline() 
{
	// Returns true, if either intial string values or 
	// current input language mode is Complex.
	
	if (ContainsComplexLangCharacters(mpInitialString, mLen) ||  
	    IsInComplexLanguageMode())
		
		return true;
	
	return false;
}


TBool InputModeSelector::ContainsComplexLangCharacters(FI_UTF16Char * string, long length) 
{
	FI_UTF16Char utfchar;
	TBool retval = false;

	for ( int i = 0; i<length; i++)
		{
		utfchar = string[i];
		
		// For testing purposes
		// open the if def block to pretend CJK is complex script
		
		#if 0
		// CJK for testing purposes
		if ((utfchar >= 0x04E00) && (utfchar <= 0x9FBF)) 
			{
			retval = true;
			break;
			}
		#endif
		
		// Arabic 
		//http://www.unicode.org/charts/PDF/U0600.pdf
		
		if ((utfchar >= 0x0600) && (utfchar <= 0x06FF)) 
			{
			retval = true;
			break;
			}
		
		//Arabic Supplement
		//http://www.unicode.org/charts/PDF/U0750.pdf
		
		if ((utfchar >= 0x0750) && (utfchar <= 0x77F)) 
			{
			retval = true;
			break;
			}
		
		// Arabic Presentation Forms-A
		// http://www.unicode.org/charts/PDF/UFB50.pdf 
		
		if ((utfchar >= 0xFB50) && (utfchar <= 0xFFFF)) 
			{
			retval = true;
			break;
			}
		
		// Arabic Presentation Forms-B
		// http://www.unicode.org/charts/PDF/UFE70.pdf
		
		if ((utfchar >= 0xFE70) && (utfchar <= 0xFEFF)) 
			{
			retval = true;
			break;
			}
		
		// Hebrew 
		// http://www.unicode.org/charts/PDF/U0590.pdf
		
		if ((utfchar >= 0x0590) && (utfchar <= 0x05FF)) 
			{
			retval = true;
			break;
			}
		
		// Hebrew presentation Forms
		// http://www.unicode.org/charts/PDF/UFB00.pdf
		
		if ((utfchar >= 0xFB00) && (utfchar <= 0xFB4F)) 
			{
			retval = true;
			break;
			}
		
		// Thai
		// http://www.unicode.org/charts/PDF/U0E00.pdf
		
		if ((utfchar >= 0x0E00) && (utfchar <= 0x0E7F)) 
			{
			retval = true;
			break;
			}

		// Devanagari
		// http://www.unicode.org/charts/PDF/U0900.pdf
		
		if ((utfchar >= 0x0900) && (utfchar <= 0x097F)) 
			{
			retval = true;
			break;
			}
		}
		
	return retval;
	}


////////////////////////////////////////////////////////////////////////////////
// CFlashPluginIMENonInlineControl
////////////////////////////////////////////////////////////////////////////////

CFlashPluginIMENonInlineControl* 
CFlashPluginIMENonInlineControl::NewL(MFlashEngine* aEngine,
                                      MFlashImeClientIf* aPluginControl
                                      /*,TPoint aPoint*/)
{
    CFlashPluginIMENonInlineControl* self = new (ELeave) 
                                            CFlashPluginIMENonInlineControl(aEngine,
                                                                            aPluginControl
                                                                            /*,aPoint*/);
                                                                            
    CleanupStack::PushL(self);    
    self->ConstructL();    
    CleanupStack::Pop(); // self
    
    return self;
}

CFlashPluginIMENonInlineControl::CFlashPluginIMENonInlineControl(
                                                    MFlashEngine* aEngine,
                                                    MFlashImeClientIf* aPluginControl
													/*,TPoint aPoint*/)
: iEngine(aEngine),
  iPluginControl(aPluginControl) 
{
}

void CFlashPluginIMENonInlineControl::ConstructL()
{
    if( !iPluginControl ) 
        User::Leave(KErrArgument);
    
    SetMopParent((CCoeControl*)iPluginControl->GetParentControl());
    
    ActivateL();	
    
    AMP_MTextNavigation *pNav= iEngine->GetTextNavigation();
 	long length = pNav->GetTextLength();
 	 		
 	AMP_ReturnString returnString(FI_UTF16EncodingForm);
 	FI_ReturnString* fiReturnString = returnString.Wrapper();

	pNav->GetText(0,length, fiReturnString);

  	const TUint16* text = returnString.GetTemporaryUTF16String();
    
    iPluginControl->GetInputTextL(text, length, 
                                  pNav->GetMaxChars(), 
                                  pNav->GetTextFieldFlags());
}


TBool CFlashPluginIMENonInlineControl::IsInLine() 
{
	return false;
}



////////////////////////////////////////////////////////////////////////////////
// CFlashPluginIMEControl
////////////////////////////////////////////////////////////////////////////////

CFlashPluginIMEControl::CFlashPluginIMEControl(MFlashEngine* aEngine,
                                               MFlashImeClientIf* aPluginControl
											   /*TPoint aPoint*/):
	iStateFepUid(TUid::Uid(-1))
	,iFepState(NULL)	
	,iIsCommitting(EFalse)
	,iCompositionStringBuf(NULL)
	,iCompositionStringBufSize(0)
	,iCompositionStringLen(0)
	,iCompositionStringCursorPos(0)
	,iCursorVisibility(EFalse)
	,iEngine(aEngine)
	,iPresentOnStack(EFalse)
	,iCursorPosAdd(0)	
	,iTextFormatRetriever(NULL)
	,iFepPointerEventHandler(NULL)
	,iPluginControl(aPluginControl)
	,iCurrentInputLanguage(ELangEnglish)
	,iIsPencilKeyPressed(EFalse)
	,iCharsConsumed(0)
	,iIsTouchFeatureSupported(EFalse)	
#ifdef PREDECTIVE_TEXT_SUPPORT	
	,iIsSpelling(EFalse)
    ,iMonitorConversionStringLengthForZi(EFalse)
    ,iLastConversionStringCharactersInserted(0)
	,iLetCallSecondTime(EFalse)
#endif // PREDECTIVE_TEXT_SUPPORT


{
	//iPoint.SetXY(aPoint.iX, aPoint.iY);
}

CFlashPluginIMEControl* CFlashPluginIMEControl::NewL(MFlashEngine* aEngine,
                                               		 MFlashImeClientIf* aPluginControl
                                                	 /*,TPoint aPoint*/)
{
    CFlashPluginIMEControl* self = new (ELeave) CFlashPluginIMEControl(aEngine,
                                                                       aPluginControl
                                                                       /*,aPoint*/);
                                                                       
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    
    return self;
}


void CFlashPluginIMEControl::HandlePointerEventL(const TPointerEvent& /*aPointerEvent*/)
{

	if(iIsTouchFeatureSupported) //adding feature manager to check the feature at run time.
	{
		LaunchVKbL();
	}
}

void CFlashPluginIMEControl::LaunchVKbL()
{
	if(!iIsTouchFeatureSupported)
		return;

    // launch VKB if sliding Qwerty keypad is Off.  
    if( iFepState && (!iPluginControl->GetQwertyKeyPadStatus()))
    {
        iFepState->ReportAknEdStateEventL(MAknEdStateObserver::EAknActivatePenInputRequest);
    }
    
    unsigned long inputflags = iEngine->GetTextNavigation()->GetTextFieldFlags();
    if((inputflags & FI_InputTextPassword))
    {
        //Place the cursor at the end of the text for the pwd field.
        //so that user can delete the text entered in the field.
        
        TInt contentLen=DocumentLengthForFep();
        if(contentLen)
        {
            iEngine->GetTextNavigation( )->SetSel(contentLen, contentLen);
        }
    }    
}

void CFlashPluginIMEControl::ConstructL()
{
    if( !iPluginControl ) 
        User::Leave(KErrArgument);
 
    FeatureManager::InitializeLibL(); // initialise feature manager
    iIsTouchFeatureSupported = FeatureManager::FeatureSupported(KFeatureIdPenSupport);
    
    SetMopParent((CCoeControl*)iPluginControl->GetParentControl());
    
    ActivateL();	
    iVKBActivateTimer = CPlgTimerObject::NewL(CActive::EPriorityStandard, *this);
    EnableIMEL();
}

void CFlashPluginIMEControl::EnableIMEL()
{
	  CreateWindowL ();
    if( !iPresentOnStack )
        {
        STATIC_CAST(CCoeAppUi*, iCoeEnv->AppUi())->AddToStackL(this, ECoeStackPriorityDefault + 1); 
                                                           
        iPresentOnStack = ETrue;
        }
}

CFlashPluginIMEControl::~CFlashPluginIMEControl()
{
	FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
    // Remove it from the control stack if not removed.
    if( iPresentOnStack )
        {
        STATIC_CAST(CCoeAppUi*, iCoeEnv->AppUi())->RemoveFromStack(this);                        
        }
    if(iVKBActivateTimer)
    {
        iVKBActivateTimer->Cancel();
        delete iVKBActivateTimer;
        iVKBActivateTimer = NULL;
    } 
	delete iCompositionStringBuf;
	iCompositionStringBuf = NULL;

	delete iFepState;
	iFepState = NULL;
}

TBool CFlashPluginIMEControl::IsInLine()
{
    return ETrue;
}


TKeyResponse CFlashPluginIMEControl::OfferKeyEventL( const TKeyEvent& aKeyEvent, 
                                                     TEventCode aType )
{
	// Start knocking down special cases

	// On Zi, when right key is used for committing a word, cursor goes
	// in the beginning of the text field. This happens because Zi doesn't
	// consume the EEventKey for the right key.. Although it uses it for
	// committing the word.
	// Please trace of the usage of iIsCommitting variable to find out how this works.
    
	// Text field shows garbage if we send these while editing text.
	
	switch(aKeyEvent.iScanCode)
	    {
		case EStdKeyLeftShift:  // Pencil key on device, shift key on wireless keyboard
		case EStdKeyRightShift: // Pencil key on emulator, shift key on wireless keyboard
		    {
		    iIsPencilKeyPressed = ETrue;
		    }
		    
		case EStdKeyTab:
		case EStdKeyEscape:
		case EStdKeyCapsLock:
		case EStdKeyLeftCtrl:
		case EStdKeyRightCtrl:
		    {
			return EKeyWasConsumed;
	    	}
		default:
		    break;	
	    }
	// in case of the vkb, hwr and full screen qwerty, unicode is sent
	// in as the key's icode
	//one exception to this is  in case of secrettext  mode EKeyF20  is sent to delete the previouse character
	//unsigned long inputflags = iEngine->GetTextNavigation()->GetTextFieldFlags();
	//if(!(inputflags & FI_InputTextPassword) || (inputflags & FI_InputTextPassword && aKeyEvent.iCode != EKeyF20)  )
	//If  we consume the previous only then delete the previous character
	  if(aKeyEvent.iCode == EKeyF20 && aKeyEvent.iModifiers == EModifierSpecial  && iCharsConsumed==0  )
	       {
	            return EKeyWasConsumed;
	        }   
	if(aKeyEvent.iCode != EKeyF20 )
	    {
	        if( aKeyEvent.iModifiers == EModifierSpecial && aKeyEvent.iCode>= 32)
	            {
	            if(aType == EEventKey)
	                {
	                if(iCompositionStringBuf)
	                    {
	                    delete iCompositionStringBuf;
	                    iCompositionStringBuf = NULL;
	                    }
			
	                iCompositionStringBuf = HBufC::NewL(2);
	                TPtr des = iCompositionStringBuf->Des();		
	                des.Append(TChar(aKeyEvent.iCode));	
	                des.ZeroTerminate();
	                TInt32 selStart = 0;
	                TInt32 selEnd = 0;
	                iEngine->GetTextNavigation( )->GetSel(selStart, selEnd);
	                FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, 1 };
	                iCharsConsumed=  iEngine->GetTextNavigation( )->SetConversionString(&txt, NULL, 0, selStart+1, ETrue);							
	                }	
	            ReportStateEventL();
	            return EKeyWasConsumed;	
	            }
		} 
	    
	if( iIsPencilKeyPressed )    
	    {
        TBool isLangChanged = IsLangChanged();
	    if (isLangChanged && aType == EEventKeyUp )
	        {
	        // NOTE : Since this will disable IME, this will be deleted.
	        // So this function should be returned immediately after this.
	        
	        iPluginControl->ResetIMEL();
	        
	        return EKeyWasConsumed;
	        }
	        	    
	    iIsPencilKeyPressed = EFalse;    
	    }

	if(aKeyEvent.iScanCode == EStdKeyRightArrow && aType == EEventKeyDown)
	    {
	   	iIsCommitting = EFalse;
	    }

	if(aKeyEvent.iCode == EKeyRightArrow && aType == EEventKey)
	    {
		if(iIsCommitting)
		    {
			iIsCommitting = EFalse;
			
			// Don't send the key event. 
			return EKeyWasConsumed;
		    }
	    }

#ifdef PREDECTIVE_TEXT_SUPPORT

	// Part of T9 question mark fix. When T9 switches to spell state
	// the conversion string doesn't include the question mark in the end.
	// The code here is for resetting the conversion string after a clear
	// key has been pressed. T9 resets the Cba key after it calls UpdateInlineEditL
	// so we can't detect the clear key press in there.

	if(aKeyEvent.iScanCode == EStdKeyBackspace && aType == EEventKeyUp && iIsSpelling)
	    {
		if (iCompositionStringBuf && iCompositionStringLen)
	    	{   
			TPtr des = iCompositionStringBuf->Des();
			FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, iCompositionStringLen };
			iEngine->GetTextNavigation( )->SetConversionString(&txt, 
			                                                   NULL, 
			                                                   0, 
			                                                   iCompositionStringLen - 1, 
			                                                   EFalse);
		    }

		iIsSpelling = EFalse;
	    }

#endif // PREDECTIVE_TEXT_SUPPORT
	
	TKeyResponse result = EKeyWasNotConsumed;

	// If we're in the middle of a conversion session (predictive session)
	// then we disable the enter key since the IME is not consuming it.
	// Otherwise, the key event gets sent to the player which will 
	// cause the field to be deactivated -- this causes a crash
	// on Japanese devices since we cannot properly cancel the
	// Fep transaction, although we can on non-Japanese devices.  This is
	// likely a Japanese-Symbian-specific issue.
	
	if ( iCompositionStringBuf && ( EStdKeyEnter == aKeyEvent.iScanCode || 
	                                  EKeyEnter    == aKeyEvent.iCode ) )
	    {
		result = EKeyWasConsumed;
	    }
	if(result == EKeyWasNotConsumed)
	{
	    return iPluginControl->HandleKeyEventL( aKeyEvent,  aType);
	}
	else
	{
	    return result;
	}
}


/*void CFlashPluginIMEControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
    {
    CCoeControl::HandlePointerEventL(aPointerEvent);
    
    if (iFepPointerEventHandler) 
    {
    	iFepPointerEventHandler->HandlePointerEventInInlineTextL( 
    														aPointerEvent.iType,
    						 								aPointerEvent.iModifiers,
    														1);
    }
    
    }
*/
void  CFlashPluginIMEControl::ReportStateEventL()
{
	if(iFepState)
	{
		iFepState->ReportAknEdStateEventL(MAknEdStateObserver::EAknCursorPositionChanged);
	}
}
void CFlashPluginIMEControl::CommitConversion(TBool aConversionCanceled)
{
	if(aConversionCanceled)
	    {
		CancelFepInlineEdit();
	    } 
	else
	    {
		TRAP_IGNORE(DoCommitFepInlineEditL());
	    }
}


TCoeInputCapabilities CFlashPluginIMEControl::InputCapabilities() const
{
	if ( iEngine->IsPanning( ) )
	    {
		return TCoeInputCapabilities::ENone;
	    }

	unsigned long inputflags = iEngine->GetTextNavigation()->GetTextFieldFlags();

    if (AknLayoutUtils::PenEnabled())
    	{
		if(inputflags & FI_InputTextPassword)
			{
			TCoeInputCapabilities inputCapabilities(
			TCoeInputCapabilities::ENavigation | TCoeInputCapabilities::ESecretText
			,(MCoeFepAwareTextEditor *)this
			,NULL /* (MCoeCaptionRetrieverForFep *) */
			);
			
			return inputCapabilities;		
			}
    	}

	TCoeInputCapabilities inputCapabilities(
	TCoeInputCapabilities::ENavigation | TCoeInputCapabilities::EAllText
	,(MCoeFepAwareTextEditor *)this
	,NULL /* (MCoeCaptionRetrieverForFep *) */
	);
	
	return inputCapabilities;
}

/** 
Starts a FEP inline editing transaction. 

Inserts a descriptor containing the initial inline text into the text editor. 
The inline text should normally replace any selected text.

The final three parameters are instances of abstract classes, so that the 
caller of this function must create and instantiate classes deriving from 
them. These instances must remain in existence for the entire duration of 
the inline editing transaction.

Inline editing should not already be taking place when this function is called.

@param aInitialInlineText The inline text to insert into the text editor.
@param aPositionOfInsertionPointInInlineText An insertion position within the 
inline text. This is an offset from the start of the inline text.
@param aCursorVisibility ETrue for visible text cursor, EFalse for invisible 
text cursor in the text editor.
@param aCustomDraw Pointer to a custom drawing object. May be used to do advanced 
formatting of the inline text. This parameter is optional; a NULL pointer 
may be specified.
@param aInlineTextFormatRetriever Defines a single member function, GetFormatOfFepInlineText() 
which is used by the text editor to find out the formatting to apply to the 
inline text. It is also possible to apply different formatting to different 
parts of the inline text.
@param aPointerEventHandlerDuringInlineEdit Defines a single function, HandlePointerEventInInlineTextL() 
which is called when a pointer event is received within the inline text. This 
function might update the cursor position within the inline text and do text 
selection. 
*/

void CFlashPluginIMEControl::StartFepInlineEditL(const TDesC& aInitialInlineText, 
                             TInt aPositionOfInsertionPointInInlineText, 
                             TBool aCursorVisibility, 
                             const MFormCustomDraw* /* aCustomDraw */, 
                             MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, 
                             MFepPointerEventHandlerDuringInlineEdit& 
                             aPointerEventHandlerDuringInlineEdit )
{
    NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::StartFepInlineEditL() ++"));
    
	iTextFormatRetriever = &aInlineTextFormatRetriever;
	iFepPointerEventHandler = &aPointerEventHandlerDuringInlineEdit;
	iCursorPosAdd        = 0;
	iCursorVisibility    = aCursorVisibility;
	
	SetInlineEditingCursorVisibilityL(aCursorVisibility);

	// Since we draw the text ourselves, ignore the aCustomDraw and the aInlineTextFormatRetriever parameter.

	// Since we do not handle pointers, ignore the aPointerEventHandlerDuringInlineEdit parameter
	// (there will never be any pointer events anywhere in the editor, let alone within the inline text)

	// Use the UpdateFepInlineTextL method to extract the text
	
	UpdateFepInlineTextL(aInitialInlineText, aPositionOfInsertionPointInInlineText);
	
	NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::StartFepInlineEditL() --"));
}

void CFlashPluginIMEControl::StartFepInlineEditL(TBool& /*aSetToTrue*/, 
                             const TCursorSelection& aCursorSelection, 
                             const TDesC& aInitialInlineText,
                             TInt  aPositionOfInsertionPointInInlineText, 
                             TBool aCursorVisibility, 
                             const MFormCustomDraw* aCustomDraw, 
                             MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, 
                             MFepPointerEventHandlerDuringInlineEdit& 
                             aPointerEventHandlerDuringInlineEdit)
{

#ifdef PREDECTIVE_TEXT_SUPPORT

	// On Chinese Nokia phones this function is used extensively in English multitap mode.
	// For some reason, enabling this functions for the multitap in those phones causes problems,
	// namely 1 additional cursor jump when a character is committed. To prevent that,
	// restrict the usage of this function to only to two cases: after spell dialog has
	// been closed and when input mode is changed from numeric to alpha by press-holding # key.
	//
	// Why does Chinese multitap go wrong if this function is enabled to it.

	// Switching from numeric mode to alpha mode by press-holding # key case
	
	if( aInitialInlineText.Length()   == KSingleChar && 
	    (aInitialInlineText.Ptr())[0] == KCharCode4 )
	    {
		SetCursorSelectionForFepL(aCursorSelection);
		StartFepInlineEditL(aInitialInlineText, 
		                    aPositionOfInsertionPointInInlineText, 
		                    aCursorVisibility, 
		                    aCustomDraw, 
		                    aInlineTextFormatRetriever, 
		                    aPointerEventHandlerDuringInlineEdit);
	    }

	// Canceling Spell dialog-case: two separate calls. 
	// First time iIsSpelling is true, the second time not. 
	
	if(iIsSpelling)
    	{
		SetCursorSelectionForFepL(aCursorSelection);
		StartFepInlineEditL(aInitialInlineText, 
		                    aPositionOfInsertionPointInInlineText, 
		                    aCursorVisibility, 
		                    aCustomDraw, 
		                    aInlineTextFormatRetriever, 
		                    aPointerEventHandlerDuringInlineEdit);
		                    
		iLetCallSecondTime = ETrue;
	    }
	else if(iLetCallSecondTime)
	    {
	    SetCursorSelectionForFepL(aCursorSelection);
		StartFepInlineEditL(aInitialInlineText, 
		                    aPositionOfInsertionPointInInlineText, 
		                    aCursorVisibility, 
		                    aCustomDraw, 
		                    aInlineTextFormatRetriever, 
		                    aPointerEventHandlerDuringInlineEdit);
		                    
		iLetCallSecondTime = EFalse;
	    }

#endif // PREDECTIVE_TEXT_SUPPORT
}


/** 
Updates the inline text. 

Called when a character is added to or deleted from the inline text. 

The descriptor aNewInlineText contains the entire new inline text string, 
not just the new text to be combined with the old inline text.

FEP inline editing should already be taking place, i.e. StartFepInlineEditL() 
should have been called beforehand.

@param aNewInlineText Descriptor which holds the entire new inline text string.
@param aPositionOfInsertionPointInInlineText The position of the insertion 
point (i.e. the cursor) within the inline text string aNewInlineText. This 
is an offset from the start of the string. 
*/

void CFlashPluginIMEControl::UpdateFepInlineTextL(const TDesC& aNewInlineText, 
                                                  TInt aPositionOfInsertionPointInInlineText)
{
    NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::UpdateFepInlineTextL() ++"));
    
	// Try to figure out the conversion string format.

	TBool underlineOn = ETrue;

	if(iTextFormatRetriever)
	    {
		// Zi crashes if we call GetFormatOfFepInlineText when the conversion string is of zero length.
		// T9 doesn't care.
		TInt conversionStringLength = aNewInlineText.Length();
		
		if(conversionStringLength)
		    {
			TCharFormat defaultFormat;      // out param
			TInt numOfCharsWithSameFormat;  // out param
			TInt posOfChars = 0;

			iTextFormatRetriever->GetFormatOfFepInlineText(defaultFormat, 
			                                               numOfCharsWithSameFormat, 
			                                               posOfChars);
			                                               
			underlineOn = (defaultFormat.iFontPresentation.iUnderline == EUnderlineOn);
	    	}
	    }

	// Delete any existing text
	if (iCompositionStringBuf) 
	    {		
		delete iCompositionStringBuf;
		iCompositionStringBuf = NULL;
		
		iCompositionStringBufSize = 0;
		iCompositionStringLen     = 0;
	    }
	
#ifdef PREDECTIVE_TEXT_SUPPORT


    //commenting out the if condition as it is preventing spell word from
    // getting rendered. If needed this check can be uncommented.
    /*-------------------------------------------------------
    // CancelFepTransaction may result in UpdateFepInlineEditL()
    // So cancel the updation if already done. (Specially in case of Cancel softkey in Spell state)
    if( iMonitorConversionStringLengthForZi )
        {
        iMonitorConversionStringLengthForZi = EFalse;
        return;
        }
     ---------------------------------------------------------*/
	
	iMonitorConversionStringLengthForZi = EFalse;
	
	// Check for the spell state
	// Works only for T9
	iIsSpelling = (iFepState->CbaState().iLeftSoftkeyCommandId == KT9SpellSoftkeyCommandId);

#endif // PREDECTIVE_TEXT_SUPPORT

	// Extract the text
	iCompositionStringLen = aNewInlineText.Length();
	
	if (iCompositionStringLen)
	    {

		iCompositionStringBuf = HBufC::NewL(iCompositionStringLen + 1);

		TPtr des = iCompositionStringBuf->Des();

		des.Copy(aNewInlineText);

		des.ZeroTerminate();

		iCompositionStringLen = iCompositionStringBuf->Length();

		// not used
		iCompositionStringBufSize = iCompositionStringBuf->Size();
	    }

	iCompositionStringCursorPos = aPositionOfInsertionPointInInlineText;

	// Write the new conversion string to the editor
	if (iCompositionStringBuf && iCompositionStringLen) 
	    {
		TPtr des = iCompositionStringBuf->Des();
		FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, iCompositionStringLen };

		FI_TextField_StyleRun styleRun;
		styleRun.style.mask = FI_TextField_StyleBit_Underline;
		styleRun.style.bits = (underlineOn) ? FI_TextField_StyleBit_Underline : 0;
		
		// styleRun.style.textColor is ignored by the player
		// styleRun.style.fieldColor is ignored by the player
		// The run is ignored by the player but set it anyway.
		
		styleRun.run.length = iCompositionStringLen;
		styleRun.run.start  = 0;

		long charsInserted = iEngine->GetTextNavigation( )->SetConversionString
		                                        (
                                                &txt, 
                                                &styleRun, 
                                                1, 
                                                aPositionOfInsertionPointInInlineText, 
                                                EFalse
                                                );

#ifdef PREDECTIVE_TEXT_SUPPORT

		iLastConversionStringCharactersInserted = charsInserted;

#endif // PREDECTIVE_TEXT_SUPPORT
		
		if(charsInserted < iCompositionStringLen)
		    {
			iCursorPosAdd = iCompositionStringLen - charsInserted;
		    }
		else
		    {
			iCursorPosAdd = 0;
		    }
	    } 
	else
	    {
		FI_Text empty;
		empty.textEncoding      = FI_UTF16EncodingForm;
		empty.textLen           = 0;
		empty.text.utf16String  = 0;
		
		iEngine->GetTextNavigation()->SetConversionString(&empty, NULL, 0, 0, true);
	    }
	
	NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::UpdateFepInlineTextL() --"));
}

/** 
Sets the visibility of the text cursor in the text editor.

The cursor visibility is initialised using StartFepInlineEditL(). 
SetInlineEditingCursorVisibilityL() is provided for FEPs which need to change 
the visibility of the cursor during the inline editing transaction.

@param aCursorVisibility ETrue for visible text cursor, EFalse for invisible 
text cursor. 
*/

void CFlashPluginIMEControl::SetInlineEditingCursorVisibilityL(TBool aCursorVisibility)
{
	iCursorVisibility = aCursorVisibility;
	
	AMP_MTextNavigation* textNavigation = iEngine->GetTextNavigation();
	if( !textNavigation )
	    User::Leave(KErrGeneral);
	
	textNavigation->ShowCursor(aCursorVisibility);
}

/** 
Cancels the inline editing transaction. 

The edit window should be rolled back to the state it was in before the FEP 
transaction started, i.e. any inline text in the document which has not yet 
been committed should be removed. If the inline text has replaced existing text, 
(e.g. a selection) the replaced text should be reinstated. 
*/

void CFlashPluginIMEControl::CancelFepInlineEdit()
{
    NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::CancelFepInlineEdit() ++"));
    
	iCursorPosAdd = 0;

	// When in T9 user presses 0-key three times T9 FEP calls
	// UpdateFepInlineEdit with a conversion string that contains 0x21b2.
	// After that, it calls CancelFepInlineEdit. 
	// For some reason it never calls DoCommitFepInlineEditL(). 
	// We do it manually here for this special case.
	// DoCommitFepInlineEditL will convert 0x21b2 to a new line character 0x0A.
	
	if (iCompositionStringBuf) 
	{
		if(iCompositionStringLen == KSingleChar && 
		  (iCompositionStringBuf->Ptr())[0] == KCharCode3)
		{
			TRAP_IGNORE(DoCommitFepInlineEditL());
			return;
		}
	}

	// Delete any existing text
	if (iCompositionStringBuf) 
	{
		delete iCompositionStringBuf;
		iCompositionStringBuf = NULL;
		
		iCompositionStringBufSize = 0;
		iCompositionStringLen     = 0;
	}
	
	iCompositionStringCursorPos = 0;
	
	TRAP_IGNORE(SetInlineEditingCursorVisibilityL(ETrue));
	
	// The error can be ignored here.  
	
	iTextFormatRetriever = NULL;
	
	NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::CancelFepInlineEdit() --"));
}

/** 
Returns the total number of characters in the text editor.

@return The total number of characters in the text editor. 
*/

TInt CFlashPluginIMEControl::DocumentLengthForFep() const
{
	TInt documentLength = 0;

	AMP_MTextNavigation* textNavigation = iEngine->GetTextNavigation();
	if( textNavigation )
	    {
	    documentLength = textNavigation->GetTextLength() + iCursorPosAdd;
	    }		
	    
	return documentLength;
}

/** 
Returns the upper limit (if any) on the length of text that the text editor 
can hold.

@return The maximum number of characters that the text editor can hold. 
*/

TInt CFlashPluginIMEControl::DocumentMaximumLengthForFep() const
{
	TInt maxLen = KDocMaxLength;
		
	AMP_MTextNavigation* textNavigation = iEngine->GetTextNavigation();
	if( textNavigation )
	    {
	    maxLen = textNavigation->GetMaxChars();
	    }		

	return maxLen;
}

/** 
Sets the range of characters in the text editor which should be selected.

@param aCursorSelection Contains the cursor and anchor positions for the selection.
*/

void CFlashPluginIMEControl::SetCursorSelectionForFepL(const TCursorSelection& aCursorSelection)
{
	// Set the cursor position in the editor
	long selStart = aCursorSelection.iAnchorPos;
	long selEnd   = aCursorSelection.iCursorPos;

	if(selStart > selEnd)
	    {
		long temp = selStart;
		selStart = selEnd;
		selEnd = temp;
    	}

#ifdef PREDECTIVE_TEXT_SUPPORT

	// This should happen only after a spell dialog is confirmed
	if(iMonitorConversionStringLengthForZi)
	    {
		long selLength = selEnd - selStart;

		if(selLength > iLastConversionStringCharactersInserted)
		    {
			selEnd = selStart + iLastConversionStringCharactersInserted;
		    }
	    }
#endif // PREDECTIVE_TEXT_SUPPORT
	
	AMP_MTextNavigation* textNavigation = iEngine->GetTextNavigation();
	if( !textNavigation )
	    User::Leave(KErrGeneral);
	
	textNavigation->SetSel(selStart, selEnd);
}

void CFlashPluginIMEControl::GetFormatForFep(TCharFormat& /* aFormat */, 
                                             TInt /* aDocumentPosition */) const
{
	// Ignored
}

/** 
Copies a portion of the text editor's text content into a descriptor.

@param aEditorContent A descriptor; on return contains a copy of a portion 
of the text.
@param aDocumentPosition The document position in the text editor from which 
to copy.
@param aLengthToRetrieve The number of characters to copy.
*/

void CFlashPluginIMEControl::GetEditorContentForFep(TDes& aEditorContent, 
                                                    TInt aDocumentPosition, 
                                                    TInt aLengthToRetrieve) const
{
	AMP_ReturnString returnString(FI_UTF16EncodingForm);
	FI_ReturnString* fiReturnString = returnString.Wrapper();

	FI_Result result = FI_ResultError;

	// In low memory situations FiReturnString may be null.
	
	if(fiReturnString)
	    {
	    AMP_MTextNavigation* textNavigation = iEngine->GetTextNavigation();
	    if( textNavigation )
	        {
	        result = textNavigation->GetText(aDocumentPosition, 
	                                         aDocumentPosition + aLengthToRetrieve, 
	                                         fiReturnString);
	        }
	    }

	long charsRetrieved = returnString.GetLength();

	if(charsRetrieved > aLengthToRetrieve)
	    {
		charsRetrieved = aLengthToRetrieve;
	    }

	// When using embedded font we sometimes insert less characters than in
	// conversion string because of missing glyphs. We should still return the
	// amount of characters requested or T9 crashes.

	if(result != FI_ResultOK || charsRetrieved < aLengthToRetrieve)
	    {   
		aEditorContent.Delete(0, aEditorContent.Length());
		
		if(charsRetrieved > 0)
		    {
			aEditorContent.Copy(returnString.GetTemporaryUTF16String(), charsRetrieved);
		    }

		long amountOfSpaces = aLengthToRetrieve - charsRetrieved;

		if(amountOfSpaces > 0)
		    {
			TChar ch(' ');
			aEditorContent.AppendFill(ch, amountOfSpaces);
		    }
	    } 
	else
	    {
		aEditorContent.Copy(returnString.GetTemporaryUTF16String(), aLengthToRetrieve);
    	}
	
	
	TChar ch = TText(CEditableText::EParagraphDelimiter);
	TChar lEnterKey = TText(0x000A);
	TInt lPos = aEditorContent.LocateF(lEnterKey);
	TBuf<2> lBuf;
	lBuf.Append(ch);
	if(lPos == 0)
	    {
        if(aEditorContent[0] == lEnterKey)
            {
            aEditorContent.Replace(lPos, 1, lBuf);
            }
	    }
	lPos = aEditorContent.LocateF(lEnterKey);
	while (lPos > 0)
	    {
        // Convert the newline to paragraph separator, because the FEP
        // input editors (vkb, etc) ignore newline
        aEditorContent.Replace(lPos, 1, lBuf);
        lPos = aEditorContent.LocateF(lEnterKey);
        }
}

/** 
Gets the range of characters in the text editor which are selected.

@param aCursorSelection On return, contains the cursor and anchor positions 
of the selection. 
*/

void CFlashPluginIMEControl::GetCursorSelectionForFep(TCursorSelection& aCursorSelection) const
{
	// Return the cursor position from the editor
	long selStart = 0;
	long selEnd   = 0;
	
	AMP_MTextNavigation* textNavigation = iEngine->GetTextNavigation();
	if( textNavigation )
	    {
	    textNavigation->GetSel(selStart, selEnd);
	    }
	
	aCursorSelection.SetSelection(selStart + iCursorPosAdd,
								  selEnd   + iCursorPosAdd);

}

void CFlashPluginIMEControl::GetScreenCoordinatesForFepL(TPoint& /*aLeftSideOfBaseLine*/, 
                                                         TInt&   /*aHeight*/,
                                                         TInt&   /*aAscent*/, 
                                                         TInt    /*aDocumentPosition*/) const
{
    // This is an empty implementation. 
}

/** Private function called by CommitFepInlineEditL(). 

Implementations should commit the inline text to the document. This 
ends the inline editing transaction and causes the inline text to 
become a part of the document. 

@publishedAll
@released 
*/

void CFlashPluginIMEControl::DoCommitFepInlineEditL()
{
    NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::DoCommitFepInlineEditL() ++"));
    
    iCursorPosAdd = 0;
	iIsCommitting = ETrue;

#ifdef PREDECTIVE_TEXT_SUPPORT

	// Zi specific missing embedded glyphs fix - set variable	
	
	CEikButtonGroupContainer* cba = CEikButtonGroupContainer::Current();
	if(cba)
	    {
		MEikButtonGroup* group = cba->ButtonGroup();
		if(group->CommandId(0) != EAknSoftkeyOptions)
		    {
			iMonitorConversionStringLengthForZi = ETrue;
		    }
	    }
	
#endif // PREDECTIVE_TEXT_SUPPORT

	if (iCompositionStringBuf && iCompositionStringLen) 
	    {
		TPtr des = iCompositionStringBuf->Des();
		FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, iCompositionStringLen };

#ifdef PREDECTIVE_TEXT_SUPPORT

		FI_UTF16Char lineFeed[2] = { KLineFeed, 0 };
		
		if(txt.textLen == KSingleChar)
		    {
			FI_UTF16Char ch = txt.text.utf16String[0];

			// After recent change core player recognizes Unicode line breaks.
			// We still need to change 0x21b2: the character code for "Enter" key.
			// This is a T9 specific fix. 
			
			if(ch == KCharCode1 || ch == KCharCode2 || ch == KCharCode3)
			    {
				txt.text.utf16String = lineFeed;
			    }
		    }
#endif // PREDECTIVE_TEXT_SUPPORT
		
  		long charsInserted = iEngine->GetTextNavigation( )->SetConversionString
      		                                            (
      		                                            &txt, 
      		                                            NULL, 
      		                                            0, 
      		                                            iCompositionStringCursorPos, 
      		                                            ETrue
      		                                            );

		if(charsInserted < iCompositionStringLen)
		    {
			iCursorPosAdd = iCompositionStringLen - charsInserted;
		    } 
		else
		    {
			iCursorPosAdd = 0;
		    }
	    }

	// Delete any existing text
	if (iCompositionStringBuf) 
	    {
		delete iCompositionStringBuf;
		iCompositionStringBuf = NULL;
	
		iCompositionStringBufSize = 0;
		iCompositionStringLen     = 0;
	    }
	
	iCompositionStringCursorPos = 0;

	SetInlineEditingCursorVisibilityL(ETrue);

	iTextFormatRetriever = NULL;
	
	NPPDebug(LOG_DEBUG,_L("CFlashPluginIMEControl::DoCommitFepInlineEditL() --"));
}

MCoeFepAwareTextEditor_Extension1* CFlashPluginIMEControl::Extension1()
{
	return this;
}

MCoeFepAwareTextEditor_Extension1* CFlashPluginIMEControl::Extension1(TBool& aSetToTrue)
{
	aSetToTrue = ETrue;
	return this;
}

void CFlashPluginIMEControl::SetStateTransferingOwnershipL(CState* /*aState*/, TUid /*fepTUid*/)
{
// This code is not needed for T9 since this API is not used in the correct way by device.
}

void CFlashPluginIMEControl::SetFepStateInputModesToTextFieldFlags(CAknEdwinState *state, 
                                                                    unsigned long flags)
{
	if(flags & FI_InputTextPassword)
	    {
		state->SetDefaultInputMode(EAknEditorSecretAlphaInputMode);
		state->SetCurrentInputMode(state->CurrentInputMode());
		state->SetPermittedInputModes(EAknEditorSecretAlphaInputMode | EAknEditorNumericInputMode | EAknEditorTextInputMode);
		state->SetCurrentCase(EAknEditorLowerCase);
		state->SetDefaultCase(EAknEditorLowerCase);
		state->SetPermittedCases(EAknEditorLowerCase | EAknEditorUpperCase);
		//avkon has different key map
		state->SetSpecialCharacterTableResourceId(R_AVKON_SPECIAL_CHARACTER_TABLE_DIALOG);
    	state->SetNumericKeymap(EAknEditorAlphanumericNumberModeKeymap);
    	state->SetFlags(EAknEditorFlagNoT9 | EAknEditorFlagNoLRNavigation | EAknEditorFlagSelectionVisible );
	    }
	else if(flags & FI_InputTextNumeric)
	    {
		state->SetDefaultInputMode(EAknEditorNumericInputMode);
		state->SetCurrentInputMode(EAknEditorNumericInputMode);
		state->SetPermittedInputModes(EAknEditorNumericInputMode);
		state->SetNumericKeymap(EAknEditorAlphanumericNumberModeKeymap);
		state->SetFlags(EAknEditorFlagNoT9 | EAknEditorFlagNoLRNavigation | EAknEditorFlagSelectionVisible);
	    }
	else
    	{ //text input mode,numbers and special chars allowed
		state->SetPermittedInputModes(EAknEditorAllInputModes);
	//	state->SetDefaultInputMode(EAknEditorTextInputMode);//text mode
	//	state->SetCurrentInputMode(EAknEditorTextInputMode);//text mode
		state->SetPermittedCases(EAknEditorAllCaseModes);//allow everything

		// In Ivalo we have to set the flag EAknEditorFlagSelectionVisible explicitily
		// in order to select text in VKB
		state->SetFlags(EAknEditorFlagSelectionVisible | EAknEditorFlagEnableScrollBars | EAknEditorFlagAllowEntersWithScrollDown );

	//	state->SetCurrentCase(EAknEditorLowerCase);//start with lower alpha
	//	state->SetDefaultCase(EAknEditorLowerCase);//default is lower alpha
		//avkon has different key map
	//	state->SetSpecialCharacterTableResourceId(R_AVKON_SPECIAL_CHARACTER_TABLE_DIALOG);
		//avkon has diff key numeric map
    //	state->SetNumericKeymap(EAknEditorPlainNumberModeKeymap);
    
	    }	
}

CFlashPluginIMEControl::CState* CFlashPluginIMEControl::State(TUid fepUid)
{
    // In documentation the State object should have been passed to us in
    // SetStateTransferOwnershipL. It should be perfectly OK to return null from
    // this function. However, T9 (at least) crashes if null is returned.
    // So create a CState object here.
	
	if (!iFepState) 
	    {	
            TRAP_IGNORE(CreateStateL());
    
            if( !iFepState ) 
                return NULL;
            
            iFepState->SetObjectProvider(this);
            iStateFepUid = fepUid;
// launch VKB only if PEN support is available
		    if(iIsTouchFeatureSupported)
		    {
           if(!iVKBActivateTimer->IsActive())
   				iVKBActivateTimer->After(1);
			}
	    }

	SetFepStateInputModesToTextFieldFlags
	(iFepState, iEngine->GetTextNavigation()->GetTextFieldFlags());

	return (iStateFepUid == fepUid) ? iFepState : NULL;
}

void CFlashPluginIMEControl::CreateStateL()
{
    iFepState = new (ELeave) CAknEdwinState();
    
    CEikButtonGroupContainer* cba = CEikButtonGroupContainer::Current();
    if(cba)
    {
	    cba->MakeCommandVisibleByPosition
	    (
	    CEikButtonGroupContainer::EMiddleSoftkeyPosition, 
	    EFalse
	    );    
    }
}

void CFlashPluginIMEControl::DoRunL(const CActive* /*aActive*/)
{
    if(iIsTouchFeatureSupported)
    {
    LaunchVKbL();
    }
}

TInt CFlashPluginIMEControl::DoRunError(const CActive* /*aActive*/, TInt aError)
{
    return aError;
}

void CFlashPluginIMEControl::MCoeFepAwareTextEditor_Reserved_2()
{
    // This is an empty implementation
}


void CFlashPluginIMEControl::SetCursorType(TBool& /*aSetToTrue*/, 
                                           const TTextCursor& /*aTextCursor*/) //Allows you to change cursor type
{
    // This is an empty implementation
}

void CFlashPluginIMEControl::MCoeFepAwareTextEditor_Extension1_Reserved_3()
{
	// This is an empty implementation
}

void CFlashPluginIMEControl::MCoeFepAwareTextEditor_Extension1_Reserved_4()
{
	// This is an empty implementation
}

void CFlashPluginIMEControl::DisableIME()
{
    CommitConversion(EFalse);
	
	CCoeFep* fep = iCoeEnv->Fep();
	
	if ( fep )
    	{
		// Conversion string is already committed above (wanted behavior).
		// Hopefully this does some good in resetting FEP variables.
		
		fep->CancelTransaction();
    	}
    	
   	// FEP may have changed the soft keys at this point so reset them
	// T9 and Zi FEPs leave soft keys un-reseted even if we called CCoeFep->CancelTransaction().
	// This should help.
	
	RemoveCba(CEikButtonGroupContainer::ELeftSoftkeyPosition);
	RemoveCba(CEikButtonGroupContainer::ERightSoftkeyPosition);
	RemoveCba(CEikButtonGroupContainer::EMiddleSoftkeyPosition);
	
	if( iPresentOnStack )
        {
        STATIC_CAST(CCoeAppUi*, iCoeEnv->AppUi())->RemoveFromStack(this);    
        
        iPresentOnStack = EFalse;
        }
}

void CFlashPluginIMEControl::RemoveCba(TUint aButtonPos)
{
	CEikButtonGroupContainer* cba = CEikButtonGroupContainer::Current();
	
	if( !cba ) return;
    
    if (!iFepState) return;
    
    CAknEdwinState::SEditorCbaState& cbaState = iFepState->CbaState();
    
    switch(aButtonPos)
		{
		case CEikButtonGroupContainer::ELeftSoftkeyPosition:
		if (!cbaState.iLeftSoftkeyCommandId) return;
		cba->RemoveCommandFromStack(aButtonPos, cba->ButtonGroup()->CommandId(aButtonPos));
		cba->RemoveCommandObserver(aButtonPos);
		cbaState.iLeftSoftkeyCommandId=0;    
		break;

		case CEikButtonGroupContainer::ERightSoftkeyPosition:
		if (!cbaState.iRightSoftkeyCommandId) return;
		cba->RemoveCommandFromStack(aButtonPos, cba->ButtonGroup()->CommandId(aButtonPos));
		cba->RemoveCommandObserver(aButtonPos);
		cbaState.iRightSoftkeyCommandId=0;    
		break;

		case CEikButtonGroupContainer::EMiddleSoftkeyPosition:
		if (!iFepState->MiddleSoftkeyCommandId()) return;
		cba->RemoveCommandFromStack(aButtonPos, cba->ButtonGroup()->CommandId(aButtonPos));
		cba->RemoveCommandObserver(aButtonPos);
		iFepState->SetMiddleSoftkeyCommandId(0);    
		break;
        }
}

void CFlashPluginIMEControl::FocusChanged(TDrawNow /*aDrawNow*/)
{
    if( !iFepState )
        return;
    
    TCursorSelection currentInlineEditSpan; 
    GetCursorSelectionForFep(currentInlineEditSpan);
    iFepState->SetInlineEditSpan(currentInlineEditSpan);
}

TBool CFlashPluginIMEControl::IsLangChanged()
{
    CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
    if (CurrentInputLanguage() != cache.InputLanguage())
	    {
	    SetCurrentInputLanguage(cache.InputLanguage());
	    return ETrue;	
	    }
    
    return EFalse;
}

#endif // FEATURE_INLINE_TEXT_ENTRY
