/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#ifndef CFLASHLONGTAPDETECTORCONTROLLER_H
#define CFLASHLONGTAPDETECTORCONTROLLER_H

#include <w32std.h>
#include <e32base.h>
#include <coemain.h>

/**
 * Flash LongTapDetector CallBack.
 *
 * Inherit from this class and implement HandleLongTapEventL to
 * get a callback when/if a long tap occurs.
 *
 * @since S60 5.0
 */
class MFlashLongTapDetectorCallBack
    {
public:
    /**
    * Callback method. Get's called when a long tap occurs.
    * @param aPenEventLocation Long tap event location relative to parent control.
    * @param aPenEventScreenLocation Long tap event location relative to screen.
    */
    virtual void HandleFlashLongTapEventL(const TPoint& aPenEventLocation, 
                                      const TPoint& aPenEventScreenLocation) = 0;
    };


/**
 * Flash Long Tap Detector. 
 *
 * Receives pointer events from owner application or owner control and returns
 * location if a long tap event occurs.
 * After short period of time (default 0,15s) animation is displayed next to stylus on the screen to
 * inform the user of long tap functionality. Animation can be turned OFF by application if wanted.
 *
 * @since S60 5.0
 */

NONSHARABLE_CLASS(CFlashLongTapDetector): public CTimer
    {
public: // Constructors and destructor  

    /**
     * Two-phased constructor.
     * @param aOwner Pointer to owner of this component. Ownership not transferred.
     */
    static CFlashLongTapDetector* NewL(MFlashLongTapDetectorCallBack* aOwner);

    /**
     * Two-phased constructor.
     * @param aOwner Pointer to owner of this component. Ownership not transferred.
     */
    static CFlashLongTapDetector* NewLC(MFlashLongTapDetectorCallBack* aOwner);

    /**
    * Destructor.
    */
    virtual ~CFlashLongTapDetector();

public: // New functions

    /**
    * Receives and handles pointer events from owner's HandlePointerEventL() -method.
    * Used to control timers in detecting long tap events.
    * @param aEvent Pointer event from owner.
    */
    void PointerEventL( const TPointerEvent& aEvent );



    /**
    * Sets new time delay before long tap detector event is returned. Default delay is 0,8s.
    * @param aMicroSeconds New delay in microseconds.
    */
    void SetLongTapDelay( const TInt aMicroSeconds );    


        
private: // From CActive

    /**
    * Handles CTimer events. Called by system framework.
    */
    void RunL();

private: // Constructors

    /**
    * C++ default constructor.
    * @param aOwner Owner.
    */
    CFlashLongTapDetector(MFlashLongTapDetectorCallBack* aOwner);

    /**
    * By default Symbian 2nd phase constructor is private.
    */
    void ConstructL();



private: // Data

    enum TFlashLongTapDetectorState
        {
        EWaiting,               // normal state
        ELongTapEvent
        };

    /**
    * Current state of Flash LongTapDetector.
    */
    TFlashLongTapDetectorState iState;
    
    /**
    * Pointer to owner of this Flash LongTapDetector object.
    * Not own.
    */
    MFlashLongTapDetectorCallBack* iOwner;


    /**
    * Delay before long tap event is reported in microseconds.
    */
    TInt iFlashLongTapDelay; 

    /**
    * Pointer event received from owner.
    */
    TPointerEvent iPointerEvent;
    
    /*
    * Flag which determines whether touch feature is supported or not.
    */
    TBool iTouchFeatureSupported;
    
    };

#endif // CFLASHLONGTAPDETECTORCONTROLLER_H


// End of File
