/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  H/w key watch observer
*
*/


#include <eikapp.h>
#include <aknconsts.h>
#include "saplayer_build.h"
#include "hwkeywatch.h" 
#include "volumepropertywatchobserver.h"
#include "hwkeywatchobserver.h"


// CONSTANTS
const TInt KFirstTimerExpiryInterval = 1; // Expire immediately
const TInt KTimerExpiryInterval = KAknStandardKeyboardRepeatRate;
// Used to set the array granularity for dynamic arrays
const TUint KHwKeyObserversArrayGranularity = 2;

// ----------------------------------------------------
// CHwKeyPropertyWatch::NewL
// Description: static constructor.
// Input:      aMinVolume : minimum volume value
//             aMaxVolume : maximum volume value
// Output: none
// Return: CHwKeyPropertyWatch object
// ----------------------------------------------------
CHwKeyPropertyWatch* CHwKeyPropertyWatch::NewL()
{
    CHwKeyPropertyWatch* me=new(ELeave) CHwKeyPropertyWatch();
    CleanupStack::PushL(me);
    me->ConstructL();
    CleanupStack::Pop(me);
    return me;
}

// ----------------------------------------------------
// CHwKeyPropertyWatch::CHwKeyPropertyWatch
// Description: constructor.
// Output: none
// Return: none
// ----------------------------------------------------
CHwKeyPropertyWatch::CHwKeyPropertyWatch():
								 iInterfaceSelector(NULL)
                                 , iSelector(NULL)
                                 , iTimer(NULL)
                                 , iObservers(NULL)
{

}

// ----------------------------------------------------
// CHwKeyPropertyWatch::ConstructL
// Description: second phase constructor.
// Input:  aObserver: link to the calling object
// Output: none
// Return: none
// ----------------------------------------------------
void CHwKeyPropertyWatch::ConstructL()
{
    //
    // Open a connection to receive Volume Key events.
    iSelector = CRemConInterfaceSelector::NewL();
    iInterfaceSelector = CRemConCoreApiTarget::NewL(*iSelector,*this);
    iSelector->OpenTargetL();
    // Timer for implementing repeat
    iTimer = CPeriodic::NewL(CActive::EPriorityStandard);

    iObservers=new(ELeave)CArrayFixSeg<MHwKeyWatchObserver*>
                (KHwKeyObserversArrayGranularity);
}

// ----------------------------------------------------
// CHwKeyPropertyWatch::~CHwKeyPropertyWatch
// Description: destructor.
// Input:  none
// Output: none
// ----------------------------------------------------
CHwKeyPropertyWatch::~CHwKeyPropertyWatch()
    {
    if(iSelector)
        {
        delete iSelector;
        iInterfaceSelector=NULL;
        iSelector=NULL; //iSelector has been deleted by "delete iInterfaceSelector"
        }
    
    if(iTimer)
        {
        iTimer->Cancel();
        delete iTimer;
        }
    
    if (iObservers)
        {
        iObservers->Reset();
        delete iObservers;
        }
    }


// ----------------------------------------------------
// CHwKeyPropertyWatch::FindObserver
// Description: search an observer in the list of
// observers
// Input:  aObserver: Point to observer
// Output: Index in the observer list
// Return: none
// ----------------------------------------------------

TInt CHwKeyPropertyWatch::FindObserver(
                MHwKeyWatchObserver* aObserver)
{
	TInt index = -1;
    TInt maxIndex = iObservers->Count();
    //Search if this StreamId is already there in the array
    for ( index=0; index < maxIndex; index++)
        {
    	MHwKeyWatchObserver* objObsrvr = iObservers->At(index); 
        	
        	if(objObsrvr == aObserver)
        		return index;
        }
       
    return -1;
}
// ----------------------------------------------------
// CHwKeyPropertyWatch::RemoveObserver
// Description: Used to remove an observer to the list of
// observers
// Input:  aObserver: Point to observer
// Output: none
// Return: none
// ----------------------------------------------------
void CHwKeyPropertyWatch::RemoveObserver(
                MHwKeyWatchObserver* aObserver)
    {
    
    TInt idx = FindObserver(aObserver);
    
    if(idx != -1)
    {
    	iObservers->Delete(idx);	
    }
    
    }



// ----------------------------------------------------
// CHwKeyPropertyWatch::AddObserverL
// Description: Used to add an observer to the list of
// observers
// Input:  aObserver: Point to observer
// Output: none
// Return: none
// ----------------------------------------------------
void CHwKeyPropertyWatch::AddObserverL(
                MHwKeyWatchObserver* aObserver)
    {
    iObservers->AppendL(aObserver);
    }


// ----------------------------------------------------
// CHwKeyPropertyWatch::HandleVolumeUp
// Description: A Volume up key command handler
//        (from MRemConCoreApiTargetObserver)
// Input:  aButtonAct The button action associated with the command.
// Output: none
// Return: none
// ----------------------------------------------------
void CHwKeyPropertyWatch::HandleVolumeUp(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	         //Start Timer
	         if (!iTimer->IsActive())
	            {
	            iCommandId = EVolumeControlCmdVolumeUpByOne;
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	        break;
	        }

	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }

	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EVolumeControlCmdVolumeUpByOne);
	        break;
	        }

	    default:
	        {
	        break;
	        }
    }
}


// ----------------------------------------------------
// CHwKeyPropertyWatch::HandleVolumeDown
// Description: A Volume down key command handler
//        (from MRemConCoreApiTargetObserver)
// Input:  aButtonAct The button action associated with the command.
// Output: none
// Return: none
// ----------------------------------------------------
void CHwKeyPropertyWatch::HandleVolumeDown(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EVolumeControlCmdVolumeDownByOne;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EVolumeControlCmdVolumeDownByOne);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }	
    
    
}


void CHwKeyPropertyWatch::HandlePlayPause(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EHwControlCmdPausePlay;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EHwControlCmdPausePlay);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }
}
		
void CHwKeyPropertyWatch::HandleStop(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EHwControlCmdStop;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EHwControlCmdStop);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }
}
	
void CHwKeyPropertyWatch::HandleRewind(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EHwControlCmdRewind;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EHwControlCmdRewind);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }
}
	
void CHwKeyPropertyWatch::HandleFastForward(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EHwControlCmdFastForward;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EHwControlCmdFastForward);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }
}
	
void CHwKeyPropertyWatch::HandleBackward(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EHwControlCmdBackward;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EHwControlCmdBackward);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }
}


void CHwKeyPropertyWatch::HandleForward(TRemConCoreApiButtonAction aButtonAct )
{
	switch (aButtonAct)
    {
	    case ERemConCoreApiButtonPress:
	        {
	        //Start Timer
	        iCommandId = EHwControlCmdForward;
	        if (!iTimer->IsActive())
	            {
	            iTimer->Start(KFirstTimerExpiryInterval,
	                          KTimerExpiryInterval, 
	                          TCallBack(TimerCallback, this));
	            }
	         break;
	        }
	    case ERemConCoreApiButtonRelease:
	        {
	        iTimer->Cancel();
	        break;
	        }
	    case ERemConCoreApiButtonClick:
	        {
	        FilterAndSendCommand(EHwControlCmdForward);
	        break;
	        }
	    default:
	        {
	        break;
	        }
    }	
}
// ----------------------------------------------------
// CHwKeyPropertyWatch::MrccatoCommand
// Description: A Volume key command has been received
//        (from MRemConCoreApiTargetObserver)
// Input:aOperationId The operation ID of the command
//         aButtonAct The button action associated with the command.
// Output: none
// Return: none
// ----------------------------------------------------
void CHwKeyPropertyWatch::MrccatoCommand(TRemConCoreApiOperationId aOperationId, 
                             TRemConCoreApiButtonAction aButtonAct )
    {

	    switch(aOperationId)
	        {
	        	case ERemConCoreApiPausePlayFunction:
	        	{
	        		HandlePlayPause(aButtonAct);
	        	}
	        	break;
	        	
	        	case ERemConCoreApiStop:
				{
	        		HandleStop(aButtonAct);
	        	}
	        	break;
	        	
	        	case ERemConCoreApiRewind:
	        	{
	    			HandleRewind(aButtonAct);    		
	        	}
				break;
				
	        	case ERemConCoreApiFastForward:
	        	{
	    			HandleFastForward(aButtonAct);    		
	        	}
				break;

				case ERemConCoreApiBackward:
				{
					HandleBackward(aButtonAct);
				}
				break;
				
		        case ERemConCoreApiVolumeUp:
	            {
	            	HandleVolumeUp(aButtonAct);
	            }
	            break;
		            
		        case ERemConCoreApiVolumeDown:
	            {
	            	HandleVolumeDown(aButtonAct);
	            }
	            break;
	            
		        default:
		            {
		            
		            }
		            break;
	        }        
    }        
        
// -----------------------------------------------------------------------
// CHwKeyPropertyWatch::HandleRepeatEvent
// -----------------------------------------------------------------------
//
void CHwKeyPropertyWatch::HandleRepeatEvent()
    {
    	FilterAndSendCommand(iCommandId);
    }

// -----------------------------------------------------------------------
// CHwKeyPropertyWatch::TimerCallback
// -----------------------------------------------------------------------
//
TInt CHwKeyPropertyWatch::TimerCallback(TAny* aPtr)
    {
	    static_cast<CHwKeyPropertyWatch*>(aPtr)->HandleRepeatEvent();
	    return KErrNone;
    }

// -----------------------------------------------------------------------
// CHwKeyPropertyWatch::FilterAndSendCommand
// -----------------------------------------------------------------------
//
void CHwKeyPropertyWatch::FilterAndSendCommand(TInt aCommandId)
    {
    	TInt index=0;
	    switch(aCommandId)
	        {
	        case EVolumeControlCmdVolumeUpByOne:
	        case EVolumeControlCmdVolumeDownByOne:
	    		{
	    			for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			TRAP_IGNORE(iObservers->At(index)->MHandleVolumeCmdL(aCommandId));     
	        		}   
	    		}
	            break; 
	            
	        case EHwControlCmdPausePlay:
	        /*{
	        	for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			iObservers->At(index)->MPausePlayKeyPressed();     
	        		} 	
	        }*/
	        break;
	        
	        case EHwControlCmdStop:
	        /*{
	        	for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			iObservers->At(index)->MStopKeyPressed();     
	        		} 
	        }*/
	        break;
	        case EHwControlCmdRewind:
	        /*{
	        	for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			iObservers->At(index)->MRewindKeyPressed();     
	        		} 
	        }*/
	        break;
	        case EHwControlCmdForward:
	        /*{
	        	for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			iObservers->At(index)->MForwardKeyPressed();     
	        		} 
	        }*/
	        break;
	        case EHwControlCmdFastForward:
	        /*{
	        	for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			iObservers->At(index)->MFastForwardKeyPressed();     
	        		} 
	        }*/
	        break;
	        case EHwControlCmdBackward:
	        /*{
	        	for (index =0 ; index < iObservers->Count(); index++)
	        		{
	        			iObservers->At(index)->MBackwardKeyPressed();     
	        		} 
	        }*/
	        break;
	        
	        default:
	            break;      
	        } 
	    
    
    }

// End of File

