/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash policy server's client-side session.
 *
*/

 
#include <e32debug.h>
#include <utf.h>

#include "flashrtpolicyclientsession.h"
#include "flashrtpolicycommon.h"

//The server version. A version must be specifyed when creating a session with 
// the server
const TUint KPolicyServMajorVersionNumber=0;
const TUint KPolicyServMinorVersionNumber=1;
const TUint KPolicyServBuildVersionNumber=1;

const TInt  KPolicyServerClientTryCount = 2;
const TUint KDefaultMessageSlots        = 16;

_LIT(KPolicyServerExeName,"!FlashRTPolicyServer.exe");
_LIT(KPolicyServerProcessName, "FlashRTPolicyServer");

const TInt  KPolicyServerUid2Int(0x200159D6);
const TUid  KPolicyServerUid2 = {KPolicyServerUid2Int};

// ---------------------------------------------------------------------------
// Establishes an IPC connection with the policy server. Starts the policy 
// server if it is not running yet
//
// ---------------------------------------------------------------------------
TInt RFlashRTClientSession::Connect()
    {
    TInt lRetry(KPolicyServerClientTryCount); // Try this twice
    TInt lErr(KErrNone);
    while(lRetry > KErrNone)
        {
        // Try to create a Server session
        lErr = CreateSession(KPolicyServerProcessName, Version(),
                KDefaultMessageSlots);

        if(lErr != KErrNotFound && lErr != KErrServerTerminated)
            {
            // KErrNone or unrecoverable lError
            lRetry = 0;
            }
        else
            {
            // Return code was KErrNotFound or KErrServerTerminated.
            // Try to start a new security manager server instance
            lErr = StartPolicyServer();
            if(lErr != KErrNone && lErr != KErrAlreadyExists)
                {
                // Unrecoverable lError
                lRetry = 0;
                }
            }

        lRetry--;
        }
    return(lErr);
    }

// ---------------------------------------------------------------------------
// Closes the IPC connection with the flash policy server
//
// ---------------------------------------------------------------------------
void RFlashRTClientSession::Close()
    {
    RSessionBase::Close();
    }

// ---------------------------------------------------------------------------
// Gives version of the flash policy server 
//
// ---------------------------------------------------------------------------
TVersion RFlashRTClientSession::Version() const
    {
    return(TVersion(KPolicyServMajorVersionNumber,
                    KPolicyServMinorVersionNumber,
                    KPolicyServBuildVersionNumber));
    }

    
// ---------------------------------------------------------------------------
// Gets policy id from flash runtime policy server
//
// ---------------------------------------------------------------------------
TPolicyID RFlashRTClientSession::GetPolicyId() const
    {
    TPolicyID lPolicyId;
    TPckg<TPolicyID> data(lPolicyId);
    TIpcArgs args(&data);
    TInt llErr = SendReceive(EGetPolicyId,args);
    
    if(llErr)
       return llErr;
              
    return lPolicyId;
    }

// ---------------------------------------------------------------------------
// Starts the policy server if it is not running yet
//
// ---------------------------------------------------------------------------
TInt RFlashRTClientSession::ReadFont(unsigned char** aFont, TBool aBold)
    {
    TInt fileHandle;
    TPckg<TInt> fileHandlePkg(fileHandle);
    TIpcArgs args(&fileHandlePkg, aBold);
    TInt rfsHandle = SendReceive(EGetFontFileHandle, args);
    
    // Got handle from server. Open the file now
    RFile fontFile;    
    TInt error = fontFile.AdoptFromServer(rfsHandle, fileHandle);    
    if( KErrNone == error )
        {
        // Got the shared handle from server.Allocate buffer and get entire file
        TInt fileSize;
        fontFile.Size(fileSize);
        unsigned char *buffer = new unsigned char[fileSize];        
        
        // Read the file into the buffer
        TPtr8 destination(buffer, fileSize);
        error = fontFile.Read(destination, fileSize);
        
        if( KErrNone == error )
            {
            (*aFont) = buffer;
            error = destination.Length();
            }
        else
            {
            (*aFont) = NULL;
            }
        }

    fontFile.Close();
    return error;
    }

// ---------------------------------------------------------------------------
// Starts the policy server if it is not running yet
//
// ---------------------------------------------------------------------------
TInt RFlashRTClientSession::StartPolicyServer() const
    {
    RProcess server;
    const TUidType serverUid(KNullUid, KPolicyServerUid2, KNullUid);
    TInt lErr = server.Create(ServerLocation(), 
            KNullDesC /*, 
            serverUid, 
            EOwnerProcess*/);

    // Return lError code if we the process couldn't be created
    if(KErrNone == lErr)
        {
        // Rendezvous is used to detect server start
        TRequestStatus status;
        server.Rendezvous(status);
        if(status != KRequestPending)
            {
            // Server start failed.
            server.Kill(0); // Abort startup
            }
        else
            {
            server.Resume(); // Logon OK - start the server
            }
        User::WaitForRequest(status); // Wait for start or death

        if(server.ExitType()== EExitPanic)
            {
            // Server start failed.
            lErr = KErrGeneral;
            }
        else
            {
            lErr = status.Int();
            }

        // We can close the handle now
        server.Close();
        }
        
    return lErr;
    }
    
// ---------------------------------------------------------------------------
// Gives the location of the policy server with full path and name.
//
// ---------------------------------------------------------------------------
TFullName RFlashRTClientSession::ServerLocation() const
    {
    TFullName fullPathAndName;
    fullPathAndName.Append(KPolicyServerExeName);
    return fullPathAndName;
    }

TInt RFlashRTClientSession::CreateSharedMemory(const char* aShMemName, const char* aLockName, unsigned long aSize,
                RChunk& aChunkHdl, unsigned long& aLockID)
    {
    TPtrC8 shMemNamePtr((const TUint8*) aShMemName);
    TBuf16<KMaxFileName> uShMemName;
    CnvUtfConverter::ConvertToUnicodeFromUtf8(uShMemName, shMemNamePtr);

    TPtrC8 lockPtr((const TUint8*) aLockName);
    TBuf16<KMaxFileName> uLockName;
    CnvUtfConverter::ConvertToUnicodeFromUtf8(uLockName, lockPtr);

    TPckgBuf<TInt> lockHdlBuf(0); //this is an in-out parameter...

    TIpcArgs args(&uShMemName, &uLockName, aSize,&lockHdlBuf);

    TInt chunkHdl = SendReceive(EGetSharedMemory, args);

    if(chunkHdl >= 0) //positive return value means there is no error..
        {
       
        aChunkHdl.SetHandle(chunkHdl);
        aLockID = (TInt)lockHdlBuf();
        }

    return KErrNone;
    }

TInt RFlashRTClientSession::LockSharedMemory(unsigned long aLockId)
    {
    TIpcArgs args((TInt)aLockId);
    return SendReceive(ELockSharedMemory, args);
    }

TInt RFlashRTClientSession::UnlockSharedMemory(unsigned long aLockId)
    {
    TIpcArgs args((TInt)aLockId);
    return SendReceive(EUnLockSharedMemory, args);
    }

TInt RFlashRTClientSession::DeleteSharedMemory(unsigned long aLockId)
    {
    TIpcArgs args((TInt)aLockId);
    return SendReceive(EDeleteSharedMemory, args);
    }

//EOF
