/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash runtime policy server. 
 *                Own and defines access policy file for flash runtime.
 *                Provides service to fetch policy id generated from access 
 *                policy file
 *
*/


#ifndef __FLASHRTPOLICYSERVER_H__
#define __FLASHRTPOLICYSERVER_H__

#include <e32base.h>

class CServSharedMemManager;
/*
 * Total number of ranges 
 */
const TUint KFlashRTPolicySrvPolicyRangeCount = 1;

// Definition of the ranges of IPC numbers
const TInt KFlashRTPolicySrvRanges[KFlashRTPolicySrvPolicyRangeCount] =
    {
    0
    };

// Policy to implement for each of the above ranges
const TUint8 KFlashRTPolicySrvElementsIndex[KFlashRTPolicySrvPolicyRangeCount] =
    {
    0
    };

// Specific capability checks
const CPolicyServer::TPolicyElement KFlashRTPolicySrvElements[] =
    {
    { _INIT_SECURITY_POLICY_C1( ECapability_None ),  CPolicyServer::EFailClient },  // Policy "0"
    };

// Package all the above together into a policy
const CPolicyServer::TPolicy KFlashRTPolicySrvPolicy =
    {
    CPolicyServer::EAlwaysPass,     // All connect attempts to Policy Manager Server pass
    KFlashRTPolicySrvPolicyRangeCount,    // Number of ranges
    KFlashRTPolicySrvRanges,              // Ranges array
    KFlashRTPolicySrvElementsIndex,       // Elements <-> ranges index
    KFlashRTPolicySrvElements,            // Array of elements
    };


class CFlashRTPolicyContentSession;
class CFlashRTPolicyStore;    
class CFlashRTPolicyUtil;

const TInt KShutdownDelay = 30000000; // approx 30 seconds

/**
 * Shut down timer. Used for a timout to close the server after 
 * the last session exits
 * 
 * @return pointer to an instance of CFlashRTPolicyClient
 */ 
class CShutdownTimer : public CTimer
    {
public:
    CShutdownTimer();
    void ConstructL();
    void Start();
private:
    void RunL();
    };


/**
 * Flash runtime policy server - A symbian CPolicyServer
 * 
 */ 
class CFlashRTPolicyServer : public CPolicyServer
    {
public:
	/**
	 * Two-phased constructor
	 * 
	 * Constructs a CFlashRTPolicyServer instance leave it on cleanup stack
	 *
	 * @return CPolicyServer* Pointer to CPolicyServer instance. 
	 */ 
    static CPolicyServer* NewLC();
    
	/**
	 * Session book keepig api. Called by the session while creation. Helps the 
	 * server keep track of the session count.
	 * 
	 */ 
    void AddSession();
    
	/**
	 * Session book keepig api. Called by the session while being destroyed. Helps the 
	 * server keep track of the session count.
	 * 
	 */ 
    void RemoveSession();
    
	/**
	 * Gives file session. A single file session is used in the whole of the process.
	 * Sessions query for the same as and when they need.
	 * 
	 */     
    RFs& FileSession();
    
    ~CFlashRTPolicyServer();
    
private:
    //Private constructor
    CFlashRTPolicyServer();
    
    //Second phase construction
    void ConstructL();
    
    // From CPolicyServer
    virtual CSession2* NewSessionL(const TVersion& aVersion,const RMessage2& aMessage) const; 

private:
    //File-session handle. Used to open access policy files and file store database file.
    RFs iFileSession;
    
    //Number of active session at any point of time.
    TInt iSessionCount;
    
    //Timer to close the session after inactivity
    CShutdownTimer iShutdownTimer;

    CServSharedMemManager* iShMemMgr;
    };


/**
 * Flash runtime policy server session
 * 
 */ 
class CFlashRTPolicyServerSession : public CSession2
    {
public:
	/**
	 * Constructor.
	 *
	 */
    CFlashRTPolicyServerSession(CServSharedMemManager* aShMemMgr);

	/**
	 * Second-phase constructor.
	 * 
	 */    
    void CreateL();

public:
	/**
	 * From CSession2. Point of entry for IPC message service requests
	 * 
	 */
    virtual void ServiceL(const RMessage2& aMessage); 
    
	/**
	 * From CSession2. Handle error when ServiceL leaves
	 * 
	 */    
    virtual void ServiceError(const RMessage2& aMessage, TInt aError); 

private:
	/**
	 * Gets policy id for the runtime. Returns stored policy id if it
	 * was fetched earlier. Checks for installations and uninstallations
	 * and manages updation of policy id or unsetting policy id accordingly.
	 * 
	 */
    TPolicyID GetPolicyIdL();
    
	/**
	 * Gets the best candidate file for use.
	 * 
	 */    
    void GetFullFileNameL(TFileName& aName, TDesC& aFileName);

private:
    // Destructor
    ~CFlashRTPolicyServerSession();

private:
    // Policy store which manages policy ids and integrity checks
    CFlashRTPolicyStore* iPolicyStore;
    
    // Util to perform security manager related operations    
    CFlashRTPolicyUtil*  iPolicyUtil;

    CServSharedMemManager* iShMemMgr;
    };




#include "flashrtpolicyserver.inl"


#endif //__FLASHRTPOLICYSERVER_H__

