/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Policy id manager for policy server. It stores policy id and
 *                other meta information in persistant store and checks for 
 *                helps determine access policy integrity, updates and 
 *                uninstallations etc
 *
*/



#ifndef __FLASHRTPOLICYSTORE_H__
#define __FLASHRTPOLICYSTORE_H__


#include <e32base.h>
#include <s32std.h>
#include <rtsecmanager.h>

const TInt KStoreFmtVerMajor = 0;  //current version is 0.1
const TInt KStoreFmtVerMinor = 1; 

/*
 * Policy id manager for policy server. It stores policy id and other meta 
 * information in persistant store and checks for helps determine access policy 
 * integrity, updates and uninstallations etc
 *
 */
class CFlashRTPolicyStore : public CBase
    {
public:
    /*
     * Represents status of a policy id in conjunction with its policy file
     *
     */
    struct TPolicyStatus 
        {
        inline TPolicyStatus();
                
        TBool     iPolicyIdExists;
        TBool     iPolicyFileExists;
        TBool     iDirty;
        TPolicyID iPolicyId;
        TFileName iPolicyFileFullName;
        TInt      iError;
        };

public:
	/**
	 * Two-phased constructor
	 * 
	 * Constructs a CFlashRTPolicyStore instance
	 * 
	 * @return CFlashRTPolicyStore* Pointer to an instance of CFlashRTPolicyStore
	 */
    static CFlashRTPolicyStore* NewL(RFs& aFs);
    
	/**
	 * Destructor
	 * 
	 */    
    ~CFlashRTPolicyStore();
    
    /*
     * Gets the status of a policy id in conjunction with its policy file
     *
	 * @param aFileName TFileName& File name of the access policy file
	 * @param aPolicyStatus TPolicyStatus& status of a policy id in conjunction 
	 *                      with its policy file
	 *
     * @see TPolicyStatus
     */
    void GetPolicyStatus(TFileName& aFileName, TPolicyStatus& aPolicyStatus) const;
    
    /*
     * Adds a policy id to the store along with the access policy file name. The 
     * API does not check for duplicates - that is policy ids registered with
     * same file name.
     *
	 * @param aFileName TFileName& File name of the access policy file
	 * @param aPolicyStatus TPolicyStatus status of a policy id in conjunction 
	 *                      with its policy file
	 *
     * @leave  One of the system errors on failure.
     */  
    void SetPolicyIdL(TFileName& aFileName, TPolicyID aPolicyId);
    
    /*
     * Removes policy id from the policy store. If policy id does not exist in 
     * it is silently ignored.
     *
	 * @param aPolicyId TPolicyID Policy Id to be removed
	 *
     */      
    void RemovePolicyId(TPolicyID aPolicyId);
    
    /*
     * Commits the current policy store data to persistant storage.
     *
     * @leave  One of the system errors on failure.
     */ 
    void CommitL();
    
private:
    /*
     * Private constructor.
     *
	 * @param aFs RFs& File session to be used by the store.     
     */ 
    inline CFlashRTPolicyStore(RFs& aFs);
    
    /*
     * Second phase constructor
     *
     */ 
    void ConstructL();

    /*
     * Utility function. Gets an MD5 (16 byte length) hash out of the 
     * specified file. 
     * Leaves the hash digest on the cleanup stack
     *
	 * @param aFileName TFileName& File name with full path.
	 *
	 * @leave One of the system wide error codes.
	 *
	 * @return HBufC8* MD5 (16 byte length) hash of the 
     * specified file.
     */     
    HBufC8* GetHashLC(TFileName& aFileName) const;
    
    /*
     * Utility function. Searches for an policy store entry with the
     * specified policy id and returns the index of the entry if found. 
     *
	 * @param aPolicyID TPolicyID& Valid policy id
	 *
	 * @return TInt Index of the store item where policy id exists.
     */ 
    TInt FindByPolicyID(TPolicyID& aPolicyID) const; 
    
    /*
     * Utility function. Searches for an policy store entry with the
     * specified policy id and returns the index of the entry if found. 
     *
	 * @param aFileName TFileName& Policy file name (without full path)
	 *
	 * @return TInt Index of the store item where policy id exists.
     */
    TInt FindByPolicyFileName(TFileName& aFileName) const; 
    
private:
    /*
     * Policy store array entry item. Stores policy id plus additional 
     * info like policy file path and hash.
     *
     * Can serialize and deserialize helping to put data onto persistant
     * store.
     *
     */
    class CPolicyEntry : public CBase
        {
    public:
    	/**
    	 * Two-phased constructor. Constructs a CPolicyEntry instance
    	 * 
    	 * @return CPolicyEntry* Pointer to an instance of CPolicyEntry
    	 */    
        static CPolicyEntry* NewL(RStoreReadStream& aSource);
        static CPolicyEntry* NewL(TPolicyID aPolicyID, TFileName& aFileName, TDes8& aHash);

    	/**
    	 * Deserialize method. Reads deserialized onto internal data members
    	 * 
    	 */    
        void InternalizeL(RStoreReadStream& aSource);
        
    	/**
    	 * Serialize method. Writes serialized data onto write stream
    	 * 
    	 */            
        void ExternalizeL(RStoreWriteStream& aDest);    
    
        /**
    	 * Destructor. 
    	 * 
    	 */  
        ~CPolicyEntry();
        
    private:
        //Private constructor
        inline CPolicyEntry();
        
        //Second-phase constructor.
        void ConstructL(TPolicyID aPolicyID, TFileName& aFileName, TDes8& aHash);
    
    public:
        //Policy id
        TPolicyID iPolicyID;    
        
        //Full name of the policy file from which policy id is generated
        TFileName iFileName;    
        
        //MD5 hash of the access policy file data
        HBufC8*   iHash;    
        
        //Length of the policy file full name.        
        TInt      iFileNameLength;
        };    
        
private:
    RFs& iFs;
    
    //Policy store database file name (with full path)
    TFileName iPolicyStoreFullFileName;
    
    //List of policy id store entries.    
    RPointerArray<CPolicyEntry> iPolicies;
    };


#include "flashrtpolicystore.inl"

#endif //__FLASHRTPOLICYSTORE_H__

