/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
// flashrtserver.cpp

#include <e32base.h>
#include <f32file.h>
#include <bautils.h>
#include <e32lang.h>
#include <rtsecmanager.h>
#include <featmgr.h>

#include "flashrtpolicystore.h"
#include "flashrtpolicyserver.h"
#include "flashrtpolicyutil.h"
#include "flashrtpolicycommon.h"

#include "sharedmemory.h"

_LIT(KServerName,"FlashRTPolicyServer");
_LIT(KThreadName,"FlashRTPolicyServerMainThread");
_LIT(KPolicyFileName,"AccessPolicy.xml");

// OTF Language fonts

/* Western regular font file name */
//Fix for Bug 14155, This file name has been changed from Week 15 onwards
_LIT(KWesternRegularFont, "nosnr60.ttf"); //_LIT(KWesternRegularFont, "s60snr.ttf");
//Fix for Bug 14155, This file name has been changed from Week 15 onwards
/*Western bold font file name*/
_LIT(KWesternBoldFont, "nssb60.ttf"); //_LIT(KWesternBoldFont, "s60ssb.ttf");
/*Traditional varient of chinese*/
_LIT(KTraditionalChinese,"S60TCHK_C.ttf");
/*Simplified Chinese fonts*/
_LIT(KSimplifiedChinese,"S60SC_C.ttf");
/*Korean language font file*/
_LIT(KKorean,"Series60Korean.ttf");
/*Japanese language font file*/
_LIT(KJapanese,"HeiseiGothS60.ttf");

// Resource path
_LIT(KFontsPath,"\\resource\\fonts\\");

//constant for descriptor array granularity
const TInt KDescArrayInit = 6;

// ---------------------------------------------------------------------------
// Utility function to panic the client in case of fatal events
//
// ---------------------------------------------------------------------------
void PanicClient(const RMessage2& aMessage, TInt aPanic)
    {
    aMessage.Panic(KServerName,aPanic);
    }

// ---------------------------------------------------------------------------
// Two-phase constructor
//
// ---------------------------------------------------------------------------
CPolicyServer* CFlashRTPolicyServer::NewLC()
    {
    CFlashRTPolicyServer* self=new(ELeave) CFlashRTPolicyServer;
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
    }

// ---------------------------------------------------------------------------
// Second phase constructor
//
// Starts the server and constructs the shutdown object
// ---------------------------------------------------------------------------
void CFlashRTPolicyServer::ConstructL()
    {
    User::LeaveIfError(iFileSession.Connect());
    iFileSession.ShareProtected();
    iShMemMgr = CServSharedMemManager::NewL();
    StartL(KServerName);

    iShutdownTimer.ConstructL();
    iShutdownTimer.Start();    
    }

CFlashRTPolicyServer::~CFlashRTPolicyServer()
	{
	if(iShMemMgr)
		{
		delete iShMemMgr;
		iShMemMgr = NULL;
		}
	}

// ---------------------------------------------------------------------------
// CServer2 API for creation of a new session creation request from a client.
//
// Creates a new policy server session
// ---------------------------------------------------------------------------
CSession2* CFlashRTPolicyServer::NewSessionL(const TVersion&,
                                             const RMessage2&) const
    {
    return new(ELeave) CFlashRTPolicyServerSession(iShMemMgr);
    }

// ---------------------------------------------------------------------------
// Increments session count. Cancel outstanding server shutdown timer.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyServer::AddSession()
    {
    ++iSessionCount;
    iShutdownTimer.Cancel();  // Cancel any outstanding shutdown timer    
    }

// ---------------------------------------------------------------------------
// Decrements session count. Starts shutdown timer.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyServer::RemoveSession()
    {
    if(--iSessionCount==0)
        iShutdownTimer.Start();    
    }

// ---------------------------------------------------------------------------
// Fetches file session from policy server. Session use this API to 
// get file session.
//
// ---------------------------------------------------------------------------
RFs& CFlashRTPolicyServer::FileSession()
    {
    return iFileSession;
    }
    

// ---------------------------------------------------------------------------
// Called by the CServer framework
//
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyServerSession::CreateL()
    {
    CFlashRTPolicyServer* server = static_cast<CFlashRTPolicyServer*>(const_cast<CServer2*>(Server()));
    User::LeaveIfNull(server);

    iPolicyStore = CFlashRTPolicyStore::NewL(server->FileSession());        
    iPolicyUtil  = CFlashRTPolicyUtil::GetInstanceL();
    
    server->AddSession(); //sessions book keeping in server
    }

// ---------------------------------------------------------------------------
// Session destructor
//
// Held resources and objects are released.
// ---------------------------------------------------------------------------
CFlashRTPolicyServerSession::~CFlashRTPolicyServerSession()
    {
    CFlashRTPolicyServer* server = static_cast<CFlashRTPolicyServer*>(const_cast<CServer2*>(Server()));

    if(iPolicyUtil)
        {
        delete iPolicyUtil;
        iPolicyUtil = NULL;
        }
            
    if(iPolicyStore)
        {
        TRAP_IGNORE(iPolicyStore->CommitL());
    
        delete iPolicyStore;
        iPolicyStore = NULL;
        }
    
    if(server)    
    	server->RemoveSession();        
    }

// ---------------------------------------------------------------------------
// Handle a client request
// Leaves are handled by CFlashRTPolicyServerSession::ServiceError()
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyServerSession::ServiceL(const RMessage2& aMessage)
    {
    switch(aMessage.Function())
        {
        case(EGetPolicyId):
            {
            TPolicyID lPolicyId;
            TRAPD(lError,lPolicyId = GetPolicyIdL());
            
            if(!lError)
                {
                TPckg<TPolicyID> policyIDDes(lPolicyId);          
                aMessage.Write(0,policyIDDes);
                }
            else
                {
                RDebug::Printf("GetPolicyIdL error = %d",lError);
                }
            aMessage.Complete(lError);        
            break;
            }
            /*
             * From the shared memory manager, find out if the named shared
             * memory exists. To perform this step, you must unpack the data
             * sent from the client; The unpacked data will have 1. shared memory
             * name, 2. lock name, 3. size of the shared memory segment to be
             * allocated
             *
             */
        case EGetSharedMemory:
        {
			TInt index = 0;
            //get the length of the argument 0 i.e shared memory name
            TInt shMemLen(aMessage.GetDesLength(index));

            //create a buffer which can hold the shared memory name
            HBufC* shMemBuffer = HBufC::NewL(shMemLen);

            //copy the shared memory name passed from the client into the
            //local buffer
			TPtr shmemptr = shMemBuffer->Des();
            aMessage.ReadL(index,shmemptr);

            //get the length of the argument 1 i.e lock name
            index = 1;
            TInt lockNameLen(aMessage.GetDesLength(index));
            HBufC* lockBuffer = HBufC::NewL(lockNameLen);
            TPtr lockptr = lockBuffer->Des();
            aMessage.ReadL(index,lockptr);

            //get the size of the shared memory segment to be created
            TInt shMemSegmentSize(aMessage.Int2()); //size in bytes

            if(!iShMemMgr)
                {
                iShMemMgr = CServSharedMemManager::NewL();
                }

            //This could be a pointer to an existing shared memory segment
            //or a pointer to a new named shared memory segment
            CSharedMemory* shMem = iShMemMgr->GetSharedMemoryL(shMemBuffer->Des(),lockBuffer->Des(),shMemSegmentSize);

            if(shMem)
                {
                //send back the lock handle to the client
                TPckgBuf<TUint> pkgLockHdl(shMem->LockId());
                aMessage.Write(3, pkgLockHdl);

                //send back the chunk
                aMessage.Complete(shMem->Chunk());
                }
            else
                {
                //return error to the client..
                aMessage.Complete(KErrGeneral);
                }
			}

            break;
        case ELockSharedMemory:
        {
            if(!iShMemMgr)
                return; //if shMemMgr is not created at all, it means that
                        //the player is trying to call lock when there was no shared memory
                        //is created

            //get the lock handle from the message
            TInt lockId = aMessage.Int0();

            //fetch the shared memory instance based on the lockID
            CSharedMemory* shMem = iShMemMgr->GetSharedMemory(lockId);

            TInt err(KErrGeneral);

            if(shMem)
                {
                	shMem->Lock();
                	err = KErrNone;
                    TPckgBuf<TInt> pkgLockCode(err);
                    aMessage.Write(0,pkgLockCode);
                    aMessage.Complete(err);
                }
            else
                {
                //return error to the client
                aMessage.Complete(err);
                }

		}
            break;

        case EUnLockSharedMemory:
        {
            if(!iShMemMgr)
                return; //if shMemMgr is not created at all, it means that
                        //the player is trying to call unlock when there was no shared memory
                        //is created

            //get the lock handle from the message
            TInt lockId = aMessage.Int0();

            //fetch the shared memory instance based on the lockID
            CSharedMemory* shMem = iShMemMgr->GetSharedMemory(lockId);

            TInt err(KErrGeneral);

            if(shMem)
                {
                	shMem->Unlock();
                	err = KErrNone;
                    TPckgBuf<TInt> pkgUnlockCode(err);
                    aMessage.Write(0,pkgUnlockCode);
                    aMessage.Complete(err);
                }
            else
                {
                //return error to the client
                aMessage.Complete(err);
                }
		}
            break;


        case EDeleteSharedMemory:
        {
            if(!iShMemMgr)
                return; //if shMemMgr is not created at all, it means that
                        //the player is trying to call delete when there was no shared memory
                        //is created

            //get the lock handle from the message
            TInt lockId = aMessage.Int0();

            TInt err(iShMemMgr->RemoveSharedMemory(lockId));

            TPckgBuf<TInt> pkgDeleteShMemCode(err);
            aMessage.Write(0,pkgDeleteShMemCode);
            aMessage.Complete(err);
		}
            break;

        case EGetFontFileHandle:
            {
            // Open session to file server
            RFs fsSession;
            User::LeaveIfError(fsSession.Connect());
            CleanupClosePushL(fsSession);
            
            // Make the file handle shared
            User::LeaveIfError(fsSession.ShareProtected());
            
            // Find the right font file to open            
            TFileName fileName;
            fileName = KFontsPath;
            
            // Initialize FeatureManager
            FeatureManager::InitializeLibL();

            // Check the current language with feature manager
            TBool fJapanese = FeatureManager::FeatureSupported(KFeatureIdJapanese);
            TBool fTCHKFonts = FeatureManager::FeatureSupported(KFeatureIdChineseTaiwanHkFonts);
            TBool fSCFonts = FeatureManager::FeatureSupported(KFeatureIdChinesePrcFonts);
            TBool fHiResFonts = FeatureManager::FeatureSupported(KFeatureIdChineseFontsHires);
            TBool fKorean = FeatureManager::FeatureSupported(KFeatureIdKorean);
            FeatureManager::UnInitializeLib();
            
            if( fJapanese )
                {
                // Japanese font
                fileName = KJapanese;
                }
            else if(fTCHKFonts)
                {
                // Traditional chinese fonts
                fileName = KTraditionalChinese;
                }
            else if( fSCFonts )
                {
                // Simplified chinese fonts
                fileName = KSimplifiedChinese;
                }
            else if( fKorean )
                {
                // Korean fonts
                fileName = KKorean;
                }
            else
                {
                // Then it is western fonts
                TBool bold = aMessage.Int1(); // find out if bold font is required
                if( bold )
                    {
                    fileName = KWesternBoldFont;                        
                    }
                else
                    {
                    fileName = KWesternRegularFont;                        
                    }
                }                
            
            // Get the complete file name of the font file
            TFileName fontFileName;
            
            CDesCArray* lDiskList = new(ELeave) CDesCArrayFlat(KDescArrayInit);
            CleanupStack::PushL(lDiskList);
            
            TInt error = KErrNotFound;
            BaflUtils::GetDiskListL(fsSession, *lDiskList);
            for(TInt lIndex(0); lIndex < lDiskList->Count(); ++lIndex)
                {
                fontFileName.Zero();
                TInt lDriveLetterInInt;
                TChar lDriveLetterInChar =((*lDiskList)[lIndex])[0];
                RFs::CharToDrive(lDriveLetterInChar, lDriveLetterInInt);
                TDriveUnit lCurrDrive(lDriveLetterInInt);

                fontFileName.Append( lCurrDrive.Name() );
                
                // Append the directory name
                fontFileName.Append(KFontsPath);
                
                // Append the file name
                fontFileName.Append(fileName);
                
                if(BaflUtils::FileExists(fsSession, fontFileName))
                    {
                    error = KErrNone;
                    break;
                    }
                }
            CleanupStack::PopAndDestroy(lDiskList);
            
            if(error != KErrNone)
                {
                CleanupStack::PopAndDestroy(); // fsSession 
                aMessage.Complete(error);
                return;
                }

            // Open the font file
            RFile fontFile;            
            User::LeaveIfError(fontFile.Open(fsSession, fontFileName, EFileShareReadersOnly));
            CleanupClosePushL( fontFile );
            
            // Transfer file handle to client process
            error = fontFile.TransferToClient(aMessage, 0);            
            if(error != KErrNone)
                {
                aMessage.Complete(error);
                }
            
            CleanupStack::PopAndDestroy(2); // fsSession and fontFile
            break;   
            }
        default:
            PanicClient(aMessage,KErrNotSupported);

        break;
        }
    }

// ---------------------------------------------------------------------------
// Handles leaves from CFlashRTPolicyServerSession::ServiceL()
// A bad descriptor error implies a badly programmed client, so panic it
// Report other errors to the client by completing the outstanding request with the error
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyServerSession::ServiceError(const RMessage2& aMessage, TInt aError)
    {
    if(KErrBadDescriptor==aError)
        PanicClient(aMessage,KErrBadDescriptor);
    else
        aMessage.Complete(aError);
    }

// ---------------------------------------------------------------------------
// Gets policy id for the runtime. Returns stored policy id if it
// was fetched earlier. Checks for installations and uninstallations
// and manages updation of policy id or unsetting policy id accordingly.
// 
// ---------------------------------------------------------------------------
TPolicyID CFlashRTPolicyServerSession::GetPolicyIdL()
    {
    TPolicyID lPolicyId;
    RFile       lPolicyFile;
    TFileName lCurrPolicyFileName = KPolicyFileName();
    TBool       lNewerPolicyFileExists = EFalse;
    TInt      lError = KErrNone;
    
    RFs& lFs = static_cast<CFlashRTPolicyServer*>(const_cast<CServer2*>(Server()))->FileSession();

    // Get status of policy id.
    CFlashRTPolicyStore::TPolicyStatus lPolicyStatus;
    iPolicyStore->GetPolicyStatus(lCurrPolicyFileName,lPolicyStatus);
    User::LeaveIfError(lPolicyStatus.iError);

    //TDesC policyFileName = KPolicyFileName;
    TFileName policyFileName = KPolicyFileName();
    TRAP(lError, GetFullFileNameL(lCurrPolicyFileName, policyFileName));
    if((lPolicyStatus.iPolicyIdExists == EFalse) && lError)
        {
          //lError - if no policy file exists.
        //if policy id exists, we have to unset policy 
        //before throwing error.
        User::Leave(lError);
        }
    
    if(lPolicyStatus.iPolicyIdExists && 
       lPolicyStatus.iPolicyFileFullName != lCurrPolicyFileName)
        {
        lNewerPolicyFileExists = ETrue;
        }
            
    if(lPolicyStatus.iPolicyIdExists == EFalse ||   //policy id does not exist. 
       lPolicyStatus.iPolicyFileExists == EFalse || //policy id exists, but corresponding file does not exist
       lNewerPolicyFileExists)                      //newer policy file exists
        {
        if(lPolicyStatus.iPolicyIdExists)
            {
            iPolicyUtil->UnsetPolicy(lPolicyStatus.iPolicyId);
            iPolicyStore->RemovePolicyId(lPolicyStatus.iPolicyId);
            }

        //Register new policy id from security manager, and update in store.
        User::LeaveIfError(lPolicyFile.Open(lFs, lCurrPolicyFileName, EFileRead | EFileShareAny));
        CleanupClosePushL(lPolicyFile);
        
        lPolicyId = iPolicyUtil->SetPolicy(lPolicyFile);
        User::LeaveIfError(lPolicyId);
        
        iPolicyStore->SetPolicyIdL(lCurrPolicyFileName,lPolicyId);
        
        CleanupStack::PopAndDestroy(); //lPolicyFile
        }
    else
        {
        lPolicyId = lPolicyStatus.iPolicyId;
        
        if(lPolicyStatus.iDirty) // policy file has been updated.
            {
            TPolicyID lOldPolicyId;
            lOldPolicyId = lPolicyId;

            iPolicyStore->RemovePolicyId(lOldPolicyId);
            
            User::LeaveIfError(lPolicyFile.Open(lFs, lPolicyStatus.iPolicyFileFullName, EFileRead | EFileShareAny)); 
            CleanupClosePushL(lPolicyFile);
            
            lPolicyId = iPolicyUtil->UpdatePolicy(lPolicyFile,lOldPolicyId);
            if(lPolicyId < 0)
                User::Leave(lPolicyId);
            else if(lOldPolicyId != lPolicyId || lPolicyId == 0)
                User::Leave(KErrGeneral);
           
            iPolicyStore->SetPolicyIdL(lPolicyStatus.iPolicyFileFullName,lPolicyId);
            
            CleanupStack::PopAndDestroy(); //lPolicyFile
            }
        }

    return lPolicyId;
    }

// ---------------------------------------------------------------------------
// Gets the best candidate file for use.
// 
// ---------------------------------------------------------------------------
void CFlashRTPolicyServerSession::GetFullFileNameL(TFileName& aFullFileName, TDesC& aFileName)
    {
    TInt lError = KErrNotFound;
    RFs& lFs = static_cast<CFlashRTPolicyServer*>(const_cast<CServer2*>(Server()))->FileSession();    
    
    aFullFileName.SetLength(0);
    
    CDesCArray* lDiskList = new(ELeave) CDesCArrayFlat(KDescArrayInit);
    CleanupStack::PushL(lDiskList);
    
    BaflUtils::GetDiskListL(lFs, *lDiskList);
    for(TInt lIndex(0); lIndex < lDiskList->Count(); ++lIndex)
        {
        TInt lDriveLetterInInt;
        TChar lDriveLetterInChar =((*lDiskList)[lIndex])[0];
        RFs::CharToDrive(lDriveLetterInChar, lDriveLetterInInt);
        TDriveUnit lCurrDrive(lDriveLetterInInt);

        TFileName lFullFileName(lCurrDrive.Name());
        TFileName lPrivatePath;
        lFs.PrivatePath(lPrivatePath);
        lFullFileName.Append(lPrivatePath);
        lFullFileName.Append(aFileName);
        
        if(BaflUtils::FileExists(lFs, lFullFileName))
            {
            aFullFileName = lFullFileName;
            lError = KErrNone;
            break;
            }
        }

    CleanupStack::PopAndDestroy(lDiskList);    
    User::LeaveIfError(lError);
    }    
    

// ---------------------------------------------------------------------------
// Shutdown timer implementation. When inactivity timer expires, that is in
// RunL, server gets shutdown
// 
// ---------------------------------------------------------------------------
CShutdownTimer::CShutdownTimer()
    :CTimer(EPriorityStandard)
    {
    CActiveScheduler::Add(this);
    }
    
void CShutdownTimer::ConstructL()
    {
    CTimer::ConstructL();
    }
    
void CShutdownTimer::Start()
    {
    After(KShutdownDelay);
    }
    
// Initiates server exit when the timer expires
void CShutdownTimer::RunL()
    {
    CActiveScheduler::Stop();
    }
    
    
// ---------------------------------------------------------------------------
// Server startup function
// 
// ---------------------------------------------------------------------------
static void RunServerL()
    {
    // Naming the server process and thread after the startup helps to debug panics
    // No error checking as names are not critical for operation
    User::RenameProcess(KServerName);
    User::RenameThread(KThreadName);

    // Set process priority
    RProcess svrProcess;
    svrProcess.SetPriority(EPriorityHigh);
    svrProcess.Close();

    // Create and install the active scheduler we need
    CActiveScheduler* scheduler = new (ELeave) CActiveScheduler;
    CleanupStack::PushL(scheduler);

    CActiveScheduler::Install(scheduler);
    CFlashRTPolicyServer* server = NULL;
    server = (CFlashRTPolicyServer*)CFlashRTPolicyServer::NewLC();

    // Initialisation complete, now signal the client
    RProcess::Rendezvous(KErrNone);
    CActiveScheduler::Start();
    
    //Cleanup
    CleanupStack::PopAndDestroy(server);
    CleanupStack::PopAndDestroy(scheduler);
    }

// Server process entry-point
TInt E32Main()
    {
    __UHEAP_MARK;

    // Start server.
    CTrapCleanup* cleanup = CTrapCleanup::New();
    TInt err(KErrNoMemory);

    if(cleanup)
        {
        TRAP(err, RunServerL());

        delete cleanup;
        cleanup = NULL;
        }

    __UHEAP_MARKEND;

    return err;
    }

//EOF
