/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
// flashrtpolicystore.cpp

#include <e32base.h>
#include <f32file.h>
#include <s32file.h>
#include <bautils.h>
#include <hash.h>

#include "flashrtpolicystore.h"

_LIT(KPolicyStoreFileName,"FlashRTPolicyStore.dat");


// ---------------------------------------------------------------------------
// Two phase construction
//
// ---------------------------------------------------------------------------
CFlashRTPolicyStore* CFlashRTPolicyStore::NewL(RFs& aFs)
    {
    CFlashRTPolicyStore* self=new(ELeave) CFlashRTPolicyStore(aFs);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    return self;
    }

// ---------------------------------------------------------------------------
// Two phase constructor
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::ConstructL()
    {
    iPolicies.Reset();

    iPolicyStoreFullFileName.SetLength(0);
    iFs.PrivatePath(iPolicyStoreFullFileName);
    iPolicyStoreFullFileName.Append(KPolicyStoreFileName);
    
 	//trying to get the drive # of C:    
    TDriveNumber lSystemDriveNumber;
    if(BaflUtils::GetSystemDrive(lSystemDriveNumber) == KErrNone)
    {    
	    TChar lSystemDriveLetter;
	    if(RFs::DriveToChar(lSystemDriveNumber,lSystemDriveLetter) == KErrNone)
	    {
	    	TBuf16<2> lTempBuf(2);
	    	lTempBuf[0] = TUint(lSystemDriveLetter);
	    	lTempBuf[1] = TUint(TChar(':'));	    	
	    	iPolicyStoreFullFileName.Insert(0,lTempBuf); 	
	    }    
    }
    
    CFileStore* store = NULL;
    if(BaflUtils::FileExists(iFs,iPolicyStoreFullFileName))
        {
        store = CPermanentFileStore::OpenLC(iFs,
                iPolicyStoreFullFileName, EFileShareAny);

        RStoreReadStream instream;
        instream.OpenLC(*store, store->Root());

		//Read Meta
	    TInt lMeta = instream.ReadInt32L();    

		//Read policy entry count
        TInt lCount = instream.ReadInt32L();

		//Read policy entries
        for(TInt lIndex=0;lIndex<lCount;lIndex++)
        {
            CPolicyEntry* policy = CPolicyEntry::NewL(instream);
            iPolicies.Append(policy);
        }

        CleanupStack::PopAndDestroy(&instream); //instream
        CleanupStack::PopAndDestroy(store); //store
        }
        else
        {
        //store = CPermanentFileStore::ReplaceLC(iFs,
        //        iPolicyStoreFullFileName, EFileShareAny);
        }
    }

// ---------------------------------------------------------------------------
// Gets policy id status w.r.t. the policy file with which it was generated.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::GetPolicyStatus(TFileName& aFileName,
                                          CFlashRTPolicyStore::TPolicyStatus& aStatus) const
    {
    aStatus.iError = KErrNone;
    aStatus.iPolicyIdExists = EFalse;
    aStatus.iPolicyFileExists = EFalse;
    aStatus.iDirty = EFalse;
    aStatus.iPolicyFileFullName = _L("");

    TInt lIndex = FindByPolicyFileName(aFileName);
    if(lIndex>=0)
        {
        aStatus.iPolicyIdExists = ETrue;
        aStatus.iPolicyId = iPolicies[lIndex]->iPolicyID;

        if(BaflUtils::FileExists(iFs,iPolicies[lIndex]->iFileName))
            {
            aStatus.iPolicyFileExists = ETrue;
            aStatus.iPolicyFileFullName = iPolicies[lIndex]->iFileName;

            //check if dirty
            HBufC8* lHash = NULL;

            TRAPD(lErr,
                lHash = GetHashLC(iPolicies[lIndex]->iFileName);
                if(lHash->Des().Compare(iPolicies[lIndex]->iHash->Des()) != 0)
                    {
                    aStatus.iDirty = ETrue;
                    }
                CleanupStack::PopAndDestroy();//lHash
                );

            if(lErr)
                {
                aStatus.iError = lErr;
                return;
                }
            }
        }
    }

// ---------------------------------------------------------------------------
// Adds a policy id to the store along with the access policy file name. The
// API does not check for duplicates - that is, policy ids registered with
// same file name.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::SetPolicyIdL(TFileName& aFileName, TPolicyID aPolicyId)
    {
    HBufC8* lHash = GetHashLC(aFileName);
    TPtr8 lHashData = lHash->Des();

    CPolicyEntry* lPolicy = CPolicyEntry::NewL(aPolicyId,aFileName,lHashData);
    iPolicies.Append(lPolicy);

    CleanupStack::PopAndDestroy(); //lHash
    }

// ---------------------------------------------------------------------------
// Removes policy id from the policy store. If policy id does not exist in
// it is silently ignored.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::RemovePolicyId(TPolicyID aPolicyId)
    {
    TInt lIndex = FindByPolicyID(aPolicyId);
    if(lIndex>=0)
        {
        delete iPolicies[lIndex];
        iPolicies.Remove(lIndex);
        }
    }

// ---------------------------------------------------------------------------
// Commits the current policy store data to persistant storage.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::CommitL()
    {
    BaflUtils::EnsurePathExistsL(iFs,iPolicyStoreFullFileName);
    
    CFileStore* store =
        CPermanentFileStore::ReplaceLC(iFs, iPolicyStoreFullFileName, EFileWrite);
    store->SetTypeL(KPermanentFileStoreLayoutUid);

    // Construct the output stream.
    RStoreWriteStream outstream;
    TStreamId id = outstream.CreateLC(*store);

	//Write meta.
    TInt lMeta = 0;
    lMeta =   KStoreFmtVerMajor << 24 |
              KStoreFmtVerMinor << 16 |
              0x0 << 8  | //reserved1 = 0
              0x0;        //reserved2 = 0
    outstream.WriteInt32L(lMeta);

	//Write the number of policy entries stored.
    TInt lCount = iPolicies.Count();
    outstream.WriteInt32L(lCount);

	//Write policy entries.
    for(TInt lIndex=0; lIndex < lCount; lIndex++)
        {
        iPolicies[lIndex]->ExternalizeL(outstream);
        }

    // Commit changes to the stream
    outstream.CommitL();

    CleanupStack::PopAndDestroy(&outstream);

    // Set this stream id as the root
    store->SetRootL(id);

    // Commit changes to the store
    store->CommitL();

    CleanupStack::PopAndDestroy(store);
    }

// ---------------------------------------------------------------------------
// Utility function. Gets an MD5 (16 byte length) hash out of the
// specified file.
//
// ---------------------------------------------------------------------------
HBufC8* CFlashRTPolicyStore::GetHashLC(TFileName& aFileName) const
    {
    CMD5* lHasher = CMD5::NewL();
    CleanupStack::PushL(lHasher);

    RFile lFile;
    User::LeaveIfError(lFile.Open(iFs,aFileName,EFileRead | EFileShareAny));
    CleanupClosePushL(lFile);

    TInt lFileSize;
    lFile.Size(lFileSize);
    HBufC8* lFileData = HBufC8::NewLC(lFileSize);

    TPtr8 lFileDataPtr = lFileData->Des();
    User::LeaveIfError(lFile.Read(lFileDataPtr));

    if(lFileDataPtr.Length() != lFileSize)
        User::Leave(KErrGeneral);

    TPtrC8 lHashPtr = lHasher->Hash(lFileDataPtr);
    HBufC8* lHash = lHashPtr.AllocL();

    CleanupStack::PopAndDestroy(3); //lHasher, lFile, lFileData,
    CleanupStack::PushL(lHash);
    return lHash;
    }

// ---------------------------------------------------------------------------
// Utility function. Searches for an policy store entry with the
// specified policy id and returns the index of the entry if found.
//
// ---------------------------------------------------------------------------
TInt CFlashRTPolicyStore::FindByPolicyID(TPolicyID& aPolicyID) const
    {
    TInt lCount = iPolicies.Count();
    for(TInt lIndex=0; lIndex < lCount; lIndex++)
        {
        if (iPolicies[lIndex]->iPolicyID == aPolicyID)
            {
            return lIndex;
            }
        }

    return KErrNotFound;
    }

// ---------------------------------------------------------------------------
// Utility function. Searches for an policy store entry with the
// specified policy id and returns the index of the entry if found.
//
// ---------------------------------------------------------------------------
TInt CFlashRTPolicyStore::FindByPolicyFileName(TFileName& aFileName) const
    {
    TParse lFileNameParser;
    TFileName lTempFileName;
    TFileName lSrcFileName;

    lSrcFileName = aFileName;
    lSrcFileName.LowerCase();

    TInt lCount = iPolicies.Count();
    for (TInt lIndex=0; lIndex < lCount; lIndex++)
        {
        lFileNameParser.Set(iPolicies[lIndex]->iFileName,NULL,NULL);

        lTempFileName = lFileNameParser.NameAndExt();
        lTempFileName.LowerCase();

        if(lSrcFileName.Compare(lTempFileName) == 0)
            {
            return lIndex;
            }
        }

    return KErrNotFound;
    }

// ---------------------------------------------------------------------------
// Destructor. Frees in-memory policy store entries.
//
// ---------------------------------------------------------------------------
CFlashRTPolicyStore::~CFlashRTPolicyStore()
    {
    iPolicies.ResetAndDestroy();
    }




// ---------------------------------------------------------------------------
// Two phase instantiator.
//
// Created from deserializes data from database file read stream. Used while
// creating store entries from database
// ---------------------------------------------------------------------------
CFlashRTPolicyStore::CPolicyEntry* CFlashRTPolicyStore::CPolicyEntry::NewL(RStoreReadStream& aSource)
    {
    CPolicyEntry* self = new (ELeave) CPolicyEntry();
    CleanupStack::PushL(self);
    self->InternalizeL(aSource);
    CleanupStack::Pop();
    return self;
    }

// ---------------------------------------------------------------------------
// Two phase instantiator.
//
// Created from actual data. Used when a policy entry is created for the first
// time.
// ---------------------------------------------------------------------------
CFlashRTPolicyStore::CPolicyEntry* CFlashRTPolicyStore::CPolicyEntry::NewL(TPolicyID aPolicyID,
                                                      TFileName& aFileName,
                                                      TDes8& aHash)
    {
    CPolicyEntry* self = new (ELeave) CPolicyEntry();
    CleanupStack::PushL(self);
    self->ConstructL(aPolicyID,aFileName,aHash);
    CleanupStack::Pop();
    return self;
    }

// ---------------------------------------------------------------------------
// Deserialize method. Reads deserialized onto internal data members
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::CPolicyEntry::InternalizeL(RStoreReadStream& aSource)
    {
    iPolicyID = aSource.ReadInt32L();
        
    TInt lHashLen = aSource.ReadInt32L();
        
    iHash = HBufC8::NewL(lHashLen);
    
    TPtr8 lHashPtr = iHash->Des();    
    
    aSource.ReadL(lHashPtr,lHashLen);                  //Get hash
        
    iFileNameLength = aSource.ReadInt32L();
        
    aSource.ReadL(iFileName,iFileNameLength); //Get filename
    
    }

// ---------------------------------------------------------------------------
// Serialize method. Writes serialized data onto write stream
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::CPolicyEntry::ExternalizeL(RStoreWriteStream& aDest)
    {
    aDest.WriteInt32L(iPolicyID);
    aDest.WriteInt32L(iHash->Length());

    TPtr8 lHashPtr = iHash->Des();
    aDest.WriteL(lHashPtr);
    aDest.WriteInt32L(iFileNameLength);
    aDest.WriteL(iFileName);
    }

// ---------------------------------------------------------------------------
// Destructor. Frees hash data memory
//
// ---------------------------------------------------------------------------
CFlashRTPolicyStore::CPolicyEntry::~CPolicyEntry()
    {
    if(iHash)
        {
        delete iHash;
        iHash = NULL;
        }
    }

// ---------------------------------------------------------------------------
// Second phase construction.
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyStore::CPolicyEntry::ConstructL(TPolicyID  aPolicyID,
                              TFileName& aFileName,
                              TDes8&     aHash)
    {
    iPolicyID = aPolicyID;
    iFileName = aFileName;
    iHash = aHash.AllocL();
    iFileNameLength = aFileName.Length();
    }

//EOF
