/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that manages all of the file requests.
*
*/


#ifndef __SYMBIAN_FILE_H__
#define __SYMBIAN_FILE_H__

#include <f32file.h>

class CPlatformFlashEngine;

struct FI_URLStreamInfo;
struct SI_StreamHandle;

class CFileEngine;


//abstract class for file object request
class IFileRequestObject
{
	public:
			virtual void CloseDestroyFile()=0;
			virtual TInt ReadFile(unsigned char* buffer
									, unsigned int bufferSize)=0;
			virtual TInt FileSize()=0;
			virtual TInt SeekFile(TSeek mode
								, TInt offset)=0; 
};


class CFileRequestObject: public CBase
							, public IFileRequestObject
{
	public:
			//public constructor
			static CFileRequestObject* OpenCreateFileL(struct MM_Object* iMMObj
									, const char* iFileName);
									
			virtual ~CFileRequestObject();
	
	public: //IFileRequestObject
	
			virtual void CloseDestroyFile();
			virtual TInt ReadFile(unsigned char* buffer
									, unsigned int bufferSize);
			virtual TInt FileSize();
			virtual TInt SeekFile(TSeek mode
								, TInt offset); 
									
	private:
	
			CFileRequestObject(TBool aOwnedHandle,RFile* aFile);
			void CloseHandle();
	
	private: //attributes
	
			RFile*	iFile;			// file handle to open file
			TBool   iOwnedHandle; //flag to indicated whether iFile is owned
			//will be used in CloseDestroyFile to close handle									
	
};


/**
 * The CFileEngineRequest class represents a single queued file request.
 * These requests are managed by CFileEngine.
 * 
 */
class CFileEngineRequest : public CActive
{
public: // c'tor and d'tor
	static CFileEngineRequest* NewL(struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo, TUint16 aStreamId);
	~CFileEngineRequest();
	static TInt ReadFile(RFile* pFile, unsigned char* buffer, unsigned int bufferSize);
	static TInt FileSize(RFile *pFile) {int size = -1; if (pFile) pFile->Size(size); return size;}
	static TInt SeekFile(RFile* pFile, TSeek mode, TInt offset) 
	{ 
		TInt error = KErrGeneral; 
		if (pFile) 
		{
			error = pFile->Seek(mode, offset);	
		}
		 return error;
	}
private:
	void ConstructL(struct FI_URLStreamInfo* aStreamInfo);
	CFileEngineRequest(struct MM_Object* aMMObj, TUint16 aStreamId);
	HBufC* AllocFullPathLC(const TDesC& aBase, const TDesC& aRest);
	TBool CanContentBeTrusted(const TDesC8& aSwfURL);
public: // API
	TUint16 Id() const;

public: // only to be called by CFileEngine
	void StartFileRequest(CFileEngine& aOwner);
	void SetNetBuf(FI_NetworkBufferInfo* aNetBuf);
	void HandleRequestFailed(TInt aError);
    TBool SetCanceled(); //returns ETrue if it is safe to delete now...
	// from CActive
private:
	void RunL();
	TInt RunError(TInt aError);
	void DoCancel();
public:
	static RFile* OpenCreateFileL(struct MM_Object* iMMObj, const char* iFileName);
	
	#ifdef __FEATURE_FLV_RECOG_PLAY
	static void CloseDestroyFile(struct MM_Object* iMMObj,RFile *pFile);
	#else
	static void CloseDestroyFile(RFile *pFile);
	#endif
private: // new functions
	RFile* OpenLocalFileL();
	/*#ifdef RD_MULTIPLE_DRIVE
    RFile* SearchInDefaultL(TDesC& aPath,RFile* aFile);
	#endif*/
	void ReadNextFileSegment();
	void UpdateImageDownScale();
	void CreateFileL();
public: // link data
	TSglQueLink iLinkItem;
private:
	enum TStateFlags { EFlhFileReqInsideFICall =0x1, EFlhFileReqCanceled =0x2 };	
private: // member data
	struct MM_Object* iMMObj;
	TUint16	iId;			// identifier for this request
	RFile*	iFile;			// file handle to open file
	HBufC8* iFileName;		// filename (in raw form from the player)
	CFileEngine* iOwner;	// owner of this object
	
	FI_NetworkBufferInfo* iNetBuf;  // owned by the player
	TPtr8 iDesWrapper;				// wraps the net buffer for Symbian file APIs.
	TInt iFileLen;					// file length in bytes
	TInt iReadSoFar;				// bytes currently read from file
	TBool iIsMp3Stream;
	TInt iStateFlags;
	TBool iImgDownScaleRequired;
	HBufC* iImgDsPath;
};

/**
 *  The CFileEngine class manages all of the file requests (CFileEngineRequest)
 *
 */
class CFileEngine : public CBase
{
public:
	virtual ~CFileEngine();

	static CFileEngine* NewL();

public: // API

	// adds the file request to the queue - starts it if nothing else going on
	unsigned short StartFileRequest(CFileEngineRequest* aRequest);

	// gets a pointer to the requested object, or null if not found
	CFileEngineRequest* GetRequestObjectOrNull(TUint16 aId);

	// cancel's the requested object, if found. If not found does nothing
	TUint16 CancelRequestObject(TUint16 aId);

	// cancels all requests, including the one being processed and any queued.
	void CancelAllRequests();

public: // should only be called by CFileEngineRequest
	void Done();

private:
	void TryQueueNextRequest();
	void CancelActiveTransaction();
	void DoIssueNextRequest(CFileEngineRequest& aRequest);
	RFile* OpenLocalFileL(FI_URLStreamInfo* aStreamInfo);
private:
	CFileEngine();
private:
	TSglQue<CFileEngineRequest>	iRequestQueue;
	CFileEngineRequest* iCurrentRequest;
};

// these are utility functions
class CFlhFileUtils : public CBase
{
public:
    static CFlhFileUtils* NewL(RFs& aFs);
    ~CFlhFileUtils();
private:
	CFlhFileUtils();
public:
	
	TInt DeleteFile(const TDesC& aFile);
	void ResolvePathDotOperators(TDes& aDesToTidy);

private:	
    CFileMan*               iFileMan;
};



#endif	// __SYMBIAN_FILE_H__
