/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*				 implement shared object adaptation.
*
*/
#ifndef USE_SYNC_MSO

#ifndef __SYMBIAN_SHARED_OBJECTS_H__
#define __SYMBIAN_SHARED_OBJECTS_H__

#include "platformbuild.h"
#include "flashliteinterfaces.h"

#include <e32def.h>
#include <e32base.h>
#include <f32file.h>

class CPersistentStorageCleanupRequest;
class CFlhFileUtils;
class CPlatformFlashEngine;

/**
 * The MFlhShObjEngQueueNofifiee interface class allows another class to be informed of
 * successful or failed completion of a request listing all nodes.
 */
class MFlhShObjNodeListObserver
{
public:
	virtual void NotifyOperationComplete(void* aServicingObject, TInt aError)=0;
};

/**
 * The CPersistentStorageNode class is used to track a single instance of a Flash SharedObject.
 *
 */
class CPersistentStorageNode : public CBase
{
public:
    // This class does not take ownership of nameSpace or name, it copies
    // them.
    static CPersistentStorageNode* NewL(const char* aNameSpace, const TDesC* aName = NULL);
    const char* GetNameSpace() const { return iNameSpace; };
    const TDesC* GetName() const { return iName; };
    ~CPersistentStorageNode();
	
public: // link data
	TSglQueLink iLinkItem;

private:
    CPersistentStorageNode();
    void ConstructL(const char* aNameSpace, const TDesC* aName);
    char* iNameSpace;	// swf url
    TDesC* iName;		// so name
};

/**
 * The MFlhShObjCleanupObserver allows a derived class to be notified when a cleanup shared objects operation is complete
 * and what the result of the attempted cleanup was.
 *
 */
class MFlhShObjCleanupObserver
{
public:
	virtual void CleanupResult(TBool aEnoughSpaceCleanedUp)=0;
};


class CFlhShObjUtils;

/**
 * The MFlhShObjReqObs class allows a derived class to receive notification that the CFlhShObjRequesBase
 * with id aId has completed with status aStatus.
 */
class MFlhShObjReqObs
{
public:
	virtual void HandleTransactionComplete(TUint32 aId, TInt aStatus)=0;
};

/**
 * The CFlhShObjRequestBase class is an abstract base class for all Shared Object requests.
 * Shared object requests are created when the player makes a call to a Shared Object function.
 * They are added to a queue, and when started perform asynchronous operations, before returning
 * the result of the request to the player through FI_NotifyDataRequestComplete. 
 */
class CFlhShObjRequestBase : public CActive
{
protected:
	CFlhShObjRequestBase(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId, TInt aBaseFlags);
	void BaseConstructL(const char* aRootMovieGUID, const char* aName);
	//
	void FI_Call(TUint32 aSizeInfo, TInt aStatus);
public: // API
	
	~CFlhShObjRequestBase();
	TUint32 Id();

	/**
	* The function QueueRequestNowL() is called when this object
	* should begin asyc processing.
	*/ 
	virtual void QueueRequestNowL()=0;

	/**
	* The function HandleTransactionFailed() is called by the request owner
	* if QueueRequestNowL() leave'd when called.
	*/ 
	virtual void HandleTransactionFailed()=0; 

	/**
	* This function is called by the request owner when the movie that started the request has ended.
	* After receiving this call the object should not notify the player on completion.
	* If the request is not useful after the movie is ended it should return ETrue so the owner may delete it.
	* If the request is still useful and should be allowed to complete, the function should return EFalse.
	*/
	TBool MovieClosedRequestDeletion();

protected:
	// to be passed into aBaseFlags constructor
	enum { ENotOfUseAfterMovieEnded=0x0, EOfUseAfterMovieEnded=0x1 };
private:
	enum { EMovieHasEnded =0x2 };
public:
	TSglQueLink iLinkItem;
protected:
	MFlhShObjReqObs&	iObs;
	CFlhShObjUtils&		iShObjUtils;
	struct MM_Object*	iMMObj;
	TDesC*				iName;
	unsigned long		iRequestId;
	RFile				iFile;
	TInt				iBaseFlags;
	HBufC*              iMSOName;          // this is the name of the MSO file

};

/**
 * The ClhShObjRequestFileSize class is a type of Shared Object request that determines the size
 * of a shared object, identified by aRootMovieGUID and aName.
 */
class CFlhShObjRequestFileSize : public CFlhShObjRequestBase
{
public:
	static CFlhShObjRequestFileSize* NewL(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId,
		const char* aRootMovieGUID, const char* aName);
private:
	CFlhShObjRequestFileSize(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId);
	void ConstructL(const char* aRootMovieGUID, const char* aName);
	~CFlhShObjRequestFileSize();
private:
	// from CActive
	void RunL();
	void DoCancel();
	TInt RunError(TInt aError);
	// from base
	void QueueRequestNowL();
	void HandleTransactionFailed();
};

/**
 * The ClhShObjRequestFileWrite class is a type of Shared Object request that writes data to a 
 * shared object, identified by aRootMovieGUID and aName.
 */
class CFlhShObjRequestFileWrite : public CFlhShObjRequestBase, public MFlhShObjCleanupObserver, public MFlhShObjNodeListObserver
{
public:
	static CFlhShObjRequestFileWrite* NewL(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId,
		const char* aRootMovieGUID, const unsigned char* aData, unsigned long aDataSize, const char* aName, TBool aEnforceMaxSize);
private:
	CFlhShObjRequestFileWrite(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId, TBool aEnforceMaxSize);
	void ConstructL(const char* aRootMovieGUID, const char* aName, const unsigned char* aData, unsigned long aDataSize);
	~CFlhShObjRequestFileWrite();
private:
	// from CActive
	void RunL();
	void DoCancel();
	TInt RunError(TInt aError);
	// from base
	void QueueRequestNowL();
	void HandleTransactionFailed();
	// from MFlhShObjCleanupObserver
	void CleanupResult(TBool aEnoughSpaceCleanedUp);
	// from MFlhShObjNodeListObserver
	void NotifyOperationComplete(void* aServicingObject, TInt aError);
	// new fns
	void DoNextStepL();
	void Reque();
private:
	enum { ECleanupUndecided=0, EGettingLocalSpace, EGotLocalSpace, EGettingTotalSpace,
		EGotTotalSpace, ECleaningUp, EEnsuringDirectory, ESavingFile, EZeroDataDelete };
private:
	HBufC8*	iRootMovieGUID;
	HBufC8*	iData;
	TInt	iState;
	TBool	iEnforceMaxSize;
	TInt	iTotalPersistentDirSize;
	TInt	iCurrentDirSize;
	CBase*	iOutstandingReq;
};

/**
 * The ClhShObjRequestFileSize class is a type of Shared Object request that reads the contents of
 * the shared object identified by aRootMovieGUID and aName into the buffer aData.
 */
class CFlhShObjRequestFileRead : public CFlhShObjRequestBase
{
public:
	static CFlhShObjRequestFileRead* NewL(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId,
		const char* aRootMovieGUID, unsigned char* aData, unsigned long aDataSize, const char* aName);
private:
	CFlhShObjRequestFileRead(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId, unsigned char* aData, unsigned long aDataSize);
	void ConstructL(const char* aRootMovieGUID, const char* aName);
	~CFlhShObjRequestFileRead();
private:
	// from CActive
	void RunL();
	void DoCancel();
	TInt RunError(TInt aError);
	// from base
	void QueueRequestNowL();
	void HandleTransactionFailed();
private:
	TPtr8 iData;
}; 

/**
 * The MFlhShObjEngQueueNofifiee interface class allows another class to be informed of
 * the number of items remaining to be processed in the Queue.
 */
class MFlhShObjEngQueueNofifiee
{
public:
	virtual void NotifyNumberOfItemsInQueue(TInt aNumItemsInQueue)=0;
};

class CFlhShObjUtils;

/**
 * The CFlhShObjEngine class is the main class for initiating shared object operations.  
 */
class CFlhShObjEngine : public CBase
, public MFlhShObjReqObs
{
public:
	static CFlhShObjEngine* NewL(RFs& aFs, CFlhFileUtils& aFileUtils, const TDesC& aPersistenceDirectory, CPlatformFlashEngine& aEngine);
	~CFlhShObjEngine();

private:
	CFlhShObjEngine();
	void ConstructL(RFs& aFs, CFlhFileUtils& aUtils, const TDesC& aPersistenceDirectory, CPlatformFlashEngine& aEngine);
	
public:
	
	short AsyncGetPersistentDataSizeL(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId);
	short AsyncGetPersistentDataL(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId, unsigned char* aData,
										unsigned long aDataSize);
	short AsyncSetPersistentDataL(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId, const unsigned char* aData,
										unsigned long aDataSize, TBool aEnforceMaxSize);
	short CancelPersistentDataRequest(struct MM_Object* aMMObj, unsigned long aRequestId);

	/**
	* closes all requests without completing, including the one being processed and any queued, synchronously.
	*/
	void CloseAllRequests();

	/** 
	* Notifies the CFlhShObjEngine that the player does not care about the results of any outstanding requests as the movie has changed.
	* note any requests added after this call are assumed to be in the scope of the new movie and will notify the player
	* upon completion.
	*/
	void MovieClosed();

	/**
	* Registers aNotifiee to be informed of the number of items in the queue.
	* To de-register call again with NULL pointer.
	*/
	TInt RegisterForNotificationsReturnNumToGo(MFlhShObjEngQueueNofifiee* aNotifiee);

	/**
	* The Notifiee function returns a pointer to the currently registered Notifiee, or NULL if no Notifiee is registered.
	*/
	const MFlhShObjEngQueueNofifiee* Notifiee() const;

private: // from MFlhShObjReqObs
	void HandleTransactionComplete(TUint32 aId, TInt aStatus);

private: // internal queue handling functions
	short StartRequest(CFlhShObjRequestBase* aRequest);
	void TryQueueNextRequest();
	void CancelActiveTransaction();
	void DoIssueNextRequestL(CFlhShObjRequestBase& aRequest);
	void CountCurrentItemsAndNotify();
	TInt CountCurrentItems();
private:
	TSglQue<CFlhShObjRequestBase>	iRequestQueue;
	CFlhShObjRequestBase* iCurrentRequest;
	MFlhShObjEngQueueNofifiee* iNotifiee;
	CFlhShObjUtils* iShObjUtils;
};

/**
 * The CPersistentStorageCleanupRequest class encapsulates an asynchronous shared object cleanup request.
 */
class CPersistentStorageCleanupRequest : public CActive, public MFlhShObjNodeListObserver
{
private:
	enum
	{
		EPrepareCleanup,
		EExtractObjects,
		ESelectObject,
		ESizeObject,
		ECleanupObject
	};

public:
    static CPersistentStorageCleanupRequest* NewL(struct MM_Object* aMMObj, 
                                                  TUint32 aNumBytesToClean,
                                                  CFlhShObjUtils& aShObjUtils,
                                                  TInt aId,
												  CPersistentStorageCleanupRequest** aSelf,
												  MFlhShObjCleanupObserver& aObs);
    ~CPersistentStorageCleanupRequest();
public:
	// from MFlhShObjNodeListObserver
	void NotifyOperationComplete(void* aServicingObject,TInt aError);

private:
    CPersistentStorageCleanupRequest(struct MM_Object* aMMObj, 
                                     TUint32 aNumBytesToClean, 
                                     CFlhShObjUtils& aShObjUtils,
                                     TInt aId,
									 CPersistentStorageCleanupRequest** aSelf,
									 MFlhShObjCleanupObserver& aObs);
private:
    void DoNextStage();
    TBool CleanupPersistentDirectoryL();
    void RunL();
    TInt RunError(TInt aError);
    void DoCancel();
    void SendCallbackAndDeleteThis(short aStatus);
    
private:
    struct MM_Object* iMMObj;
    TInt	          iId;			    // identifier for this request
    TUint32           iNumBytesToClean; // How many bytes at least must be cleaned.
    CFlhShObjUtils&	  iShObjUtils;		// not owned
    TBool             iDone;           // True if all work is done.
    TBool             iRequestGranted; // True if user accepts cleanup request.

    // These are needed by CleanupPersistentDirectory.
    TInt                   iStage; // Which stage of cleanup is done.
    TSglQue<CPersistentStorageNode>*	iNodeList;
    TUint32                iDeletedBytes;
    CPersistentStorageNode* iOldest;
    const char*            iNameSpace;
	CPersistentStorageCleanupRequest** iSelf;
	MFlhShObjCleanupObserver& iObs;
	CBase* iOutstandingReq;
	TInt iDirSize;
};

#endif // __SYMBIAN_SHARED_OBJECTS_H__

#endif //USE_SYNC_MSO

