/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*				 implement socket adaptation.
*
*/

#ifndef __SYMBIAN_SOCKETS_H__
#define __SYMBIAN_SOCKETS_H__

#include <in_sock.h>
#include "symbian_engine.h"
#include "timer_object.h"
#include "ipconn_mgr.h"
#include "conn_handler.h"

class MSocketNotifier  
{
public:
	enum TErrorType
	{
		EDisconnected,
		ETimeoutOnConnect,
		ETimeOutOnWrite,
		EGeneralReadError,
		EGeneralWriteError,
		EUnknownError
	};

	virtual void ReportErrorL(TErrorType aErrorType, TInt aErrorCode) = 0;

	virtual TBool ResponseReceivedL(const TDesC8& aBuffer) = 0;

	virtual void WriteCompleted() = 0;
	
	virtual TBool IsOpen() = 0;
	
};
#ifdef FEATURE_XML_SOCKET
//Forward declaration
class CFlashSocketEngine;
class CFlashSocketReader;
class CFlashSocketWriter;

class CFlashSocket : public CActive
							, public MSocketNotifier
							, public MFlhAsyncHandler
{
public:
	enum TStatusCode
	{
		ENotConnected,
		EResolving,
		EResolved,
		EFailedResolve,
		EConnecting,
		EConnected,
		EFailedConnect,
		EClosed,
		EClosedWithError
	};
	
public:

	static CFlashSocket* NewL(CFlashSocketEngine& aSocketEngine, const char* aHost, unsigned short aPort/*, long maxWrite*/);

	virtual ~CFlashSocket();

    void ConnectL();

	void CloseConnection();

	void WriteL(const TDesC8& aBuffer);

private:

	static CFlashSocket* NewL();

public: // from MSocketNotifier
	TBool ResponseReceivedL(const TDesC8& aBuffer);
	void WriteCompleted();
	void ReportErrorL(TErrorType aErrorType, TInt aErrorCode);
	TBool IsOpen();

protected: // from CActive
	void DoCancel();

    void RunL();

protected: // from MFlhAsyncHandler
	void DoRunL(const CActive* /*aActive*/);

	TInt DoRunError(const CActive* /*aActive*/, TInt aError);

protected: //New methods

	void ConnectL(TUint32 aAddr);

    void ChangeStatus(TStatusCode aNewStatus, TInt aErrorCode);

	void StartTimer();

private: // New methods
	
	CFlashSocket(CFlashSocketEngine& aSocketEngine, unsigned short aPort/*, long maxWrite*/);
	
    void ConstructL(const char* aHost);

private: // Member variables

	enum TSocketState
	{
		EStateOpen,
		EStateClosed
	};

    static const TInt			KSocketTimeOut;

	TBool						iNetActive;

	TStatusCode					iSocketStatus;

	RSocket                     iSocket;

	CFlashSocketReader*         iSocketReader;
	
	CFlashSocketWriter*         iSocketWriter;


	RHostResolver               iResolver;

	TNameEntry                  iNameEntry;

	TNameRecord					iNameRecord;

	CFlhTimerObject*			iTimer;

	TInetAddr                   iAddress;

    TInt                        iPort;

	HBufC*						iServerName;

	CFlashSocketEngine&			iSocketEngine;

	TSocketState				iState;

};

class CFlashSocketEngine : public CBase,
						   public MFlhIPConnectObserver,
						   public MFlhAsyncHandler

{
public:
	static CFlashSocketEngine* NewL(CPlatformFlashEngine& aEngine);

	void NotifyStatusChange(CFlashSocket* aSocket, CFlashSocket::TStatusCode aStatusCode, TInt aErrorCode);

	~CFlashSocketEngine();
	void ExpediteConnectionL(); 

	bool OpenNewSocketL(FI_SocketConnectionInfo* aConnInfo);

	bool CloseSocket(unsigned long aSocketId);

	short SendL(unsigned long aSocketId, FI_NetworkBufferInfo* aDataBuffer);

	short ReceiveL(CFlashSocket* aSocket, const TDesC8& aBuffer);

	CPlatformFlashEngine* GetPlatformEngine() const;

	void ResetEngine();
	void NotifyIPConnectionStatus(CConnHanlder* aConnHandler);
	
	CConnHandler* GetConnHandler() { return iConnHandler; }
private: // from CFlhIPConnectObserver
	void ConnectionEventL(TFlhIPEvent aEvent);


	void HandleNetworkAccessDenied();

protected: // from MFlhAsyncHandler
	void DoRunL(const CActive* /*aActive*/);

	TInt DoRunError(const CActive* /*aActive*/, TInt aError);

private:
	void ConstructL();
	
	CFlashSocketEngine(CPlatformFlashEngine& aEngine);

	void Reset();
	
	void CleanupSocketList();

	void HandleNoConnection();

	void AttemptStartNetworkTimer();

private:

	enum TNetworkAccessConnCode
	{
		EFlhNReqNotConnectedToNetwork	=0x0,	// there is no current network connection
		EFlhNReqConnectingToNetwork		=0x1,	// connecting
		EFlhNReqConnectedToNetwork		=0x2,	// connected
	};

	static const TInt				KNetworkTimeOut;

	CPlatformFlashEngine&			iEngine;

	CArrayPtrFlat<CFlashSocket>*	iSocketList;



	CFlhTimerObject*				iTimer;

	TInt							iConnStat;

	CIPConnMgr* iIPConnectionMgr;
	CConnHandler* iConnHandler;
};

#endif //FEATURE_XML_SOCKET
class CFlashSocketReader : public CActive
{
public:
	static CFlashSocketReader* NewL(MSocketNotifier& aSocketNotifier, RSocket& aSocket/*, unsigned long bufferSize*/);

	~CFlashSocketReader();

	void Start();

protected: // from CActive
	void DoCancel();

	void RunL();	

private:
	CFlashSocketReader(MSocketNotifier& aSocketNotifier, RSocket& aSocket/*, unsigned long bufferSize*/);

	void ConstructL();

	void IssueRead();

private: // Member variables

    RSocket&                iSocket;

	MSocketNotifier&        iSocketNotifier;

	TBuf8<2048>				iBuffer;

	TSockXfrLength			iDummyLength;
};

class CFlashSocketWriter : public CActive
							, public MFlhAsyncHandler
{
private:
	enum
	{
		/*
		 * In the current model, we will only ever need two transfer buffers:
		 * one for the HTTP header, and possibly a second one for the POST
		 * data.
		 */
		kMaxWriteBuffers = 8
	};
public:
    static CFlashSocketWriter* NewL(MSocketNotifier& aSocketNotifier, RSocket& aSocket);

	~CFlashSocketWriter();

    void IssueWriteL(const TDesC8& aData);

protected: // from CActive
	void DoCancel();

	void RunL();	

protected: // from MFlhAsyncHandler
	void DoRunL(const CActive* /*aActive*/);

	TInt DoRunError(const CActive* /*aActive*/, TInt aError);

private:
	CFlashSocketWriter(MSocketNotifier& aSocketNotifier, RSocket& aSocket);

	void ConstructL();

    void SendNextPacket();


private: // Member variables

    static const TInt KWriterTimeOut;
    static const TInt KMeanReadWriteTimeDelay;
 
	enum TWriteState 
	{
		ESending, 
		EWaiting
	};

    RSocket&				iSocket;

	MSocketNotifier&		iSocketNotifier;

	// At most, there will be kMaxTransferBuffers buffers of data to write
	HBufC8*					iWriteBuffers[kMaxWriteBuffers];

	TInt					iWriteBufferQueueFront;
	TInt					iWriteBufferQueueRear;
	TInt					iWriteBufferQueueCount;

	CFlhTimerObject*		iTimer;


	TWriteState				iWriteStatus;
};

#endif	// __SYMBIAN_SOCKETS_H__