/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that provides implementation for all 
*				 Flash sound playback
*
*/


#ifndef __SYMBIAN_SOUND_H__
#define __SYMBIAN_SOUND_H__

#include <mdaaudiosampleplayer.h>
#include <mda/common/audio.h>
#include <mdaaudiooutputstream.h>

#include "platformbuild.h"

#include "platformtypes.h"
#include "commondef.h"


#include "snddecodersinterface.h"


#include "saflash_consts.h"
#include "saplayer_build.h"


#define STREAM_SOUND_TEMP_QUEUE_SIZE 32
#define FIRST_MIDI_NUM 1000

//#ifdef EKA2
// EKA2 performs better with larger buffers.
// DK - now we support variable sound rates, we may need dedicated buffer sizes for each rate.
#define STREAM_SOUND_BUFFER_SIZE (16384)
//#else
//#define STREAM_SOUND_BUFFER_SIZE 1024
//#endif // EKA2

#define STREAM_SOUND_BUFFER_TIME 32000
#define STREAM_SOUND_NUM_BUFFERS 8

enum ESoundHandlerMode
{
	ESwfMP3Audio=0,
	EFlvSorensonAudio=1,
	EFlvOn2Vp6Audio=2
};

/**
 * The CachedSound object represents an instance of sound data passed to CPlatformFlashEngine through
 * SI_CacheSound().
 */
class CachedSound
{
public:
	CachedSound(TBool aIsReference);
	~CachedSound();
	TFileName iFname;
	TUint32 iId;
	TBool iBReference;
	CachedSound* iNext;
};

class CPlatformFlashEngine;
class CAknKeySoundSystem;

/**
 * The CSoundHandler class provides an implementation for all Flash sound playback. It supports
 * Device Sound (supporting SI_CacheSound() and SI_PlayCachedSound()), Device Sound references
 * (supporting SI_PlaySound()) and streaming sound (supporting SI_OpenStreamSoundDevice(), SI_CloseStreamSoundDevice(),
 * and SI_PlayStreamSoundBuffer(). It also handles loading of the sound decoder DLL and delegating calls to
 * SI_SI_CreateSoundDecompressor(), SI_DecompressSound(), SI_DestroySoundDecompressor() and SI_SetupSoundDecompressor().
 */
class CSoundHandler : public CBase, public MMdaAudioPlayerCallback, public MMdaAudioOutputStreamCallback
{
public:
	CSoundHandler();
	virtual ~CSoundHandler();
	void InitializeSoundL(CPlatformFlashEngine* aEngine);

public: // from MMdaAudioPlayerCallback
	virtual void MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& aDuration);
	virtual void MapcPlayComplete(TInt aError);

public: // sound API
	TInt16 PlaySoundL(TAny* aSoundBuffer, TUint32 aSoundBufferSize, TUint32 aCount);
	TInt16 CacheSoundL(TAny* aSoundBuffer, TUint32 aSoundBufferSize, TUint32* aSoundID);
	TUint16 PlayCachedSound(TUint32 aSoundID, TUint32 aCount);
	void FreeCachedSound(TUint32 aSoundID);
	void StopSound();
	TUint16 PauseSound();
	TUint16 ResumeSound();

	TBool OKToPause();

public: // from MMdaAudioOutputStreamCallback
    virtual void MaoscPlayComplete(TInt aError);
	virtual void MaoscBufferCopied(TInt aError, const TDesC8& aBuffer);
	virtual void MaoscOpenComplete(TInt aError);

public: // streaming sound API
	TUint16 OpenStreamSoundDeviceL();
	TUint16 CloseStreamSoundDevice();
	TUint16 PlayStreamSoundBufferL(TAny* aData);
	void PollStreamSound();

public: // sound decompressor API
	TAny* CreateSoundDecompressor(FI_StreamSoundInfo* aSoundInfo, TUint32 aFormatInfo);
	void SetupSoundDecompressor(TAny* aDecompressor, FI_StreamSoundDecompressorInfo* aDecompressorInfo);
	TInt32 DecompressSound(TAny* aDecompressor, TAny* aDestBuffer, TUint32 aNumSamples);
	void DestroySoundDecompressor(TAny* aDecompressor);
	TBool LoadSoundLibrary();
	void  UnloadSoundLibrary();

private:
	
	void DoMaoscOpenCompleteL(TInt aError);

	// make sure that keysounds don't interfere with Flash sound playback
	void SuppressKeySounds();
	void RestoreKeySounds();
	TInt GetChannelType();

	unsigned short HandlePlayStreamPlayL(TAny* aData);
	unsigned short HandlePlayStreamUnderflowL(TAny* aData);
	void AllocSoundBuff();
	void FreeSoundBuf();	
	void SetSoundHandlerMode(ESoundHandlerMode aMode);					
						
public: // other sound functions
	void SetVolume(TInt aVolume);
	TInt GetVolume();
	void CleanupCachedFiles(RFs &aFs);
	void CleanupSoundPlayer();
	void DoSoundL();
	TBool IsMuted() { return ((iVolume & KMuteMask) == KMuteMask); };
	TBool IsPlaying();
	void HandleFlvSetup(TInt32 aCodecType);
private:
	enum {
		EMdaNone = 0,
		EMdaInitializing,
		EMdaPlaying,
		EMdaPaused,
		EMdaPausedAtStart,
		EMdaUnderflow,
		EMdaDestroy
	};

	CPlatformFlashEngine* iEngine;
	CAknKeySoundSystem* iKeySoundSystem; // for suppressing keysounds while Flash sound is playing.
	TBool iKeySoundsOff;

	CMdaAudioPlayerUtility* iMdaAudioPlayerUtility;
	TUint iVolume;
	TUint iOldVolume;
	TUint32 iMidi_num;
	TUint8 iMdaEventState;
	CachedSound* iCachedSounds;
	TUint32 iSoundQueue[1]; // short queue, only 1 sound
	TInt iSoundQueuePlayCount[1];
	TBool iMdaStreamReady;
	TBool iDeviceAudioSet;
	CMdaAudioOutputStream * iMdaAudioStream;
	TMdaAudioDataSettings iSettings;
	TUint8 iMdaStreamState;
	TUint8* iTmpQueue[STREAM_SOUND_TEMP_QUEUE_SIZE];
	TInt iBufSamples;
	TInt iTmpOutBuf;
	TPtrC8* iOutQueue[STREAM_SOUND_TEMP_QUEUE_SIZE];
	TAny* iAllocQueue[STREAM_SOUND_TEMP_QUEUE_SIZE];
	TInt iOutQueuePos;
	TUint32 iSndPos;
	TUint iBuffersQueued;
	TMdaPriority iSoundPriority;

	RLibrary iSndDecDll;
	TBool	iSndLibLoaded;
	TUint16 iAllocdInitNo;
	TUint16 iAllocdBlockNo;
	

	PFNC_DI_FLASHSNDDEC_GETINTERFACE	iFncGetInterface;



	PFNC_DI_CREATESOUNDDECOMPRESSOR		iFncCreateSoundDecompressor;
	PFNC_DI_DECOMPRESSSOUND				iFncDecompressSound;
	PFNC_DI_SETUPSOUNDDECOMPRESSOR		iFncSetupSoundDecompressor;
	PFNC_DI_DESTROYSOUNDDECOMPRESSOR	iFncDestroySoundDecompressor;
	PFNC_DI_GETSUPPORTEDFORMATS			iFncGetSupportedFormats;
	PFNC_DI_SETUPDECODERLIBRARY			iFncSetupDecoderLibrary;
	
	TBool 								iStereoSupported;
	ESoundHandlerMode					iHandlerMode;
	FI_StreamSoundSetupInfo 			iSetupInfo;
	FI_StreamSoundInfo					iSoundInfo;

};

#endif	// __SYMBIAN_SOUND_H__

