/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which implements the
*                main hostengine functionality.
*
*/


#include <sensrvchannelinfo.h>
#include <sensrvchannelfinder.h>
#include <sensrvaccelerometersensor.h>
#include <sensrvgeneralproperties.h>
#include <e32math.h>
#include <e32debug.h>

#include "symbian_accelerometer.h"
#include "extension_mem_mgmt.h"


void* operator new(unsigned int size, IAMAllocator* pAllocator, unsigned int arraySize)

{
    int* mem = NULL;
    (void)arraySize; 

    mem = (int*)IAMALLOCATOR_Alloc(pAllocator, size, NULL);
    return (void*)mem;
}

void deletefunc(void* mem)
{
    IAMALLOCATOR_Free(NULL,mem);
}

void operator delete(void*, IAMAllocator* /*pAllocator*/, unsigned int) throw() { };

AccelerometerImpl::AccelerometerImpl():
        m_refCount(0),
        iChannel(NULL),
		iStartTime(0),
		iLastUpdateTime(0),
		iUpdateInterval(0), 
        iScaleFactor(0),
        iChannelDataRateWriteIndex(-1)
    {
    }

AccelerometerImpl::~AccelerometerImpl()
{
    if(iChannel)
        {
        iChannel->CloseChannel();
        delete iChannel;
        }
    
    if(iCancel)
        delete iCancel;
    
    CHostEngineGlobals* globals = NULL;
    TRAP_IGNORE(globals = CHostEngineGlobals::CreateHostEngineGlobalsL());
    if(globals)
    	globals->GetPlatformFlashEngine()->DeregisterExtensionCallback(this);
}

S32 AccelerometerImpl::GetElapsedTime(S32 aTime)
{
    if (!iStartTime)
        iStartTime = aTime;
    return aTime - iStartTime;
}

AMErr AccelerometerImpl::Init()
    {
    DEBUGPRINT("%s\n", "AccelerometerImpl-Init() ++");
    AMErr err = kAMErrNone;
    //! create the vtbl
    m_pFncs = (AMFNCS(IAMAccelerometer)*) AMC_MALLOC(m_pAllocator, sizeof(AMFNCS(IAMAccelerometer)));
    if (!m_pFncs)
        {
        err = kAMErrOutOfMemory;
        goto exit_gracefully;
        }

    //Set up function pointers
    m_pFncs->AddRef = AccelerometerImpl::AddRef;
    m_pFncs->Release = AccelerometerImpl::Release;
    m_pFncs->Start = AccelerometerImpl::Start;
    m_pFncs->SetUpdateInterval = AccelerometerImpl::SetUpdateInterval;
    m_pFncs->QueryInterface = NULL;

    iCancel = new AccelerometerCancel(*this, m_pAllocator);
    iCancel->Init();
    
    CHostEngineGlobals* globals = NULL;
    TRAP_IGNORE(globals = CHostEngineGlobals::CreateHostEngineGlobalsL());
    if(globals)
    	globals->GetPlatformFlashEngine()->RegisterExtensionCallback(this);
    
    DEBUGPRINT("%s\n", "AccelerometerImpl-Init() --");
    exit_gracefully: return err;
}

AMErr AccelerometerImpl::Start(IAMAccelerometer* pAccel,
        AMAccelerometerStartCallback* pCallback, void* pUserData, /*S32 timer,*/
        IAMAccelerometerStart** ppStart)
{    
    DEBUGPRINT("%s\n", "AccelerometerImpl::Start() ++");
    AccelerometerImpl* pImpl = (AccelerometerImpl*) pAccel;
    TRAPD(err, pImpl->StartImplL(pCallback,pUserData,/*timer,*/ppStart) );
    
    if(err==KErrNone){
        DEBUGPRINT("%s\n", "AccelerometerImpl::Start() --");
        return kAMErrNone;
    }
    else{
        DEBUGPRINT("%s\n", "AccelerometerImpl::Start() --");
        return kAMErrCouldNotBeCreated;
    }
}

void AccelerometerImpl::StartImplL(AMAccelerometerStartCallback *pCallback, void* pUserData, /*S32 timer,*/
        IAMAccelerometerStart** ppStart)
    {    
    DEBUGPRINT("%s\n", "AccelerometerImpl-StartImplL() ++");
    iCallback = pCallback;
    ipUserData = pUserData;
    
    CSensrvChannelFinder* sensrvChannelFinder = CSensrvChannelFinder::NewLC();
    RSensrvChannelInfoList channelInfoList;
    CleanupClosePushL( channelInfoList );
    
    TSensrvChannelInfo channelInfo;
    channelInfo.iChannelType = KSensrvChannelTypeIdAccelerometerXYZAxisData;
    
    sensrvChannelFinder->FindChannelsL(channelInfoList,channelInfo);
     
    const int NO_CHANNEL_FOUND(-1);
    TInt channelIdx(NO_CHANNEL_FOUND);
    
    for (TInt i = 0; i < channelInfoList.Count(); i++)
        {
        if (channelInfoList[i].iChannelType
                == KSensrvChannelTypeIdAccelerometerXYZAxisData)
            {
            channelIdx = i;
            break;
            }
        }
    
    if(NO_CHANNEL_FOUND != channelIdx)
        {
        iChannel = CSensrvChannel::NewL(channelInfoList[channelIdx]);
        *ppStart = iCancel;
        iChannel->OpenChannelL();
        iChannel->StartDataListeningL( this, 
                                                 1, //KDesiredCount, 
                                                 1, //KMaxCount, 
                                                 0//(int)timer 
                                                 );
        DEBUGPRINT("%s\n", "AccelerometerImpl-StartImplL() Started data listening");
        
        }
    else
        {
        //error handling
        //inform player that there is no sensor channel found..
        (*iCallback)(NULL, kAMErrAlreadyCreated, NULL);
        }
    
    channelInfoList.Close();
    CleanupStack::Pop( &channelInfoList );
    CleanupStack::PopAndDestroy( sensrvChannelFinder );
    DEBUGPRINT("%s\n", "AccelerometerImpl-StartImplL() --");
    }

void AccelerometerImpl::GetChannelDataRateInfoL(){
    if (iChannelDataRateWriteIndex!=-1)
        return;

    RSensrvPropertyList propertyList;
    CleanupClosePushL(propertyList);

    iChannel->GetAllPropertiesL(propertyList);
    for (TInt i = 0; i < propertyList.Count(); i++ ){
        TSensrvProperty property = (TSensrvProperty)propertyList[i];

        if (property.GetPropertyId() == KSensrvPropIdDataRate){
            TInt arrayIndex = property.GetArrayIndex();
            TInt propertyValue;
            property.GetValue(propertyValue);
            if(property.ReadOnly()){
                iChannelDataRatesIndexes.Append(arrayIndex);
                iChannelDataRatesValues.Append(propertyValue);
            }
            else
                iChannelDataRateWriteIndex = arrayIndex;
        }
    }
    CleanupStack::PopAndDestroy(&propertyList);
}

TInt AccelerometerImpl::GetDataRateValueIndex(S32 aTimer)
{
    RArray<TInt> channelDataRates;
    TRAPD(err, GetChannelDataRateInfoL());
    if(err != KErrNone)
    	return -1;
    
    if(!iChannelDataRatesValues.Count())
        return -1;

    channelDataRates = iChannelDataRatesValues;
    channelDataRates.Sort();

    aTimer = aTimer/1000;
    TInt htzValue = 1/aTimer;

    if(htzValue<channelDataRates[0])
        return iChannelDataRatesIndexes[iChannelDataRatesValues.Find(channelDataRates[0])];

    if(htzValue>channelDataRates[channelDataRates.Count()-1])
            return iChannelDataRatesIndexes[iChannelDataRatesValues.Find(channelDataRates.Count()-1)];

   
    TInt i;
    for (i=1; i<channelDataRates.Count(); ++i){
        if(htzValue<channelDataRates[i])
            break;
    }
    
    TInt midVal = channelDataRates[i-1] + channelDataRates[i];
    if (htzValue < midVal)
        return iChannelDataRatesIndexes[iChannelDataRatesValues.Find(channelDataRates[i-1])];
    return iChannelDataRatesIndexes[iChannelDataRatesValues.Find(channelDataRates[i])];
}

void AccelerometerImpl::SetUpdateInterval(IAMAccelerometer* pAccel, S32 timer)
    {
    AccelerometerImpl* pImpl = (AccelerometerImpl*) pAccel;
    pImpl->SetUpdateIntervalImpl(pAccel,timer);
    }

void AccelerometerImpl::SetUpdateIntervalImpl(IAMAccelerometer* /*pAccel*/, S32 timer)
    {
    DEBUGPRINT("%s\n", "AccelerometerImpl-SetUpdateIntervalImpl() ++");
    iUpdateInterval = timer;
    if (!iChannel)
        return;

    TInt dataRateValueIndex = GetDataRateValueIndex(timer);
    TSensrvProperty dataRateProperty;

    dataRateProperty.SetPropertyId(KSensrvPropIdDataRate);
    dataRateProperty.SetArrayIndex(iChannelDataRateWriteIndex);
    dataRateProperty.SetValue(dataRateValueIndex);

    TInt retVal = iChannel->SetProperty(dataRateProperty);

    DEBUGPRINT("%s\n", "AccelerometerImpl-SetUpdateIntervalImpl() --");
    }

AMErr AccelerometerImpl::Create(IAMAllocator* pAllocator,
        IAMAccelerometer** pAccel)
    {
    DEBUGPRINT("%s\n", "AccelerometerImpl-Create() ++");
    AMErr err = kAMErrNone;
    AccelerometerImpl* pImpl = NULL;
    if (! pAccel)
    {
        err = kAMErrBadParam;
        goto exit_gracefully; 
    }

    *pAccel = NULL;
    pImpl = AMC_NEW(pAllocator, AccelerometerImpl);
    if ( !pImpl )
    {
        err = kAMErrOutOfMemory;
        goto exit_gracefully; 
    }
    pImpl->m_pAllocator = pAllocator;
    err = pImpl->Init();
    if ( kAMErrNone != err )
    {
        goto exit_gracefully; 
    }

    // transfer ownership to caller 
    *pAccel =  pImpl;
    pImpl = NULL;

exit_gracefully:

    if ( pImpl )
    {
        AMC_DELETE(pImpl);
        pImpl = NULL;
    }
    DEBUGPRINT("%s\n", "AccelerometerImpl-Create() --");
    return err;
    }

U32 AccelerometerImpl::AddRef(IAMAccelerometer* pAccel)
    {
    AccelerometerImpl* pImpl = (AccelerometerImpl*) pAccel;
    return ++(pImpl->m_refCount);
    }
U32 AccelerometerImpl::Release(IAMAccelerometer* pAccel)
    {
    AccelerometerImpl* pImpl = (AccelerometerImpl*) pAccel;
    --(pImpl->m_refCount);
    if (0 == pImpl->m_refCount)
    {
        AMC_DELETE(pImpl);
        return 0;
    }
    return pImpl->m_refCount;
    }

TReal AccelerometerImpl::GetScaleFactor()
{
    TInt scaleVal=0, propertyCount;
    TReal scaleFactor = -1, measureVal=0;
    if (iScaleFactor)
        return iScaleFactor;
    RSensrvPropertyList propertyList;
    CleanupClosePushL(propertyList);
    TRAPD(err, iChannel->GetAllPropertiesL(propertyList));
    propertyCount = propertyList.Count();
    if(KErrNone != err || !propertyCount)
        return scaleFactor;
    TBool getMeasureValue = ETrue;
    TBool getScaleValue = ETrue;
    for(TInt i=0; i<propertyCount; ++i)
    {
        if(propertyList[i].GetPropertyId() == KSensrvPropIdMeasureRange &&
                                                               getMeasureValue)
        {
            propertyList[i].GetValue(measureVal);
            getMeasureValue = EFalse;
            DEBUGPRINT("Found KSensrvPropIdMeasureRange: %d", measureVal);
        }
        if(propertyList[i].GetPropertyId() == KSensrvPropIdScaledRange &&
                                                               getScaleValue)
        {
            propertyList[i].GetValue(scaleVal);
            getScaleValue = EFalse;
            DEBUGPRINT("Found KSensrvPropIdScaledRange: %d", scaleVal);
        }
        if (!getMeasureValue && !getScaleValue)
            break;
    }
    CleanupStack::PopAndDestroy(&propertyList);
    TReal output;
    if(Math::Pow(output, 2, scaleVal-1) != KErrNone)
        return -1;
    if(0.0 == measureVal)
        scaleFactor = 2/output;
    else if(1.0 == measureVal)
        scaleFactor = 8/output;
    DEBUGPRINT("Final scalefactor: %f", scaleFactor);
    iScaleFactor = scaleFactor;
    return scaleFactor;
}
void AccelerometerImpl::DataReceived( CSensrvChannel& aChannel, TInt /*aCount*/, TInt /*aDataLost*/ ) 
    {
    DEBUGPRINT("%s\n", "AccelerometerImpl-DataReceived() ++");
    if (aChannel.GetChannelInfo().iChannelType
            == KSensrvChannelTypeIdAccelerometerXYZAxisData)
        {
        TSensrvAccelerometerAxisData accData;
        TPckg<TSensrvAccelerometerAxisData> accPackage( accData );
        aChannel.GetData( accPackage );
        
        AMAccelerometerInfo* accelInfo = (AMAccelerometerInfo*)AMC_MALLOC(m_pAllocator, sizeof(AMAccelerometerInfo));
        TReal scalefactor = GetScaleFactor();
        accelInfo->accelerationX = scalefactor * accData.iAxisX;
        accelInfo->accelerationY = scalefactor * accData.iAxisY;
        accelInfo->accelerationZ = scalefactor * accData.iAxisZ;
        accelInfo->timeStamp = GetElapsedTime(accData.iTimeStamp.Int64()/1000);
        
        DEBUGPRINT("%d\n", "Acceloerometer data recived...");
        DEBUGPRINT("x=%f\n, y=%f, z=%f", accelInfo->accelerationX, accelInfo->accelerationY, accelInfo->accelerationZ);
        DEBUGPRINT("timestamp=%ld\n", accelInfo->timeStamp);

        /*Since Flash Player dont have device write capability, Accelerometer calls this API
         * as soon as it detects the change, irrespective of UpdateInterval set.
         * So here we ensure, data is updated only if UpdateInterval has elapsed.
         */
        if (accelInfo->timeStamp - iLastUpdateTime >= iUpdateInterval || iLastUpdateTime == 0)
            {
                iLastUpdateTime = accelInfo->timeStamp;
                if(accelInfo)
                    (*iCallback)(ipUserData, kAMErrNone, accelInfo);
            }

        }
        DEBUGPRINT("%s\n", "AccelerometerImpl-DataReceived() --");
    }

void AccelerometerImpl::DataError( CSensrvChannel& aChannel, TSensrvErrorSeverity /*aError*/ ) 
    {
    DEBUGPRINT("%s\n", "AccelerometerImpl-DataError() ++");
    if (aChannel.GetChannelInfo().iChannelType
            == KSensrvChannelTypeIdAccelerometerXYZAxisData)
        {
       (*iCallback)(NULL, kAMErrAlreadyCreated, NULL);
        }
    DEBUGPRINT("%s\n", "AccelerometerImpl-DataError() --");
    }

void AccelerometerImpl::GetDataListenerInterfaceL( TUid /*aInterfaceUid*/, TAny*& /*aInterface*/) 
    {    
    DEBUGPRINT("%s\n", "AccelerometerImpl-GetDataListenerInterfaceL() ++");
    DEBUGPRINT("%s\n", "AccelerometerImpl-GetDataListenerInterfaceL() --");
    }


void AccelerometerImpl::PlayerStateChanged(TPlayerState2 aPlayerState)
    {
        if(!iChannel)
            return;

        switch(aPlayerState){
            case ePlayerPause:
                iChannel->StopDataListening();
                break;
            case ePlayerResume:
			if(iCancel->IsPlayerListening())
            	TRAP_IGNORE(
                	iChannel->StartDataListeningL(this,                                                             1, //KDesiredCount, 
                    	                          1, //KMaxCount, 
                        	                      0//(int)timer
                            	                 )
                       );
                                              
                break;
            default:
                break;
        }
    }


AccelerometerCancel::~AccelerometerCancel()
                {
                    DEBUGPRINT("AccelerometerCancel::~AccelerometerCancel");
                }

U32 AccelerometerCancel::AddRef(IAMAccelerometerStart* /*pAccelStart*/)
    {
    //AccelerometerCancel* cancel = (AccelerometerCancel*)pAccelStart;
    //cancel->
	return 0;
    }

U32 AccelerometerCancel::Release(IAMAccelerometerStart* pAccelStart)
    {
    AccelerometerCancel* cancel = (AccelerometerCancel*) pAccelStart;
    cancel->Release();
    return 0;
    }

AccelerometerCancel::AccelerometerCancel(AccelerometerImpl& aImpl, IAMAllocator* allocator) :
    iImpl(aImpl),
    m_pAllocator(allocator),
    iPlayerListening(ETrue)
    {
    }

TBool AccelerometerCancel::IsPlayerListening()
{
    return iPlayerListening;
    }
	
void AccelerometerCancel::Init()
    {
    DEBUGPRINT("%s\n", "AccelerometerCancel-Init() ++");
   

    m_pFncs = NULL;

    //! create the vtbl
    m_pFncs = (AMFNCS(IAMAccelerometerStart)*) AMC_MALLOC(m_pAllocator,
            sizeof(AMFNCS(IAMAccelerometerStart)));
    if (!m_pFncs)
        {
		goto exit_gracefully;
        }

    //Set up function pointers
    m_pFncs->AddRef = AccelerometerCancel::AddRef;
    m_pFncs->Release = AccelerometerCancel::Release;

    DEBUGPRINT("%s\n", "AccelerometerCancel-Init() --");
    exit_gracefully: return;
    }

void AccelerometerCancel::AddRef()
    {
    }
void AccelerometerCancel::Release()
    {
    if (iImpl.iChannel)
        {
        iImpl.iChannel->StopDataListening();
        iPlayerListening = EFalse;
        }
    }

