/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 keep backlight ON.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"

#include "symbian_bklighthandler.h"

const TInt  KBacklightExtensionTime = 2*1000*1000;   // 2 Sec
const TInt  KBacklightExtensionInterval = 2;

CBackLightHandler::CBackLightHandler():
					 iBacklightExtensionDelay(0)
					,iBacklightExtensionTimer(NULL)
{
	
}

CBackLightHandler* CBackLightHandler::NewL()
{
	CBackLightHandler* self=new(ELeave) CBackLightHandler();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

void CBackLightHandler::ConstructL()
{
}


CBackLightHandler::~CBackLightHandler()
{
	DeleteTimer();
}

void CBackLightHandler::DeleteTimer()
{
	if(iBacklightExtensionTimer)
	{
		iBacklightExtensionTimer->Cancel();
    	delete iBacklightExtensionTimer;
    	iBacklightExtensionTimer = NULL;
	}
}

/*********************************************************************************
  Function: ExtendBacklight()
  
  Description: Called by CPlatformFlashEngine::ExtendBacklight, to extend the
                  backlight duration. It creats Timer object(if it is not already 
                  existing) and activate after 2sec. It resets the user Inactivity
                  time to the original phone settings. 
  
  Input:   TUint16 aDuration -time(in Secs), by which backlight has to be extended
  
  Output:  None
  
  Return:  Errorcode  - returns 1 if success
                      - returns o if failed
  
  History:  This function is changed for iSource #[#13052] DCTS -FL4.0
 ********************************************************************************/
TInt CBackLightHandler::ExtendBacklight(TUint16 aDuration)
{
	// If the timer is running just add new extended value to the counter
	// couple of cases like active idle and startup image starts flash
	// during startup, during this time user has not set the clock
	// in this case using UniversalTime can create issue.
			
    if (iBacklightExtensionTimer)
    { 
        // backlight extension  timer is already running
        // add new duration to existing duration
        iBacklightExtensionDelay += aDuration;
    }
    else
    {
        // create a timer that will switch off backlight extensions after aDuration seconds
        iBacklightExtensionDelay = aDuration;
       
        TRAPD(err, iBacklightExtensionTimer = CFlhTimerObject::NewL(
                                        CActive::EPriorityStandard, *this));
        
        if(err != KErrNone)
        {
            iBacklightExtensionTimer = NULL;
            iBacklightExtensionDelay = 0;
            return 0;
        }
        
        // if already existing (hypothetical case this should be possible)
        iBacklightExtensionTimer->Cancel();
        iBacklightExtensionTimer->After(KBacklightExtensionTime);
                    
        // Reset the User inactivity time for the 1st time, so that
        // the backlight delay starts from this time.
        User::ResetInactivityTime();    
    }
    
	return 1;
}

/*********************************************************************************
  Function: DoRunL()
  
  Description: This is a callback function, called by Timer. It is called in 2Sec,
                   periodically. Its also activate itself to be called after 2Sec
                   Delay is reduced by 2 after every 2 secs.
  
  Input:  CActive* aActive  - this*
  
  Output:  None
  
  Return:  None
  
  History:  This function is added for iSource #[#13052] DCTS -FL4.0
 ********************************************************************************/
void CBackLightHandler::DoRunL(const CActive* aActive)
{
    if(aActive == iBacklightExtensionTimer )
    {   
        // reduce by 2 (KBacklightExtensionInterval)
        iBacklightExtensionDelay -= KBacklightExtensionInterval;
        
        if(iBacklightExtensionDelay > 0)
        { 
            // if delay is still +ve, reset the user inactivity time
            // Activate the timer again(to be called in 2Secs)
            User::ResetInactivityTime();
            iBacklightExtensionTimer->After(KBacklightExtensionTime);
        }
        else
        {
            // If delay is 0 or -ve, timer is no more required, delete.
            DeleteTimer();
            iBacklightExtensionDelay = 0;
        }
    }
    
    return; 
}

/*********************************************************************************
  Function: DoRunError()
  
  Description:  This is a callback function, called by Timer. This is called when
                  some error occurs.
                  
  Input:  CActive* aActive
          TInt aError
          
  Output:  None
  
  Return:  Error code
  
  History:  This function is added for iSource #[#13052] DCTS -FL4.0
 ********************************************************************************/
TInt CBackLightHandler :: DoRunError(const CActive* /*aActive*/, TInt aError)
{
    return aError;
}

