/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 support the SI_GetImageInfo() and SI_DecodeImage() Flash Lite 
*				 API functions.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"

#include "flashliteinterfaces.h"

#include "symbian_engine.h"
#include "symbian_image.h"
#include <eikenv.h>
#include <bautils.h>

#include "engine_data_caging_literals.hrh"
_LIT8(KLitImageDownScaling,"_IMAGEPROXY_");

//-------------------------------------------------------------------------------------------------
CImageConverter::CImageConverter(CPlatformFlashEngine& aEngine) 
: CActive(CActive::EPriorityStandard),
 iEngine(aEngine),
 iImageRequestList(NULL),
 iCurrentImageRequest(NULL),
 iDecoder(NULL),
 iBitmap(NULL),
 iMaskBitmap(NULL)
{
	
	TUid identity = User::Identity();
	TUidName uidName = identity.Name();
	//Deleting '[' from the Uid name.
	uidName.Delete(0,1);
	//Deleting ']' from the Uid name.
	uidName.Delete(uidName.Length()-1,1);
   	iTempImgFile.Copy(KPersistentSubdirName);
	iTempImgFile.Append(uidName);
	iTempImgFile.Append(KDoubleSlash);
	iTempImgFile.Append(KImgFile);
}

//-------------------------------------------------------------------------------------------------
CImageConverter::~CImageConverter()
{
	Cancel(); //cancel existing requests
	
	Reset();

	delete iImageRequestList;
}

void CImageConverter::Reset()
{
	iImageRequestList->ResetAndDestroy();
}

CImageConverter* CImageConverter::NewL(CPlatformFlashEngine& aEngine)
{
	CImageConverter* self = new (ELeave) CImageConverter(aEngine);
	
	CleanupStack::PushL(self);
	
	self->ConstructL();

	CleanupStack::Pop(self);
	
	return self;
}

//-------------------------------------------------------------------------------------------------
void CImageConverter::ConstructL()
{
	iImageRequestList = new (ELeave) CArrayPtrSeg<CImageRequest>(1);

	CActiveScheduler::Add(this);
}

TInt16 CImageConverter::CancelImageRequest(unsigned long aImageId)
{
	TInt16 success = 0;

	//if the request to be cancelled is being decoded
	//then cancel the process
	if(iCurrentImageRequest && iCurrentImageRequest->GetImageId() == aImageId)
	{
		Cancel();
		success = 1;
	}
	else //lookup request in the list
	{
		CImageRequest* imageRequest = RemoveFromList(aImageId);
		if(imageRequest)
		{
			delete imageRequest;
			success = 1;
		}
	}

	return success;
}

CImageRequest* CImageConverter::RemoveFromList(unsigned long aImageId)
{
	//Lookup and return image request based on imageId
	TInt i;
	for(i=0;i<iImageRequestList->Count();i++)
	{
		CImageRequest* pImageRequest = iImageRequestList->At(i);
		if(pImageRequest->GetImageId() == aImageId)
		{
			iImageRequestList->Delete(i);

			return pImageRequest;
		}
	}

	return NULL;
}

CImageRequest* CImageConverter::GetNextAvailableRequest()
{
	//Return the next image request ready to be decoded
	TInt i;
	for(i=0;i<iImageRequestList->Count();i++)
	{
		CImageRequest* pImageRequest = iImageRequestList->At(i);
		if(pImageRequest->IsReadyForDecode())
		{
			iImageRequestList->Delete(i);
			return pImageRequest;
		}
	}

	return NULL;
}

//-------------------------------------------------------------------------------------------------
void CImageConverter::HandleImageInfoRequestL(unsigned long aImageId, const FI_ImageData* aData)
{
	CImageRequest* newRequest = CImageRequest::NewL(this, aImageId, aData, iEngine.GetExecutionMode(),iEngine.GetBrPluginMode());

	CleanupStack::PushL(newRequest);
	//get image information
	newRequest->ReadImageHeaderL(iEngine.GetRFs());
	iImageRequestList->AppendL(newRequest);
	CleanupStack::Pop(newRequest);
}

TBool CImageConverter::NotifyImageInfo(CImageRequest& aImageRequest, TInt aError)
{
	FI_DecodeImageInfo* decodeInfo = NULL;

	TBool success = EFalse;

	//notify the host
	if(aError == KErrNone)
	{
		decodeInfo = iEngine.GetFIObject().FI_GetDecodeImageBuffer(aImageRequest.GetImageId(), &aImageRequest.GetImageInfo()) ;

		if(decodeInfo) //image buffer returned by the player
		{
			aImageRequest.SetDecodeImageInfo(decodeInfo); //give the buffer to the CImageRequest instnace

			if(!iCurrentImageRequest) //if no outstanding image decoding request begin one
			{
				ProcessNextRequest();
			}
			success = ETrue;
		}
	}
	else
	{
		iEngine.GetFIObject().FI_NotifyImageDecodeStatus(aImageRequest.GetImageId(), (short) aError);
	}

	if(!success) //error, no buffer given by the player
	{
		//remove the request from the queue
		//the CImageRequest instance will delete itself
		RemoveFromList(aImageRequest.GetImageId());
	}

	return success;
}

void CImageConverter::ProcessNextRequest()
{
	if( iImageRequestList->Count() > 0 )
	{
		iCurrentImageRequest = GetNextAvailableRequest(); //get the next request ready to be decoded

		if(iCurrentImageRequest)
		{
			TRAPD(err, DecodeImageL(iCurrentImageRequest));

			if(err != KErrNone)
			{
				iEngine.GetFIObject().FI_NotifyImageDecodeStatus(iCurrentImageRequest->GetImageId(), KErrNone);

				//delete the image request, decoder and bitmaps
				DecodeImageCompleted();

				ProcessNextRequest(); //process the next image, if any
			}
		}
	}
}

void CImageConverter::RunL()
{
	if(iCurrentImageRequest)
	{
		//copy data to destination buffer
		CImageConverter::CopyImageDataL(iBitmap, iMaskBitmap, iCurrentImageRequest->GetDecodeImageInfo(), iCurrentImageRequest->GetImageInfo());

		unsigned long pImageId = iCurrentImageRequest->GetImageId();
		iEngine.GetFIObject().FI_NotifyImageDecodeStatus(pImageId, (short) iStatus.Int());

		//delete the image request, decoder and bitmaps
		DecodeImageCompleted();
		
		ProcessNextRequest(); //process the next image, if any
	}
	
}

TInt CImageConverter::RunError(TInt aError)
{
	return aError;
}

void CImageConverter::DoCancel()
{
	//delete any outstanding requests
	DecodeImageCompleted();
}

void CImageConverter::DecodeImageCompleted()
{
	//delete the temporary image data file
	RFs fs = iEngine.GetRFs();
	fs.Delete(iTempImgFile);
	

	delete iCurrentImageRequest;
	iCurrentImageRequest = NULL;

	delete iMaskBitmap;
	iMaskBitmap = NULL;

	delete iBitmap;
	iBitmap = NULL;

	delete iDecoder;
	iDecoder = NULL;
}

void CImageConverter::DecodeImageL(CImageRequest* aImageRequest)
{
	FI_ImageData& pImageData = aImageRequest->GetImageData();
	FI_ImageInfo& pImageInfo = aImageRequest->GetImageInfo();

	iBitmap = new (ELeave) CFbsBitmap();

	User::LeaveIfError(iBitmap->Create(TSize(pImageInfo.width, pImageInfo.height), EColor16M));

	//  Create the target transparent mask, if necessary
	if (pImageInfo.hasTransparency)
    {
		iMaskBitmap = new (ELeave) CFbsBitmap();

   		User::LeaveIfError(iMaskBitmap->Create(TSize(pImageInfo.width, pImageInfo.height), EGray256));
	}

	RFs fs = iEngine.GetRFs();	
	//Implementation changed ,as in case of widget filepath can also be passed as imagedata.
	//Check whether the data received in aImageRequest->GetImageData() is filepath or image data
	//accordingly create the decoder.
	
	if(aImageRequest->IsImageDownScaling() && aImageRequest->iImgDsFileName!=NULL)
		{
		TPath pFileName;
		pFileName.Copy(aImageRequest->iImgDsFileName->Des());
		iDecoder = CImageDecoder::FileNewL(fs,  pFileName, CImageDecoder::EOptionAlwaysThread);
		}
	else
		{			
		TPtrC8	dataPtr;

		dataPtr.Set((TUint8*)pImageData.imageData, pImageData.imageDataLength);
		RFile file;
     
        /* HIIN-7ZTDEY: When core image is created, private directory of FL is not created in 'C:' drive.         
         * Hence creation of the temporary file "img.dat" fails.  Due to which image decode operation does
         * not happen.   Now, we have ensured the path exists.
         */
        BaflUtils::EnsurePathExistsL(fs, iTempImgFile);
        
        TInt err = file.Create(fs, iTempImgFile, EFileWrite | EFileShareExclusive | EFileStream);
        if (err == KErrAlreadyExists)
        {	
            err = file.Replace(fs, iTempImgFile, EFileWrite | EFileShareExclusive | EFileStream);
        }
    
        if(err == KErrNone)
        {
            file.Write(dataPtr);
            file.Close();
        }
        else
        {
            User::Leave(err);
        }
	    iDecoder = CImageDecoder::FileNewL(fs,  iTempImgFile, CImageDecoder::EOptionAlwaysThread);
		
	}

	iStatus = KRequestPending;

	//Decode Image
	if (pImageInfo.hasTransparency)
    {
        iDecoder->Convert(&iStatus, *iBitmap, *iMaskBitmap, 0);
    }
    else
    {
        iDecoder->Convert(&iStatus, *iBitmap, 0);
	}
	
	SetActive();

	return;
}

/* static */
void CImageConverter::CopyImageDataL(CFbsBitmap* aBitmap, CFbsBitmap* aMaskBitmap, FI_DecodeImageInfo& aDecodeInfo, FI_ImageInfo& aImageInfo)
{
	aBitmap->LockHeapLC();

	TInt width = aImageInfo.width;
	TInt height = aImageInfo.height;

	U8* dest = (U8*)aDecodeInfo.imageBuffer;
	U8* src = (U8*)aBitmap->DataAddress();
    U8* maskSrc = (U8*) NULL;
	S32 srcRowBytes = aBitmap->ScanLineLength(width, EColor16M);
    S32 maskSrcRowBytes = 0;
	S32 destRowBytes = aDecodeInfo.scanlineBytes;


    if (aMaskBitmap)
    {
        maskSrc = (U8*)aMaskBitmap->DataAddress();
        maskSrcRowBytes = aMaskBitmap->ScanLineLength(width, EGray256);
    }
 
	if (src && dest)
	{
		for (S32 row=0;row<height;row++)
		{
			for (S32 pix=0;pix<width;pix++)
			{
				switch (aDecodeInfo.requestedFormat)
				{
					case FI_ImageFormat_RGB_555:
					{
						U16 p = (U16)(((U16)(src[pix*3+2]) & 0xF8) << 7 | ((U16)(src[pix*3+1]) & 0xF8) << 2 | ((U16)(src[pix*3]) & 0xF8) >> 3);
						dest[pix*2] = (U8)(p & 0xFF);
						dest[pix*2+1] = (U8)((p & 0xFF00) >> 8);
					}
					break;

					case FI_ImageFormat_RGB_565:
					{
						U16 p = (U16)(((U16)(src[pix*3+2]) & 0xF8) << 8 | ((U16)(src[pix*3+1]) & 0xFC) << 3 | ((U16)(src[pix*3]) & 0xF8) >> 3);
						dest[pix*2] = (U8)(p & 0xFF);
						dest[pix*2+1] = (U8)((p & 0xFF00) >> 8);
					}
					break;

					case FI_ImageFormat_ARGB_8888:
					{
	    				dest[pix*4] = src[pix*3]; // red
						dest[pix*4+1] = src[pix*3+1]; // green
						dest[pix*4+2] = src[pix*3+2]; // blue
                        dest[pix*4+3] = (U8)(aMaskBitmap ? maskSrc[pix] : 0xFF); // alpha
					}
					break;
			    }
		    }
			dest += destRowBytes;
			src += srcRowBytes;
            if (aMaskBitmap)
            {
                maskSrc += maskSrcRowBytes;
            }
		}
	}
	CleanupStack::PopAndDestroy();  // Free the Heap lock - This is the safest way to manage it.
									//           		    If leaving code is added to this fn there will
									//			   			be no chance of accidentally leaving it locked on a leave.
}

/*********************************************************************************
*********************************************************************************/

CImageRequest::CImageRequest(CImageConverter* aConverter, unsigned long aImageId, const FI_ImageData* aData, EEngineExecutionMode aExecutionMode,EBrPluginExecutionMode aBrPluginMode)
: CActive(CActive::EPriorityStandard)
, iConverter(aConverter)
, iImageId(aImageId)
, iDecodeImageInfo(NULL)
, iExecutionMode (aExecutionMode)
, iBrPluginMode (aBrPluginMode)
{
	iImageData.imageData = aData->imageData;
	iImageData.imageDataLength = aData->imageDataLength;
}

CImageRequest::~CImageRequest()
{
	Cancel();
	if(iImgDsFileName)
		{
		delete iImgDsFileName;
		iImgDsFileName = NULL;
		}
}
unsigned long CImageRequest::GetImageId()
{
	return iImageId;
}

CImageRequest* CImageRequest::NewL(CImageConverter*	aConverter,  unsigned long aImageId, const FI_ImageData* aData, EEngineExecutionMode aExecutionMode,EBrPluginExecutionMode aBrPluginMode)
{
	CImageRequest* self = new (ELeave) CImageRequest(aConverter, aImageId, aData, aExecutionMode,aBrPluginMode);

	CActiveScheduler::Add(self);

	return self;
}

//from CActive
void CImageRequest::RunL()
{
	if(!iConverter->NotifyImageInfo(*this, KErrNone))
	{
		//no buffer returned by player, delete self
		delete this;
	}
}

TInt CImageRequest::RunError(TInt aError)
{
	return aError; //should never be called as CImageRequest::RunL cannot leaves
}

void CImageRequest::DoCancel()
{
}

FI_ImageData& CImageRequest::GetImageData()
{
	return iImageData;
}

FI_ImageInfo& CImageRequest::GetImageInfo()
{
	return iImageInfo;
}

FI_DecodeImageInfo& CImageRequest::GetDecodeImageInfo()
{
	return *iDecodeImageInfo;
}

void CImageRequest::SetDecodeImageInfo(FI_DecodeImageInfo* aDecodeInfo)
{
	iDecodeImageInfo = aDecodeInfo;
}

void CImageRequest::ReadImageHeaderL(RFs& aFs)
{
	//  Copy the Flash image data into a TPtrC8 object
/*IMGDS
 * Check if Image Down Scaling has to be done. 
 */
  
    // update the FileName iImgDsFileName in all cases.
	if(iImgDsFileName)
	{
		delete iImgDsFileName;
		iImgDsFileName = NULL;
	}
    iImgDsFileName = UpdateFileNameFromImageDataL(&iImageData, aFs);
    
	if(ImageDownScalingSupported(aFs,&iImageData))
	{
		iImageDownScaleRequest = ETrue;
		// filename updated in all cases.
		//if(iExecutionMode == eStandAlone)
		//iImgDsFileName = UpdateFileNameFromImageDataL(&iImageData, aFs);
	}
	else
		iImageDownScaleRequest = EFalse;
	
	TFrameInfo frameInfo;
	CImageDecoder *decoder = NULL;
	TSize aDeviceScreenSize = CCoeEnv::Static()->ScreenDevice()->SizeInPixels();
/* In All Cases, create the decoder and find out the width and height of the Frame */
	/*Check whether the data received in aImageRequest->GetImageData() is filepath or image data
	  accordingly create the decoder.*/
	if(iImgDsFileName)
	{
		TPath pFileName;
		pFileName.Copy(iImgDsFileName->Des());
		decoder = CImageDecoder::FileNewL(aFs,pFileName,CImageDecoder::EOptionAlwaysThread);
		frameInfo = decoder->FrameInfo(0);
	}
	else
	{
		TPtrC8 pDataC8;
		pDataC8.Set((TUint8*)iImageData.imageData, iImageData.imageDataLength);
		
		//  Create the new image decoder
		decoder = CImageDecoder::DataNewL(aFs,pDataC8);
		frameInfo = decoder->FrameInfo(0);
	}
	if(decoder)
	{
		delete decoder;
		decoder = NULL;	
	}
	iImageInfo.width = (unsigned short)frameInfo.iOverallSizeInPixels.iWidth;
	iImageInfo.height = (unsigned short)frameInfo.iOverallSizeInPixels.iHeight;
	if(iImageDownScaleRequest)
	{
		if((iImageInfo.width > aDeviceScreenSize.iWidth) || (iImageInfo.height >aDeviceScreenSize.iHeight))
		{
            TInt widthFactor = (iImageInfo.width)/(aDeviceScreenSize.iWidth);
            TInt heightFactor = (iImageInfo.height)/(aDeviceScreenSize.iHeight);
            TInt reductionFactor = widthFactor > heightFactor ? widthFactor : heightFactor;
            if(!reductionFactor)
                reductionFactor = 1; //in case of reduction factor 0 we will get crash 
            iImageInfo.width = iImageInfo.width/reductionFactor;
            iImageInfo.height = iImageInfo.height/reductionFactor;
		}
	}
	iImageInfo.hasTransparency = ( frameInfo.iFlags & TFrameInfo::ETransparencyPossible ) != 0;

	//Image info retrieved
	//Notify self to be called later
	SetActive();

	iStatus = KRequestPending;
	TRequestStatus* pointer = &iStatus;

	User::RequestComplete(pointer, KErrNone);
}
TBool CImageRequest::IsImageDownScaling()
{
	return iImageDownScaleRequest;
}
TBool CImageRequest::IsReadyForDecode()
{
	//returns true, if the buffer to store the decoded image
	//has been received from the host, and the image is ready to be decoded
	//else returns false;
	return (iDecodeImageInfo != NULL);
}
HBufC* CImageRequest::UpdateFileNameFromImageDataL(FI_ImageData* aData,RFs& aFs)
{
	
	if(!aData || !aData->imageData)
	{
		return NULL;
	}	
	
	TInt lFileNameLen = aData->imageDataLength - KLitImageDownScaling.iTypeLength; //calculate file name length
	
	TPtrC8 pDataC8(reinterpret_cast<const TUint8*>(aData->imageData), aData->imageDataLength);

	TPtrC8 lFileNamePtr = pDataC8.Right(lFileNameLen);
	HBufC8* lTempFileName = lFileNamePtr.AllocLC();

	HBufC* lTempFileName16 = HBufC::NewL(lTempFileName->Des().Length());
	lTempFileName16->Des().Copy(lTempFileName->Des());
		
	CleanupStack::PopAndDestroy(lTempFileName);
	
	
	//Check if the filepath is Valid
	if(!aFs.IsValidName(lTempFileName16->Des()))
	{
		delete lTempFileName16;
		lTempFileName16 = NULL;
	}

	return lTempFileName16;
	
}
TBool CImageRequest::ImageDownScalingSupported(RFs& aFs,const FI_ImageData* aData)
{
	/* We check here if _IMAGEPROXY_ is present if yes then return 1 else return 0 */
	TPtrC8	dataPtr;
	TBuf8<50> lMimeType;
	_LIT8(KJPEGMimeType,"image/jpeg");
	dataPtr.Set((TUint8*)aData->imageData, aData->imageDataLength);
	if(	iExecutionMode == eStandAlone)
	{
		TInt pos = dataPtr.Find(KLitImageDownScaling);
		if(pos==0)
			return ETrue;
		else
			return EFalse;
	}
	else // in case of widget,brplugin and screensaver
		{
	    if(EDefaultMode == iBrPluginMode)
	        return EFalse;
	    
        if(iImgDsFileName!=NULL)
        {
            CImageDecoder::GetMimeTypeFileL(aFs,iImgDsFileName->Des(),lMimeType);
        }         
        else
            CImageDecoder::GetMimeTypeDataL(dataPtr,lMimeType);
        return ((0 != lMimeType.CompareF(KJPEGMimeType))? EFalse : ETrue);
	}
}
