/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that provides 
*				 simple interface to HTTP Client API.
*
*/


#include "platformbuild.h"
#include "platformtypes.h"
#include "saplayer_build.h"

#include "saflash_consts.h"
#include "symbian_engine.h"
#include "symbian_http.h"
#include "symbian_file.h"
#include "symbian_ui_interface.h"
#include "symbian_urlrequestinfo.h"
#include "symbian_localhttp.h"

const TInt KRTMPTCloseTimeout=1*1000*1000;  // 1 second

CLocalHttpEngine::CLocalHttpEngine()
: CActive(EPriorityStandard)
, iRequestQueue( _FOFF(CHttpRequestInfo,iLinkItem))	
, iPreparingToExit(EInitial)
{
}

CLocalHttpEngine::~CLocalHttpEngine()
{
	// Cancel any pending transaction and clean up items related to current transaction
	CloseAllTransactions();
	if (iSocketOpen)
	{
		iSocket.Close();
	}
	delete iSocketReader;
	delete iSocketWriter;
}

unsigned short CLocalHttpEngine::StartLocalRequestL(CHttpRequestInfo* aInfo)
{
	iRequestQueue.AddLast(*aInfo);

	TryQueueNextRequest();

	return 1;
}

void CLocalHttpEngine::TryQueueNextRequest()
{
	if(!iCurrentRequest)
	{
		//don't want to display multiple errors in a row...
		CPlatformFlashEngine* pEngine=NULL;
		
		// keep firing off requests until one successfully starts.
		while(!iRequestQueue.IsEmpty() && !iCurrentRequest)
		{
			// get first in line for running
			CLocalUrlRequestInfo* head=(CLocalUrlRequestInfo*)iRequestQueue.First();
			iRequestQueue.Remove(*head);
			TRAPD(error,DoIssueNextRequestL(*head));

			if(error)
			{
				head->HandleRequestFailed(error,KFlashLiteHttpClientSideCancelErr);
				//iTransStat=EFlhNReqFailedError; // could not even start the request...
				pEngine= head->MM_Object()->pHostObj->iFIInterface->GetEngine();
				delete head;
			}
			else
			{
				iCurrentRequest=head;
				//iTransStat=EFlhNReqNoDataSentRecvd; // HTTP stack is now resolving the URL etc...
				if(iCurrentRequest->IsRTMPTClose() && iPreparingToExit == EPreparingToExit)
				{
					TRAP_IGNORE(DoSynchronousRTMPTCloseL());
				}
			}
		}
		
		if(pEngine) // will only be set if something fails...
		{
			//pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
            pEngine->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
		}
	}
}

void CLocalHttpEngine::ResetEngine()
{
	CloseAllTransactions();
	iPreparingToExit = EInitial;
}

void CLocalHttpEngine::CloseAllTransactions()
{
    TSglQue<CHttpRequestInfo> rtmptRequestQueue;
	
    //Move rtmp reqs to separate local array and close non-rtmpt requests in the queue.
    //close active transaction. if rtmpt close synchronously with ack
    //for each rtmp req, close transaction synchronously.
    
    TInt nRequestQueue = 0;
    TInt nRtmptRequestQueue = 0;
    
    CHttpRequestInfo* info;
	while(!iRequestQueue.IsEmpty())
	{
		nRequestQueue++;
		
		info=iRequestQueue.First();
		iRequestQueue.Remove(*info);

		if(info->IsRTMPTClose())
		{
			rtmptRequestQueue.AddLast(*info);
			nRtmptRequestQueue++;
		}
		else    // if its not an RTMTP close, inform the player that the request failed.
		{
			info->HandleRequestFailed(KErrCancel,KFlashLiteHttpClientSideCancelErr);
			delete info;
			info = NULL;		
		}
	}

   	CancelActiveTransaction();
    
    // synchronously send the rtmpt close requests
    while(!rtmptRequestQueue.IsEmpty())
    {
        info=rtmptRequestQueue.First();
        rtmptRequestQueue.Remove(*info);
        
        iRequestQueue.AddLast(*info);
        TryQueueNextRequest();        
    }
    
}

//const TInt KMaxWrite = 2048;

// ----------------------------------------------------------------------------
// CHttpEngine::ConstructL()
//
// Second phase construction.
// ----------------------------------------------------------------------------
void CLocalHttpEngine::ConstructL()
{
	CActiveScheduler::Add(this); 
}

void CLocalHttpEngine::DoIssueNextRequestL(CLocalUrlRequestInfo& aInfo)
{
	iHttpStage = STAGE_NONE;
	iConnStat = ENotConnected;

	aInfo.PrepareToBeServicedL();

	// Open channel to Socket Server
	User::LeaveIfError(iSocketServ.Connect());

    // Create socket read and write active objects
	iSocketReader = CFlashSocketReader::NewL(*this, iSocket/*, KMaxWrite*/);
	iSocketWriter = CFlashSocketWriter::NewL(*this, iSocket);

	iSizeExpected = 0;
	iSizeRead = 0;

	iCurrentRequest = &aInfo;
	ConnectL(aInfo);
}

void CLocalHttpEngine::CancelActiveTransaction()
{
	if(!iCurrentRequest)
		return;
	
	if(iCurrentRequest->IsRTMPTClose() && (iPreparingToExit == EPreparingToExit))
    {
        TRAP_IGNORE(DoSynchronousRTMPTCloseL());
        //At this point, iCurrentRequest would already be deleted.
    }
	else
	{
		if (iSocketReader)
		{
			iSocketReader->Cancel();
		}
		if (iSocketWriter)
		{
			iSocketWriter->Cancel();
		}
	
		if(!iCurrentRequest)
		{
			return;
		}
		
		TRAP_IGNORE(RequestCompletedL(EFalse));
	}
}

void CLocalHttpEngine::DoCancel()
{
	if (iConnStat == EConnecting)
	{
		iSocket.CancelConnect();
		iSocket.Close();
		iSocketOpen = EFalse;
	}
}

void CLocalHttpEngine::RunL()
{
	if (iConnStat == EConnecting)
	{
		if (iStatus == KErrNone)
		{ // Connection completed successfully
			iConnStat = EConnected;
			IssueRequestL(*iCurrentRequest);
		}
		else
		{
			iSocket.Close();
			iSocketOpen = EFalse;

			if (iCurrentRequest)
			{
				TBool isRtmptClose = iCurrentRequest->IsRTMPTClose();
				
				CPlatformFlashEngine* pEngine = iCurrentRequest->MM_Object()->pHostObj->iFIInterface->GetEngine();
				if (pEngine)
				{
					//pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedConnect);
                    pEngine->NotifyDataAccessError(NULL, ECompleteFailedConnect);
				}

				RequestCompletedL(EFalse);

				if(isRtmptClose)
				{
					RTMPTCloseCompleted();
				}
			}
		}
	}
}

void CLocalHttpEngine::ConnectL(CLocalUrlRequestInfo& aInfo)
{
	TInt err = iSocket.Open(iSocketServ, KAfInet, KSockStream, KProtocolInetTcp);
	if (err == KErrNone)
	{
		iSocketOpen = ETrue;

		// Set up address information
		iAddress.SetAddress(INET_ADDR(127,0,0,1));
#ifdef __WINS__
		_LIT(KIAddress,"66.218.77.68");
		iAddress.Input(KIAddress);
#endif
		TUint16 port = aInfo.Port();
		iAddress.SetPort(port);

		iConnStat = EConnecting;
		// Initiate socket connection
		iSocket.Connect(iAddress, iStatus);
        
		SetActive();
	}
}

_LIT8(KHeaderPost, "POST");
_LIT8(KHeaderGet, "GET");
_LIT8(KHeaderSpace, " ");
_LIT8(KHeaderHttp, " HTTP/1.1\r\n");
_LIT8(KHeaderHost, "Host: ");
_LIT8(KHeaderColon, ":");
_LIT8(KHeaderEnd, "\r\nConnection: close\r\n");
_LIT8(KHeaderPostContent, "Content-type: application/x-www-form-urlencoded\r\nContent-length: ");
_LIT8(KHeaderLineEnd, "\r\n");

void CLocalHttpEngine::IssueRequestL(CLocalUrlRequestInfo& aInfo)
{
	if (iConnStat != EConnected)
	{
		return;
	}

	iHttpStage = STAGE_QUERYHEADER;

	HBufC8* sendHeaderBuf = HBufC8::NewLC(1024);
	TPtr8 sendHeader = sendHeaderBuf->Des();

	TBool fPost = EFalse;

	if (aInfo.PostData())
	{
		sendHeader.Copy(KHeaderPost);
		fPost = ETrue;
	}
	else
	{
		sendHeader.Copy(KHeaderGet);
	}
	
	//Prepare the header
	
	sendHeader.Append(KHeaderSpace);
	sendHeader.Append(aInfo.Path());
	sendHeader.Append(KHeaderHttp);
	sendHeader.Append(KHeaderHost);
	sendHeader.Append(aInfo.HostName());
	if (aInfo.Port() != 80)
	{
		sendHeader.Append(KHeaderColon);
		sendHeader.AppendNum((TInt)aInfo.Port());
	}
	sendHeader.Append(KHeaderEnd);

	// POST needs some addtional header fields
	if ( fPost ) // some addtional header fields
	{
		sendHeader.Append(KHeaderPostContent);
		sendHeader.AppendNum(aInfo.PostData()->Length());
		sendHeader.Append(KHeaderLineEnd);
	}

	sendHeader.Append(KHeaderLineEnd);


	//send header data
	iSocketWriter->IssueWriteL(sendHeader);

	CleanupStack::PopAndDestroy(sendHeaderBuf);

	if (fPost)
	{
		iSocketWriter->IssueWriteL(*aInfo.PostData());
	}

}

CLocalHttpEngine* CLocalHttpEngine::NewL()
{
	CLocalHttpEngine* self = new (ELeave) CLocalHttpEngine();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
    return self;
}

TBool CLocalHttpEngine::ProcessDataL(const TDesC8& aDataBuffer)
{
	TBool result = ETrue;
	if (iChunkedTransfer)
	{
		// not supported!
		result = EFalse;
	}
	else
	{
		iSizeRead += aDataBuffer.Length();
		iCurrentRequest->HandleResponseDataL(aDataBuffer);
		if (iSizeRead >= iSizeExpected)
		{
			RequestCompletedL(ETrue);
			result = EFalse;
		}
	}
	return result;
}



_LIT8(KLit8HttpResponse, "HTTP/1.");
#define	FlashIsDigit(x)	( x >= '0' && x <= '9' )? 1 : 0

TInt CLocalHttpEngine::ScanHeader(const TDesC8& aBuffer)
{
	TInt statuscode = 0;
	if (aBuffer.Left(KLit8HttpResponse().Length()).Compare(KLit8HttpResponse) == 0) // we have a HTTP response header
	{
		// check the first line: Status Line
		const TUint8* firstLn = aBuffer.Ptr();
		// skip the http version
		while ( ( (*firstLn) != '\r' ) &&  ( (*firstLn) != ' ' ) )
		{
			firstLn ++;
		}
		// skip the whitel space
		while ( ( (*firstLn) != '\r' ) && ( (*firstLn) == ' ' ) ) 
		{
			firstLn ++;
		}

		while ( ( (*firstLn) != '\r' ) && ( FlashIsDigit(*firstLn) ) ) 
		{
			statuscode = statuscode * 10 + *firstLn - '0';	
			firstLn++;
		}
	}

	return statuscode;
}



_LIT8(KLit8TransferEncoding, "Transfer-Encoding:");
_LIT8(KLit8EndOfHeader, "\r\n\r\n");
_LIT8(KLit8ContentLength, "Content-Length");

TBool CLocalHttpEngine::ResponseReceivedL(const TDesC8& aBuffer)
{
	TInt statuscode = 0;

	TBool result = ETrue;

	if ( iHttpStage == STAGE_QUERYHEADER)
	{
		if (aBuffer.FindF(KLit8TransferEncoding) != KErrNotFound)
		{
			iChunkedTransfer = true;
		}

		statuscode = ScanHeader(aBuffer);
		if (statuscode == 100) // 100 == Continue, look for more header
		{
			TInt endPos = aBuffer.Find(KLit8EndOfHeader);
			if (endPos != KErrNotFound)
			{
				TPtrC8 tmp = aBuffer.Mid(endPos+4);
				statuscode = ScanHeader(tmp);
			}
			else
			{
				return result;
			}
		}

		if (statuscode >= 400) // error
		{
			CPlatformFlashEngine* pEngine = iCurrentRequest->MM_Object()->pHostObj->iFIInterface->GetEngine();
			//pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
			pEngine->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
			RequestCompletedL(EFalse);
			return EFalse;
		}

		TInt dataHead = aBuffer.Find(KLit8EndOfHeader);
		if ( dataHead != KErrNotFound )// double CR-LF, header ends
		{
			TInt dataBufLen = aBuffer.Length() - dataHead - KLit8EndOfHeader().Length();
				
			iHttpStage = STAGE_HEADEREND;


			FI_URLStreamHeaderInfo headerInfo;
			headerInfo.numHeaders = 0;
			headerInfo.headerNames = NULL;
			headerInfo.headerValues = NULL;
					
			TInt sizePos = aBuffer.FindF(KLit8ContentLength);
			if (sizePos != KErrNotFound)
			{
				TInt i = 0;
				TInt c = 0;
				TPtrC8 sizePtr = aBuffer.Mid(sizePos + KLit8ContentLength().Length());
				char sizeStr[40];
				while (sizePtr[i] != '\r')
				{
					if ((sizePtr[i] >= '0') && (sizePtr[i] <= '9'))
					{
						iSizeExpected = iSizeExpected * 10 + (sizePtr[i] - '0');
						sizeStr[c++] = sizePtr[i];
					}
					i++;
				}
				sizeStr[c] = '\0';

				headerInfo.numHeaders = 1;
				headerInfo.headerNames = new(ELeave) char*[1];
				headerInfo.headerValues = new(ELeave) char*[1];
				headerInfo.headerNames[0] = "Content-length"; //KLit8ContentLength().PtrZ();
				headerInfo.headerValues[0] = sizeStr;
			}
			else
			{
				iSizeExpected = 0x7FFFFFFF;
			}

			// let the request do the right thing with it
			iCurrentRequest->HandleHeaderResponseL(headerInfo, KNullDesC8);

			delete headerInfo.headerNames;
			delete headerInfo.headerValues;

			// real SWF data comes
			if ( dataBufLen > 0 )
			{
				TPtrC8 dataBuf = aBuffer.Mid(dataHead + KLit8EndOfHeader().Length());
				result = ProcessDataL(dataBuf);
			}
			else if (iSizeRead >= iSizeExpected)
			{
				RequestCompletedL(ETrue);
				result = EFalse;
			}
			iHttpStage = STAGE_QUERYDATA;
			return result;
		}
		else
		{	// the header is very long
			User::Leave(KErrOverflow);
		}
	}
	
	if (iHttpStage == STAGE_QUERYDATA )
	{
		result = ProcessDataL(aBuffer);
	}

	return result;
}

void CLocalHttpEngine::WriteCompleted()
{
	iSocketReader->Start();
}


void CLocalHttpEngine::RequestCompletedL(TBool aSuccessStatus)
{
	// if its an RTMPT close request && we are resetting, lets keep it in the queue.
	// the request will be re-issued.
	if(!(iSynchronizeRTMPTClose && iCurrentRequest->IsRTMPTClose()))
	{
		if (aSuccessStatus)
		{
			iCurrentRequest->HandleResponseCompleteL(200);
		}
		else
		{
	//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
	// sending "200" for success transaction and '0' for all other cases.

			iCurrentRequest->HandleRequestFailed(KErrCancel,0); //error'd out
		}
	}
	
	delete iSocketReader;
	iSocketReader = NULL;
	delete iSocketWriter;
	iSocketWriter = NULL;
	if (iSocketOpen)
	{
		iSocket.Close();
		iSocketOpen = EFalse;
	}
	iSocketServ.Close();
	delete iCurrentRequest;
	iCurrentRequest = NULL;
}

void CLocalHttpEngine::ReportErrorL(TErrorType /*aErrorType*/, TInt aErrorCode)
{
	if (aErrorCode == KErrEof)
	{
		HandleReadEOFL();
	}
	else
	{
		RequestCompletedL(EFalse);
	}
}

void CLocalHttpEngine::HandleReadEOFL()
{

	if (iSizeExpected == 0x7FFFFFFF)
	{ // no Content-Length field in HTTP header, so we didn't know how much to read. This is OK.
		RequestCompletedL(ETrue);
	}
	else
	{ // we didn't get as much content as we expected. Not OK.
		RequestCompletedL(EFalse);
	}
}

void CLocalHttpEngine::PrepareToExit()
{
	iPreparingToExit = EPreparingToExit;
}

void CLocalHttpEngine::DoSynchronousRTMPTCloseL()
{
    // The RTMPT "/close" command is synchronous *only* if the player is being destroyed
	// in this case there only one request in the queue which is deleted soon after it is issued.
    // So, when this point is reached, the queue *must* be empty.
	ASSERT(iRequestQueue.IsEmpty());
	
	if(!iRTMPTCloseWait)
	{
		iRTMPTCloseWait = new (ELeave) CActiveSchedulerWait();
	}
	
	ASSERT(!iRTMPTCloseWait->IsStarted());	// there is no way this flow is reached if iRTMPTCloseWait is running 
	
	if(!iRTMPTCloseWaitTimer)
	{
		TCallBack timerCallback(RTMPTTimerCallback,static_cast<TAny*>(this));
		iRTMPTCloseWaitTimer = CFlhTimerObject2::NewL(CActive::EPriorityStandard,timerCallback);
	}

	ASSERT(!iRTMPTCloseWaitTimer->IsActive());	// the timer *must* be cancelled
	
	iRTMPTCloseWaitTimer->After(KRTMPTCloseTimeout);
	iRTMPTCloseWait->Start(); // blocks this AO untill its cancelled in MHFRunL() or it times out and is cancelled in DoRunL()

}

void CLocalHttpEngine::CancelSynchronousRTMPTClose()
{
	ASSERT(iCurrentRequest->IsRTMPTClose());
	
	TRAP_IGNORE(RequestCompletedL(EFalse));
	
	// TransactionOverQueueNextL() closes the transaction and starts the new request
	// incase of a synchronous rtmpt close request (during shutdown), 
	// the queue would be empty and hence no request will be issued
	// Once the CActiveSchedulerWait is stopped and the control goes back to the core player
	// via CHttpEngine::StartHttpRequestL(), more "/close" requests may be issued
	
	RTMPTCloseCompleted();
	ASSERT(iCurrentRequest->IsRTMPTClose());
	
	CancelActiveTransaction();

	// the current RTMPT close request was cancelled.
	// continue issuing new requests, if any.
	RTMPTCloseCompleted();
}

void CLocalHttpEngine::RTMPTCloseCompleted()
{
    if(iRTMPTCloseWait && iRTMPTCloseWait->IsStarted())
        iRTMPTCloseWait->AsyncStop();
    
    if(iRTMPTCloseWaitTimer && iRTMPTCloseWaitTimer->IsActive())
        iRTMPTCloseWaitTimer->Cancel();
}

/*
 * CHttpEngine::RTMPTTimerCallback is a callback from the timer
 */

TInt CLocalHttpEngine::RTMPTTimerCallback(TAny* aArg)
{
	CLocalHttpEngine* self = static_cast<CLocalHttpEngine*>(aArg);
	if(self)
		self->CancelSynchronousRTMPTClose();
		
	return KErrNone;
}

//	static
CLocalUrlRequestInfo* CLocalUrlRequestInfo::NewL(void* aOwner, struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo, TUint aId)
{
	CLocalUrlRequestInfo* self = new(ELeave) CLocalUrlRequestInfo(aOwner, aMMObj, aId, aStreamInfo);
	CleanupStack::PushL(self);
	self->ConstructL(aStreamInfo);
	CleanupStack::Pop(self);
	return self;
}


CLocalUrlRequestInfo::CLocalUrlRequestInfo(void* aOwner, struct MM_Object* aMMObj, TUint32 aId, struct FI_URLStreamInfo* aStreamInfo)
: CUrlRequestInfo(aOwner, aMMObj, aId, aStreamInfo)
{}

void CLocalUrlRequestInfo::ConstructL(struct FI_URLStreamInfo* aStreamInfo)
{
	TPtrC8 hostPtr((TUint8*)aStreamInfo->hostName);
	iHostName = hostPtr.AllocL();
	iPort = aStreamInfo->port;
	TPtrC8 pathPtr((TUint8*)aStreamInfo->documentPath);
	iPath = pathPtr.AllocL();
	iContentLength = aStreamInfo->contentLength;
	if (aStreamInfo->contentType)
	{
		TPtrC8 contentPtr((TUint8*)aStreamInfo->contentType);
		iContentType = contentPtr.AllocL();
	}
	if (aStreamInfo->postData)
	{
		TPtrC8 postPtr((TUint8*)aStreamInfo->postData);
		iPostData = postPtr.AllocL();
	}
}

CLocalUrlRequestInfo::~CLocalUrlRequestInfo()
{
	delete iHostName;
	delete iPath;
	delete iContentType;
}
