/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the classes used for 
*                shared object operations.
*
*/

#include <bautils.h>
#include <f32file.h>
#include "saplayer_build.h"
#include "symbian_mso.h"
#include "symbian_msoutils.h"
#include "symbian_engine.h"

extern void NPPDebug(TUint8 aLogLevel,TBool aIsPlugin, TRefByValue<const TDesC> aText, ... );
/*
 * CSharedObjectStorage functions
 */
CSharedObjectStorage* CSharedObjectStorage::NewL(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir)
{
   NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::NewL():++"));
   CSharedObjectStorage *self = new (ELeave)CSharedObjectStorage(aEngine);
   CleanupStack::PushL(self);
   self->ConstructL(aPersistentDir);
   CleanupStack::Pop(self);
   NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::NewL():--"));
   return self;
}

CSharedObjectStorage::CSharedObjectStorage(CPlatformFlashEngine& aEngine)
    :iEngine(aEngine)
{

}

void CSharedObjectStorage::ConstructL(const TDesC& aPersistentDir)
{
    iShObjUtils = CSharedObjectUtils::NewL(iEngine, aPersistentDir);
}

CSharedObjectStorage::~CSharedObjectStorage()
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::~CSharedObjectStorage():++"));
    if(iShObjUtils)
    {
        delete iShObjUtils;
        iShObjUtils = NULL;
    }
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::~CSharedObjectStorage():--"));
}

TInt CSharedObjectStorage::StoreL(const char* rootMovieGUID, const char* name, const unsigned char* data, unsigned long dataSize)
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::Store():++"));
    ASSERT(data!=NULL);    

    TInt err(KErrNotFound); //any better error code??
    HBufC* msoFileName = iShObjUtils->GetSharedObjectFileNameL(rootMovieGUID, name);
    
    CleanupStack::PushL(msoFileName); 
    if (msoFileName)
    {
        if (iShObjUtils->IsDataSizeExceeded(rootMovieGUID, *msoFileName, dataSize))        
    	{
    		/* Data size exceeds the total MSO size per movie
    		 * Hence, return failure to core player.
    		 */
            CleanupStack::PopAndDestroy(msoFileName);
            NPPDebug(LOG_ERROR, EFalse, _L("CSharedObjectStorage::Store(): DataSizeExceeded"));
    		return KErrNoMemory;
    	}
        
        /* Check if we have enough space to store the 
         * MSO. Storage limit for MSO is 4 MB. If no space 
         * delete the LRU MSO.
         */
        if(!iShObjUtils->HasEnoughSpace(dataSize))
        {
        	iShObjUtils->StartMSOCleanup(dataSize);        	
        }

        RFile file;
        TPtr16 ptrToFile(msoFileName->Des());

        err = file.Create(iEngine.GetRFs(), ptrToFile, EFileWrite | EFileShareExclusive);

        if (KErrAlreadyExists == err) //file exists already.
            err = file.Open(iEngine.GetRFs(), ptrToFile, EFileWrite | EFileShareExclusive);

        if (KErrNone == err)
            {
     		TPtr8 dataPtr(const_cast<TUint8*>(data), dataSize, dataSize);
            err = file.Write(dataPtr);
            file.Close();
            }
        CleanupStack::PopAndDestroy(msoFileName);
    }

    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::Store():--"));
    return err;
}
    
TInt CSharedObjectStorage::Read(const char* rootMovieGUID, const char* name, unsigned char* data, unsigned long dataSize)
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::Read():++"));
    TInt err(KErrNotFound); //any better error code??
    HBufC* msoFileName = NULL;
    
    TRAP(err, msoFileName = iShObjUtils->GetSharedObjectFileNameL(rootMovieGUID, name));        
    
    if (KErrNone != err)
    {
        NPPDebug(LOG_ERROR, EFalse, _L("CSharedObjectStorage::Read():ERR: %d"), err);
        if(msoFileName)
            delete msoFileName;
        return err;
    }
    
    if (msoFileName)
        {
        RFile file;
        err = file.Open(iEngine.GetRFs(), msoFileName->Des(), EFileRead);
        if (KErrNone == err)
            {
            TPtr8 dataPtr((unsigned char*) data, dataSize);
            err = file.Read(dataPtr, dataSize);
            }

        file.Close();
        delete msoFileName;
        }

    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::Read():--"));
    return err;
}
    
TInt CSharedObjectStorage::GetStoreSpace(const char* rootMovieGUID, const char* name, TInt& aSize)
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::GetStoreSpace():++"));
    aSize = 0;
    TInt err(KErrNotFound); //any better error code??
    HBufC* msoFileName = NULL;
    
    TRAP(err, msoFileName = iShObjUtils->GetSharedObjectFileNameL(rootMovieGUID, name));

    if (KErrNone != err)
    {
        NPPDebug(LOG_ERROR, EFalse, _L("CSharedObjectStorage::GetStoreSpace(): ERR:%d"), err);
        if(msoFileName)
            delete msoFileName;
        return err;
    }
    if (msoFileName)
        {
        RFile file;
        
        err = file.Open(iEngine.GetRFs(), msoFileName->Des(), EFileRead /*| EFileShareExclusive*/); //EFileShareExclusive not reqd for reading oper'n

        if ((err == KErrNotFound) || (err == KErrPathNotFound))
        {
            // the new file does not exist
            // The directory where MSO files are stored are different in FL 3.0 and FL 3.1
            // If the mso file was created using FL 3.0, then we have to migrate the file to FL 3.1 location

            // the complete filepath of the mso is stored in iName, which is initialized during object creation 
            // extract the new directory from iName
            TPtrC newMSODir = msoFileName->Left(msoFileName->LocateReverse(KPathDelimiter) + 1);

            // check if the MSO file exists in the old location.
            // if it exists, then move to the new location
            //error = iShObjUtils.CheckAndMigrateToNewLocation(iMMObj->pHostObj->iFIInterface,*iMSOName,newMSODir);
            TRAP(err,iShObjUtils->MigrateMSODirectoryL(*msoFileName,newMSODir));
            if (err == KErrNone)
                {
                // successfully migrated to the new location
                // try to open the file again...
                err = file.Open(iEngine.GetRFs(), msoFileName->Des(),
                        EFileRead /* |EFileShareExclusive */); //EFileShareExclusive not reqd for reading oper'n
                }
        }

        if (err == KErrNone)
            {
            err = file.Size(aSize);            
            }

        delete msoFileName;
        file.Close();
        }
    
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::GetStoreSpace():--"));
    return err;
}

TInt CSharedObjectStorage::RemoveDataL(const char* aNameSpace, const char* aName)
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::RemoveData():++"));
    HBufC* msoFileName = iShObjUtils->GetSharedObjectFileNameL(aNameSpace, aName);
    HBufC* msoNameSpace = iShObjUtils->GetPersistenceDirL(aNameSpace);

    CleanupStack::PushL(msoFileName);
    
    if (msoNameSpace && msoFileName)
    {
        CFileMan* fm = NULL;
        TRAPD(err, fm = CFileMan::NewL(iEngine.GetRFs()));

        if (fm && KErrNone == err)
            {
                fm->Delete(*msoFileName); // Delete MSO file
                
                if (IsEmptyDir(msoNameSpace)) // Delete the directory if it is empty.[RC ou1cimx1#708244 ]
                    fm->RmDir(*msoNameSpace); 
            }
    
        delete fm;
        
        CleanupStack::PopAndDestroy(msoFileName);
    }
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectStorage::RemoveData():--"));
    return KErrNone;
}



/*
 * CSharedObjectHandler functions
 */
CSharedObjectHandler* CSharedObjectHandler::NewL(CPlatformFlashEngine& aEngine,const TDesC& aPersistentDir)
{
    CSharedObjectHandler* self = new (ELeave) CSharedObjectHandler(aEngine);
    CleanupStack::PushL(self);
    self->ConstructL(aPersistentDir);
    CleanupStack::Pop();
    return self;
}

CSharedObjectHandler::CSharedObjectHandler(CPlatformFlashEngine& aEngine):iEngine(aEngine)
{
}

void CSharedObjectHandler::ConstructL(const TDesC& aPersistentDir)
{
    if(iPersistentDir)
    {
        delete iPersistentDir;
        iPersistentDir = NULL;
    }
    
    iPersistentDir = aPersistentDir.AllocL();
    
    iShObjStorage = CSharedObjectStorage::NewL(iEngine, *iPersistentDir);
}

CSharedObjectHandler::~CSharedObjectHandler()
{
    if(iPersistentDir)
    {
        delete iPersistentDir;
        iPersistentDir = NULL;
    }
    
    if(iShObjStorage)
    {
        delete iShObjStorage;
        iShObjStorage = NULL;
    }
}

TInt CSharedObjectHandler::StorageDataSize(const char* rootMovieGUID, const char* name, unsigned long& aSize)
{    
    TInt iSize(0);
    TInt err = iShObjStorage->GetStoreSpace(rootMovieGUID,name, iSize);
    aSize = (unsigned long)iSize;
    return err;
}

TInt CSharedObjectHandler::StoreData(const char* rootMovieGUID,
                                     const char* name,
                                     const unsigned char* data,
                                     unsigned long dataSize
                                     )
{
    TRAPD(err, iShObjStorage->StoreL(rootMovieGUID,name,data,dataSize));
    return err;
}

TInt CSharedObjectHandler::RetrieveData(const char* rootMovieGUID,
                                        const char* name,
                                        unsigned char* data,
                                        unsigned long dataSize)
{
    return iShObjStorage->Read(rootMovieGUID,name,data,dataSize);        
}

TInt CSharedObjectHandler::RemoveData(const char* rootMovieGUID,
                                      const char* name)
{
    TRAPD(err, iShObjStorage->RemoveDataL(rootMovieGUID,name));
    return err;
}
// This function checks whether the specified directory is empty or not
// If empty return true, and false if not.
TBool CSharedObjectStorage::IsEmptyDir(HBufC* msoNameSpace)
{
    TBool result = EFalse ;
    RFs fSession = iEngine.GetRFs();

    CDir* dirList;
    CDir* fileList;
    
    TRAPD(err, fSession.GetDir(*msoNameSpace,KEntryAttNormal,ESortByName,fileList,dirList)); // If error , directory can't be deleted, hence reutrn "false"
    if(err == KErrNone)
    {
        if(!fileList->Count()) // If the file count is zero, the directory is empty.
            result = ETrue;
    }
    return result;
}
