/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Contains shared object utility functions
*
*/

#include <e32std.h>
#include <bautils.h>
#include <utf.h>
#include <f32file.h>
#include "saplayer_build.h"
#include "symbian_msoutils.h"


const int KPathDelimiterCharSize = 2; //size of "\\"
#define KRootMovieGUIDLen 32

extern void NPPDebug(TUint8 aLogLevel,TBool aIsPlugin, TRefByValue<const TDesC> aText, ... );

/*
 * CFileNode functions
 */

CFileNode* CFileNode::NewL(TEntry &aFileEntry, HBufC* &aFilePath)
{
    CFileNode* node = new (ELeave) CFileNode(aFileEntry);
    CleanupStack::PushL(node);
    node->ConstructL(aFilePath);
    CleanupStack::Pop(node);
    return node;
}


CFileNode::CFileNode(TEntry &aFileEntry) : 
    iFileEntry(aFileEntry)
{
}

void CFileNode::ConstructL(HBufC *&aFilePath)
{
    if (aFilePath)
    {
        iFilePath = aFilePath->AllocL();
    }
}

CFileNode::~CFileNode() 
{
    if (iFilePath)
        delete iFilePath;
}

/*
 * CSharedObjectUtils functions
 */
CSharedObjectUtils* CSharedObjectUtils::NewL(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir)
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::NewL():++"));
    CSharedObjectUtils *self = new (ELeave)CSharedObjectUtils(aEngine, aPersistentDir);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);   
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::NewL():--"));
    return self;
}

CSharedObjectUtils::CSharedObjectUtils(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir)
    : iEngine(aEngine), iPersistentDir(aPersistentDir), iAsyncObject(NULL)
{    
    iCurrentMSOSize         = 0;
    iSizeToCleanup          = 0;
    iDeletedSize            = 0;
    
    iFs = iEngine.GetRFs();
}

void CSharedObjectUtils::ConstructL()
{    
    iAsyncObject = CFlhAsyncObject::NewL(CActive::EPriorityIdle,*this);
    iFileManager = CFileMan::NewL(iFs);       
}

CSharedObjectUtils::~CSharedObjectUtils()
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::CSharedObjectUtils():++"));
    if (iAsyncObject)
    {
        delete iAsyncObject;
        iAsyncObject = NULL;
    }
    if (iFileManager)
    {
        delete iFileManager;
        iFileManager = NULL;
    }
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::~CSharedObjectUtils():--"));
}

HBufC* CSharedObjectUtils::GetSharedObjectFileNameL(const char* aNameSpace, const char* aName)
{
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::GetSharedObjectFileNameL():++"));
    TPtrC8 namePtr((const TUint8*)aName, KMaxFileName);
    TBuf16<KMaxFileName> unicodeName;    
    HBufC* file = NULL;
    
    if (CnvUtfConverter::ConvertToUnicodeFromUtf8(unicodeName, namePtr) >= 0)
    {
        TRAPD(err,file = GetPersistenceDirL(aNameSpace));
        if (file && KErrNone == err)
        {
            BaflUtils::EnsurePathExistsL(iFs,file->Des()); //ensure the path exists..
            //file = file->ReAllocL(file->Length() + unicodeName.Length()); 
            file = file->ReAllocL(file->Length() + KRootMovieGUIDLen + 1); //ZT         
            file->Des().Append(unicodeName.Left(KRootMovieGUIDLen));
            file->Des().ZeroTerminate();
        }       
    }
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::GetSharedObjectFileNameL():--"));
    return file;
}


HBufC* CSharedObjectUtils::GetPersistenceDirL(const char* aNameSpace) const 
{
    HBufC* directory = NULL;
    
    if (aNameSpace)
    {        
        directory = HBufC::NewL(iPersistentDir.Length() + KPathDelimiterCharSize);
        
        directory->Des().Append(iPersistentDir);
        directory->Des().Append(KPathDelimiter);
        
        TPtrC8 namePtr((const TUint8*) aNameSpace);
        TBuf16<KMaxFileName> url;
        if (CnvUtfConverter::ConvertToUnicodeFromUtf8(url, namePtr) >= 0)
        {
            directory = directory->ReAllocL(directory->Length() + url.Length() + KPathDelimiterCharSize);
            directory->Des().Append(url);
            directory->Des().Append(KPathDelimiter);
        }
    }

    return directory;
}


void CSharedObjectUtils::MigrateMSODirectoryL(const TDesC& aMSOFileName, const TDesC& aNewMSODirectory)
{
    // old folder : C:\private\UID\SwfGUID\MSOFileName
    // new folder : 
    // widget mode : drive:\private\UID\WidgetName\SwfGUID\MSOFileName
    // other modes : C:\private\UID\SwfGUID\MSOFileName
    
    
    // got to find out where the mso files were stored in fl 3.0
    
    // get the directory
    
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::MigrateMSODirectoryL():++"));
    HBufC* oldMSOFile = NULL;
    oldMSOFile = GetFlash8MSOFileLC(aMSOFileName);
    MoveMSOFileToFlash9LocationL(*oldMSOFile,aNewMSODirectory);
    CleanupStack::PopAndDestroy(oldMSOFile);
    NPPDebug( LOG_DEBUG, EFalse, _L("CSharedObjectUtils::MigrateMSODirectoryL():--"));
}

HBufC* CSharedObjectUtils::GetFlash8MSOFileLC(const TDesC& aMSOFileName)
{
    HBufC* oldMSOFilePath = NULL;
    oldMSOFilePath = GetFlash8PersistentDirectoryLC();
    
    // append the mso name to construct the path
    oldMSOFilePath->Des().Append(KPathDelimiter);
    TPtrC newMSOFileName = aMSOFileName.Right((aMSOFileName.Length() - aMSOFileName.LocateReverse(KPathDelimiter)) - 1);
    oldMSOFilePath->Des().Append(newMSOFileName);
    
    TBool fileExists = BaflUtils::FileExists(iFs,*oldMSOFilePath);
    if(!fileExists)
    {
        User::Leave(KErrNotFound);
    }
    
    return oldMSOFilePath;
}

void CSharedObjectUtils::MoveMSOFileToFlash9LocationL(const TDesC& aOldFile, const TDesC& aNewMSODirectory)
{
    TInt retVal = iFs.MkDirAll(aNewMSODirectory);
    if(retVal == KErrNone || retVal == KErrAlreadyExists)
    {
        retVal = iFileManager->Move(aOldFile, aNewMSODirectory);
    }
}

HBufC* CSharedObjectUtils::GetFlash8PersistentDirectoryLC()
{        
    HBufC* swfGUID = NULL;
    swfGUID = iEngine.GetSwfGuidLC();
    
    HBufC* oldPersistentDirectory = NULL;
    oldPersistentDirectory=HBufC::NewL(KMaxFileName);
    
    TPtr odDes = oldPersistentDirectory->Des();
    iFs.PrivatePath(odDes);
    TBuf<2> driveLetterAndDelimiter;
    TChar drive = iFs.GetSystemDriveChar();       
    driveLetterAndDelimiter.Append(drive);
    driveLetterAndDelimiter.Append(KDriveDelimiter); 
    odDes.Insert(0,driveLetterAndDelimiter); // C:
    odDes.Append(*swfGUID);

    CleanupStack::PopAndDestroy(swfGUID);
    CleanupStack::PushL(oldPersistentDirectory);
    return oldPersistentDirectory;
}

TBool CSharedObjectUtils::IsDataSizeExceeded(const char* aNameSpace,  TDesC& msoFileName, unsigned long dataSize)
{
    TInt currentMovieSize = 0;
    TInt err = KErrNone;
    TSglQue<CFileNode>* fileList = NULL;
    TRAP_IGNORE(err = GetCurrentMSOSizeL(aNameSpace, iCurrentMSOSize, fileList));
    err = GetCurrentMovieSize(msoFileName, currentMovieSize);
    if (KErrNone != err && KErrNotFound != err)
    {
        NPPDebug( LOG_ERROR, EFalse, _L("CSharedObjectUtils::IsDataSizeExceeded():ERR: %d"), err);
        return ETrue;
    }
    
    if ((iCurrentMSOSize - currentMovieSize + dataSize) > KMaxPersistentStoragePerMovie)
    {
        return ETrue;
    }
    else
    {
        return EFalse;
    }       
}

TBool CSharedObjectUtils::HasEnoughSpace(unsigned long dataSize) const
{
    TSglQue<CFileNode>* fileList = NULL;
    
    TInt totalPersistentSize = 0;
    TRAP_IGNORE(GetCurrentMSOSizeL(NULL, totalPersistentSize, fileList));
    
    if ((totalPersistentSize - iCurrentMSOSize + dataSize)  > KMaxPersistentStoragePerMode)
    {
        return EFalse;
    }
    else
    {
        return ETrue;
    }
}

TInt CSharedObjectUtils::GetCurrentMovieSize (TDesC& msoFileName, TInt& aSize) const
{
    aSize = 0;
    
    TInt err = KErrNone;
    TEntry fileEntry;   
    
    err = iFs.Entry(msoFileName, fileEntry);
    if (KErrNone == err)
    {
        aSize = fileEntry.iSize;
    }
    return err; 
}

TInt CSharedObjectUtils::GetCurrentMSOSizeL(const char* aNameSpace, TInt& aSize, TSglQue<CFileNode>*& aFileList) const
{
    TInt err = KErrNone;
    aSize   = 0;
    HBufC* directory = NULL;
    CDir* fileList;
    CDir* subDirList;
    
    /* Get the directory name */
    if (aNameSpace)
    {       
        directory = GetPersistenceDirL(aNameSpace);
    }
    else
    {
        directory = HBufC::NewL(iPersistentDir.Length() + KPathDelimiterCharSize);
        directory->Des().Copy(iPersistentDir);
        directory->Des().Append(KPathDelimiter);
    }
    
    TFileName rootDirName;
    CArrayFix <TFileName> *dirArray = new (ELeave) CArrayFixFlat<TFileName> (10);
    
    rootDirName.Append(directory->Des());   
    dirArray->AppendL(rootDirName);
    
    while (dirArray->Count()!=0)
    {   
        err = iFs.GetDir(dirArray->At(0), KEntryAttNormal, ESortByName, fileList, subDirList);   
        
        if (KErrNone != err)
        {
            NPPDebug( LOG_ERROR, EFalse, _L("CSharedObjectUtils::GetCurrentMSOSizeL():ERR: %d"), err);
            if (directory)
                delete directory;
            
            delete subDirList;
            delete fileList;
            return err;
        }

        TInt subDirIndex, fileIndex;
        TInt subDirCount = subDirList->Count();
        for (subDirIndex = 0 ; subDirIndex < subDirCount; subDirIndex++)
        {
            TEntry subDirEntry((*subDirList)[subDirIndex]);
            TFileName subDirName (dirArray->At(0));
            
            if (subDirEntry.IsDir())
            {
                subDirName.Append(subDirEntry.iName);
                subDirName.Append(KPathDelimiter);
                dirArray->AppendL(subDirName);
            }
        }

        for (fileIndex = 0; fileIndex < fileList->Count(); fileIndex++)
        {
            /* Not to consider the file present under the root directory of 
             * persistent store for MSO. C:\Private\<FL UID>\
             */
            if (!aNameSpace && (0 == dirArray->At(0).Compare(rootDirName)))
            {
                break;
            }
            
            TEntry tempEntry ((*fileList)[fileIndex]);
            if (!tempEntry.IsDir())
            {
                aSize += tempEntry.iSize;
                
                _LIT16(KFileExtension,".");
                /* Include only the MSO files in to file list, we have other files like
                 * Trusted Store & MMC caching files present in Persistent directory.
                 * MSO files does not have any extensions hence check for such files.
                 */
                if (aFileList && (KErrNotFound == tempEntry.iName.Find(KFileExtension)))
                {
                    HBufC* filePath = NULL; 
                    CFileNode* node = NULL;
                    filePath = HBufC::NewL((dirArray->At(0)).Length() + tempEntry.iName.Length());
                    filePath->Des().Append(dirArray->At(0));
                    filePath->Des().Append(tempEntry.iName);                    
                    
                    node = CFileNode::NewL(tempEntry, filePath);
                    aFileList->AddLast(*node);
                    delete filePath;
                }
            }
        }
        
        if (subDirIndex == subDirList->Count())
        {
            dirArray->Delete(0);
            dirArray->Compress();
        }
        
        delete subDirList;
        delete fileList;
        
    }
    
    delete directory;
    delete dirArray;
    return err;
}

void CSharedObjectUtils::StartMSOCleanup(unsigned long dataSize)
{
    iSizeToCleanup = KMaxPersistentStoragePerMovie;
    
    if (dataSize > KMaxPersistentStoragePerMovie)
    {
        iSizeToCleanup = dataSize;
    }
    
    /* If CleanupRequest is not active, start a lower priority 
     * AO for cleanuping the MSO directory
     */
    if(!iAsyncObject->IsActive())
    {
        iAsyncObject->Start();
    }
}

void CSharedObjectUtils::DoRunL(const CActive* /* aActive*/)
{
    DoMSOCleanupL();
}

TInt CSharedObjectUtils::DoRunError(const CActive* /*aActive*/, TInt /*aError*/)
{
    return KErrNone;
}

void CSharedObjectUtils::DoMSOCleanupL()
{
    TInt err                        = KErrNoMemory;
    TSglQue<CFileNode>* fileList    = NULL;
    
    fileList = new TSglQue<CFileNode>(_FOFF(CFileNode, iLinkItem));
    User::LeaveIfNull(fileList);
                   
    /* Get the complete list of files present inside sub directories 
     * of FL privae folder.
     */  
    TInt aSize = 0;
    err = GetCurrentMSOSizeL(NULL, aSize, fileList);   
    User::LeaveIfError(err);
    
    /* Delete the LRU MSO untill we reach the Minimum cleanup
     * and untill we have the recently modified objects in the FileList.
     */    
    if (fileList->First() != fileList->Last())
    {       
        TInt dirSize = 0;
        TEntry oldestEntry(fileList->First()->GetFileEntry());
        HBufC* oldFilePath = fileList->First()->GetFilePath();
        HBufC* persistentDir = HBufC::NewL(iPersistentDir.Length() + KPathDelimiter);
        persistentDir->Des().Append(iPersistentDir);
        persistentDir->Des().Append(KPathDelimiter);
        
        TSglQueIter<CFileNode> iterator(*fileList);
        CFileNode *node = iterator;
        CFileNode *oldestNode = iterator;
        
        for (;iterator; iterator++)
        {
            node = iterator;
            /*  Check for oldest entry in the list */
            if (node->GetFileEntry().iModified < oldestNode->GetFileEntry().iModified)
            {
                /* Update current node as the oldest */
                oldestNode = node;
            }
        }
        
        oldestEntry = oldestNode->GetFileEntry();
        oldFilePath = oldestNode->GetFilePath();
        
        HBufC*  dirToDelete = NULL;
        TInt length = persistentDir->Length();      
        TPtr16 rootMovieID(oldFilePath->Des().MidTPtr((length), KRootMovieGUIDLen));
        TSglQue<CFileNode>* tempFileList = NULL;
        
        TBuf8<KMaxFileName> nameSpace;
        if (CnvUtfConverter::ConvertFromUnicodeToUtf8(nameSpace, rootMovieID) < 0)
        {
            DestoryFileList(fileList);
            delete persistentDir;
            return;
        }
        nameSpace.ZeroTerminate();
        
        err = GetCurrentMSOSizeL((const char *)nameSpace.Ptr(), dirSize, tempFileList);
        
        if (KErrNone != err)
        {
            NPPDebug( LOG_ERROR, EFalse, _L("CSharedObjectUtils::DoMSOCleanupL():ERR: %d"), err);
            DestoryFileList(fileList);
            delete persistentDir;
            return;
        }
        
        dirToDelete = HBufC::NewL(persistentDir->Length() + rootMovieID.Length() + KPathDelimiterCharSize);
        
        dirToDelete->Des().Append(persistentDir->Des());
        dirToDelete->Des().Append(rootMovieID);
        dirToDelete->Des().Append(KPathDelimiter);
        
        /* Remove the oldest node from the file list */
        fileList->Remove(*oldestNode);
        
        /* Remove the MSO directory of the SWF */ 
        err = iFileManager->RmDir(dirToDelete->Des());
        if (KErrNone != err)
        {
            NPPDebug( LOG_ERROR, EFalse, _L("CSharedObjectUtils::DoMSOCleanupL():ERR: %d"), err);
            delete persistentDir;
            delete dirToDelete;
            DestoryFileList(fileList);
            return;
        }
        
        iDeletedSize += dirSize;
        
        delete persistentDir;
        delete dirToDelete;
    }
    else
    {
        /* We do not have anyother MSO currently in this mode. 
         * Hence stop the cleanup.
         */
        return;
    }
    
    if (iDeletedSize < iSizeToCleanup)
    {
        iAsyncObject->Start();
    }
    
}

void CSharedObjectUtils::DestoryFileList(TSglQue<CFileNode>* aFileList) const
{
    if (aFileList)
    {
        CFileNode* item;
        while(!aFileList->IsEmpty())
        {
            item = aFileList->Last();
            aFileList->Remove(*item);
            delete item;
        }
        delete aFileList;
    }
}
