/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class CFlashSocketEngine and other 
		 related utilities
*
*/

#include "platformbuild.h"
#include "platformtypes.h"
#include "saplayer_build.h"
#include "ipconn_mgr.h"
#include "conn_handler.h"

//#include "saflash_consts.h"
#include "symbian_socket.h"
#include "symbian_clib.h"
#include <nifman.h>
#include <e32std.h>

#define SOCKET_ARRAY_SIZE	5

#ifdef FEATURE_PERFORMANCE_AUTOMATION
const TInt CFlashSocketEngine::KNetworkTimeOut = 60000000*6; // 60 seconds time-out
#else
const TInt CFlashSocketEngine::KNetworkTimeOut = 60000000; // 60 seconds time-out
#endif

#ifdef FEATURE_XML_SOCKET

CFlashSocketEngine::CFlashSocketEngine(CPlatformFlashEngine& aEngine)
: iEngine(aEngine)
, iSocketList(NULL)
, iTimer(NULL)
, iConnStat(EFlhNReqNotConnectedToNetwork)
{
}

CFlashSocketEngine* CFlashSocketEngine::NewL(CPlatformFlashEngine& aEngine)
{
	CFlashSocketEngine* self = new (ELeave) CFlashSocketEngine(aEngine);

	CleanupStack::PushL(self);

	self->ConstructL();

	CleanupStack::Pop(self);
	
	return self;
}
	
CFlashSocketEngine::~CFlashSocketEngine()
{
	if(iSocketList)
	{
		iSocketList->ResetAndDestroy();
	}
	delete iSocketList;

	iIPConnectionMgr = NULL;
	if(iConnHandler)
		{
		delete iConnHandler;
		iConnHandler = NULL;
		}
	delete iTimer;
}

void CFlashSocketEngine::ConstructL()
{
	iSocketList = new (ELeave) CArrayPtrFlat<CFlashSocket>(SOCKET_ARRAY_SIZE);
	iIPConnectionMgr = iEngine.GetIPConnectionMgr();//CFlhIPConnection::NewL(iEngine.ConnectionPromptNeeded());

	iTimer = CFlhTimerObject::NewL(CActive::EPriorityIdle,*this);

	return;
}

void CFlashSocketEngine::NotifyStatusChange(CFlashSocket* aSocket, CFlashSocket::TStatusCode aStatusCode, TInt aErrorCode)
{
	switch(aStatusCode)
	{
	case CFlashSocket::EConnected:
		iEngine.GetFIObject().FI_NotifySocketStatus((unsigned long) aSocket, FI_SocketStatus_Connected, (short) aErrorCode);
		break;

	case CFlashSocket::EFailedResolve:
	case CFlashSocket::EFailedConnect:
		iEngine.GetFIObject().FI_NotifySocketStatus((unsigned long) aSocket, FI_SocketStatus_Failed, (short) aErrorCode);
		AttemptStartNetworkTimer(); //start timer for closing network connection
		break;

	case CFlashSocket::EClosed:
	case CFlashSocket::EClosedWithError:
		iEngine.GetFIObject().FI_NotifySocketStatus((unsigned long) aSocket, FI_SocketStatus_Closed, (short) aErrorCode);
		AttemptStartNetworkTimer(); //start timer for closing network connection
		break;

	default:
		//dont report anything to the player for all other messages
		break;
	}

}
void CFlashSocketEngine::ExpediteConnectionL() 
{
    if(iConnStat == EFlhNReqNotConnectedToNetwork)
    {
		iConnHandler = iIPConnectionMgr->GetConnectionL(this);
        iConnStat = EFlhNReqConnectingToNetwork;
    }
}

bool CFlashSocketEngine::OpenNewSocketL(FI_SocketConnectionInfo* aConnInfo)
{
	//Free all closed sockets
	CleanupSocketList();

	//create a new socket instance
	CFlashSocket* pNewSocket = CFlashSocket::NewL(*this, aConnInfo->hostName, aConnInfo->port);

	CleanupStack::PushL(pNewSocket);

	//set the socketId to this instance of socket for future reference (to be used by player for subsequent SI calls)
	aConnInfo->socketId = (unsigned long) pNewSocket;

	//add it to the list of sockets
	iSocketList->AppendL(pNewSocket);

	//cancel any existing timer
	iTimer->Cancel();

	if(iConnStat == EFlhNReqConnectedToNetwork)
	{
		pNewSocket->ConnectL();
	}
	else if(iConnStat == EFlhNReqNotConnectedToNetwork)
	{
	iConnHandler = iIPConnectionMgr->GetConnectionL(this);
		iConnStat = EFlhNReqConnectingToNetwork;
	}

	CleanupStack::Pop(pNewSocket);
	
	return true;
}

bool CFlashSocketEngine::CloseSocket(unsigned long aSocketId)
{
	//lookup the socket based on socket id, close it 
	//and remove from the socket list of sockets

	CFlashSocket* socket = reinterpret_cast<CFlashSocket *>(aSocketId);

	TKeyArrayFix pKey(0,ECmpTUint32);
	int pIndex;
	if(iSocketList && (!iSocketList->Find(socket, pKey, pIndex)) )
	{
		//socket found

		//close connection
		socket->CloseConnection();

		AttemptStartNetworkTimer(); //start timer for closing network connection
		
		return true;
	}
	else
	{
		return false;
	}
}

short CFlashSocketEngine::SendL(unsigned long aSocketId, FI_NetworkBufferInfo* aDataBuffer)
{
	//lookup the socket based on socket and write data to it

	CFlashSocket* socket = reinterpret_cast<CFlashSocket *>(aSocketId);
	
	TKeyArrayFix pKey(0,ECmpTUint32);
	int pIndex;
	if(iSocketList && (!iSocketList->Find(socket, pKey, pIndex)) )
	{
		TPtr8 pData(aDataBuffer->pBuffer,aDataBuffer->dataSize, aDataBuffer->dataSize);
		socket->WriteL(pData);
	
		return 1;
	}
	else
	{
		return 0;
	}
}

short CFlashSocketEngine::ReceiveL(CFlashSocket* aSocket, const TDesC8& aBuffer)
{
	unsigned long bytesToWrite = (unsigned long) aBuffer.Length();
	const unsigned char* dataToWrite = aBuffer.Ptr();

	FI_NetworkBufferInfo pNetBuf;

	pNetBuf.pBuffer = new (ELeave) unsigned char[bytesToWrite+1];
	if(pNetBuf.pBuffer)
	{
		FlashMemCpy(pNetBuf.pBuffer, dataToWrite, bytesToWrite);
		pNetBuf.dataSize = bytesToWrite;
		pNetBuf.maxBufSize = 0; //this is ignored in context of FlashLite Sockets

		iEngine.GetFIObject().FI_SocketRecv((unsigned long) aSocket, &pNetBuf, NULL); //send data to the player

		//delete the buffer as the player will make a copy of the data
		delete [] pNetBuf.pBuffer;

		return true;
	}
	else
	{
		return false;
	}

}

void CFlashSocketEngine::ResetEngine()
{
	//close all active socket connections and free up the memory
	if(iSocketList)
	{
		iSocketList->ResetAndDestroy();
	}

	Reset();
}

void CFlashSocketEngine::Reset()
{
	if(iTimer->IsActive()) // we were already shutting down the connection...
	{
		iTimer->Cancel();
	}

	//No connection request in specified timeout period..shutdown network connection
	if(iConnHandler)
		{
		delete iConnHandler;
		iConnHandler = NULL;
		}
	
	iConnStat = EFlhNReqNotConnectedToNetwork;
}

void CFlashSocketEngine::HandleNoConnection()
{
	//Network connection not available
	//Remove all sockets pending connection and reset connection status
	iConnStat = EFlhNReqNotConnectedToNetwork;

	for(int i=iSocketList->Count()-1; i>=0; i--)
	{
		CFlashSocket* socket = iSocketList->At(i);
		NotifyStatusChange(socket, CFlashSocket::EFailedConnect, KConnectionFailure); //notify player of failed connection
		iSocketList->Delete(i);
		delete socket;
	}
}

void CFlashSocketEngine::CleanupSocketList()
{
	//free all closed sockets
	for(int i=iSocketList->Count()-1; i>=0; i--)
	{
		CFlashSocket* socket = iSocketList->At(i);
		if( !socket->IsOpen() )
		{
			iSocketList->Delete(i);
			delete socket;
		}
	}
}

CPlatformFlashEngine* CFlashSocketEngine::GetPlatformEngine() const
{
	return &iEngine;
}

void CFlashSocketEngine::AttemptStartNetworkTimer()
{
	//Go thru the socket list.  If all sockets are closed
	//then start timer to initiate network shutdown

	int count = iSocketList->Count();
	for(int i=0;i<count;i++)
	{
		CFlashSocket* socket = iSocketList->At(i);
		if(socket->IsOpen())
		{
			return;
		}
	}

	iTimer->Cancel(); //cancel any existing timer event
	
	iTimer->After(KNetworkTimeOut);

}

void CFlashSocketEngine::ConnectionEventL(TFlhIPEvent aEvent)
{
	CleanupSocketList(); //remove all sockets that might have been closed while they were waiting for access point selection

	if(aEvent == EEventIsConnectedOK)
	{
		iConnStat = EFlhNReqConnectedToNetwork;

		int count = iSocketList->Count();
		for(int i=0; i<count; i++)
		{
			CFlashSocket* socket = iSocketList->At(i);
			socket->ConnectL();
		}

	}
	else //connection failed...close and cleanup all sockets
	{
		HandleNoConnection();
	}

}

void CFlashSocketEngine::HandleNetworkAccessDenied()
{
	// the user declined to pick an access point, or a Leave happened setting up the connection
	HandleNoConnection();
}

void CFlashSocketEngine::DoRunL(const CActive* /*aActive*/)
{
	CleanupSocketList();

	Reset();
}

TInt CFlashSocketEngine::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	return aError; // never should happen in this case as our RunL() cannot leave...
}


const TInt CFlashSocket::KSocketTimeOut = 30000000; // 30 seconds time-out


CFlashSocket::CFlashSocket(CFlashSocketEngine& aSocketEngine, unsigned short aPort)
: CActive(EPriorityStandard)
, iSocketEngine(aSocketEngine)
, iState(EStateOpen)
{
	iPort = aPort;
	iServerName = NULL;
	iSocketStatus = ENotConnected;
}


CFlashSocket::~CFlashSocket()
{
	CloseConnection();
}


CFlashSocket* CFlashSocket::NewL(CFlashSocketEngine& aSocketEngine, const char* aHost, unsigned short aPort)
{
	CFlashSocket* self = new (ELeave) CFlashSocket(aSocketEngine, aPort);

	CleanupStack::PushL(self);
	
	self->ConstructL(aHost);
	
	CleanupStack::Pop(self);
	
	return self;
}


void CFlashSocket::ConstructL(const char* aHost)
{
	CActiveScheduler::Add(this); 

    // Start a timer
	iTimer = CFlhTimerObject::NewL(EPriorityStandard, *this);

    // Create socket read and write active objects
	iSocketReader = CFlashSocketReader::NewL(*this, iSocket);
	iSocketWriter = CFlashSocketWriter::NewL(*this, iSocket);

	CPlatformFlashEngine* pEngine = iSocketEngine.GetPlatformEngine();
	iServerName = pEngine->CreateEpocStringL( aHost, FlashStrLen(aHost) );	
}

void CFlashSocket::ConnectL()
{
	// Initiate connection process
	if (iSocketStatus == ENotConnected)
	{
		TInetAddr addr;

		if (addr.Input(*iServerName) == KErrNone)
		{ 
			// server name is already a valid IP address
			ChangeStatus(EResolved, KErrNone);
			ConnectL(addr.Address());
		}
		else // need to look up name using dns
		{ // Initiate DNS
		CConnHandler* lConnhandler = iSocketEngine.GetConnHandler();
		if(lConnhandler)
			{
			TInt err = iResolver.Open(lConnhandler->SocketSession(), KAfInet, KProtocolInetUdp ,lConnhandler->ConnectionInstance());
			if (err == KErrNone)
			{
				// DNS request for name resolution
				iResolver.GetByName(*iServerName, iNameEntry, iStatus);
	        
				ChangeStatus(EResolving, KErrNone);
				// Request time out
				StartTimer();
				SetActive();
			}
			else
			{
				ChangeStatus(EFailedResolve, err);
			}
			}
		}
	}
}

void CFlashSocket::ConnectL(TUint32 aAddr)
{
	// Initiate attempt to connect to a socket by IP address	
	if (iSocketStatus == EResolved)
	{
		// Open a TCP socket
		CConnHandler* lConnhandler = iSocketEngine.GetConnHandler();
		if(lConnhandler)
				{
				TInt err = iSocket.Open(lConnhandler->SocketSession(), KAfInet, KSockStream, KProtocolInetTcp,lConnhandler->ConnectionInstance());
		if (err == KErrNone)
		{
			// Set up address information
			iAddress.SetPort(iPort);
			iAddress.SetAddress(aAddr);

			// Initiate socket connection
			iSocket.Connect(iAddress, iStatus);
			ChangeStatus(EConnecting, KErrNone);
        
			// Start a timeout
			StartTimer();
			SetActive();
		}
		else
		{
			ChangeStatus(EFailedConnect, err);
		}
	}
}
}

// from CActive
void CFlashSocket::DoCancel()
{
	iTimer->Cancel();
	
    // Cancel appropriate request to socket
    switch (iSocketStatus)
	{
		case EResolving:
		// Cancel look up attempt
		iResolver.Cancel();
		iResolver.Close();
	    ChangeStatus(EFailedResolve, iStatus.Int());
		break;

		case EResolved:
	    ChangeStatus(EFailedConnect, iStatus.Int());
		break;
		
		case EConnecting:
		iSocket.CancelConnect();
	    ChangeStatus(EFailedConnect, iStatus.Int());
		break;

		case EConnected:
	    ChangeStatus(EClosedWithError, iStatus.Int());
		break;

		default: //all other cases do nothing
		break;
	}

}

// from CActive
void CFlashSocket::RunL()
{
	iTimer->Cancel(); // Cancel TimeOut timer before completion

	switch(iSocketStatus)
	{
		case EResolving:
		iResolver.Close();
		if (iStatus == KErrNone)
		{ // DNS look up successful
			iNameRecord = iNameEntry();
			// Extract IP address from name record
			// and connect to the IP address
			ChangeStatus(EResolved, KErrNone);
			ConnectL(TInetAddr::Cast(iNameRecord.iAddr).Address());
		}
		else
		{	
			// DNS lookup failed
			ChangeStatus(EFailedResolve, iStatus.Int());
		}
		break;

		case EConnecting:
		// IP connection request
		if (iStatus == KErrNone)
		{ // Connection completed successfully
			ChangeStatus(EConnected, KErrNone);
		    iSocketReader->Start(); //start receiving data
		}
		else
		{
			ChangeStatus(EFailedConnect, iStatus.Int());
		}
		break;

		default: //all other cases do nothing
		break;
	}
}

void CFlashSocket::DoRunL(const CActive* /*aActive*/)
{
	if(iSocketStatus == EConnected)
	{
		//connection was established.  close connection with timeout error
		ChangeStatus(EClosedWithError, MSocketNotifier::EUnknownError);
	}
	else
	{
		//connection was not established, report connection error.
		ChangeStatus(EFailedConnect, MSocketNotifier::ETimeoutOnConnect);
	}
}

TInt CFlashSocket::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	return aError; // never should happen in this case as our RunL() cannot leave...
}

void CFlashSocket::ReportErrorL(MSocketNotifier::TErrorType aErrorType, TInt aErrorCode)
{
	if(aErrorType == EDisconnected)
	{
		ChangeStatus(EClosed, KErrNone);
	}
	else
	{
		ChangeStatus(EClosedWithError, aErrorCode);
	}
}

void CFlashSocket::WriteCompleted()
{
	iSocketReader->Start();
}

void CFlashSocket::WriteL(const TDesC8& aBuffer)
{
	iSocketWriter->IssueWriteL(aBuffer);
}

TBool CFlashSocket::ResponseReceivedL(const TDesC8& aBuffer)
{
	return iSocketEngine.ReceiveL(this, aBuffer);
}

void CFlashSocket::CloseConnection()
{
	if( IsOpen() )
	{
		iState = EStateClosed; //set socket state to closed to prevent recursive calls to CloseConnection()

		if(iSocketReader)
		{
			iSocketReader->Cancel();
		}

		if(iSocketWriter)
		{
			iSocketWriter->Cancel();
		}

		Cancel();

		if (iSocket.SubSessionHandle())
		{
			iSocket.Close();
		}

		delete iSocketReader;
		iSocketReader = NULL;

		delete iSocketWriter;
		iSocketWriter = NULL;

		delete iTimer;
		iTimer = NULL;

		delete iServerName;
		iServerName = NULL;
	}
}

void CFlashSocket::ChangeStatus(TStatusCode aNewStatus, TInt aErrorCode)
{
	if(aNewStatus == EFailedResolve || aNewStatus == EFailedConnect 
		|| aNewStatus == EClosed || aNewStatus == EClosedWithError)
	{
		CloseConnection();
	}
	else
	{
		iSocketStatus = aNewStatus; //change the socket status
	}

	iSocketEngine.NotifyStatusChange( this, aNewStatus, aErrorCode); //ask the engine to notify the player of status change
}

TBool CFlashSocket::IsOpen()
{
	if(iState == EStateOpen)
	{
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}

void CFlashSocket::StartTimer()
{
	iTimer->Cancel();
	iTimer->After(KSocketTimeOut);
}

#endif //FEATURE_XML_SOCKET
CFlashSocketReader* CFlashSocketReader::NewL(MSocketNotifier& aSocketNotifier, RSocket& aSocket/*, unsigned long bufferSize*/)
{
	CFlashSocketReader* self = new (ELeave) CFlashSocketReader(aSocketNotifier, aSocket/*, bufferSize*/);

	CleanupStack::PushL(self);
	
	self->ConstructL();
	
	CleanupStack::Pop(self);
	
	return self;
}

	
CFlashSocketReader::CFlashSocketReader(MSocketNotifier& aSocketNotifier, RSocket& aSocket/*, unsigned long bufferSize*/)
: CActive(EPriorityStandard),
  iSocket(aSocket),
  iSocketNotifier(aSocketNotifier)
{
}


CFlashSocketReader::~CFlashSocketReader()
{
	Cancel();
}


void CFlashSocketReader::ConstructL()
{
	//iBuffer = HBufC8::NewL(iMaxBuffer);
	CActiveScheduler::Add(this);
}

void CFlashSocketReader::DoCancel()
{
	// Cancel asynchronous read request
	iSocket.CancelRead();
}

void CFlashSocketReader::RunL()
{
	// Active object request complete handler
	switch (iStatus.Int())
	{
		case KErrNone:
		// Character has been read from socket
		if (iSocketNotifier.ResponseReceivedL(iBuffer))
		{ // if not done, immediately start another read
			IssueRead(); 
		}
		break;

		case KErrDisconnected:
		case KErrEof:
		iSocketNotifier.ReportErrorL(MSocketNotifier::EDisconnected, iStatus.Int());
		break;

		default:
		iSocketNotifier.ReportErrorL(MSocketNotifier::EGeneralReadError, iStatus.Int());
		break;
	}	
}

void CFlashSocketReader::IssueRead()
{
    // Initiate a new read from socket into iBuffer
    if(!iSocketNotifier.IsOpen())
    	return;
    
	iBuffer.FillZ();
    iBuffer.Zero();
	iSocket.RecvOneOrMore(iBuffer, 0, iStatus, iDummyLength);
    SetActive();
}

void CFlashSocketReader::Start()
{
    // Initiate a new read from socket into iBuffer
	if (!IsActive())
	{
		IssueRead();
	}
}

const TInt CFlashSocketWriter::KWriterTimeOut = 10000000; // 10 seconds time-out
const TInt CFlashSocketWriter::KMeanReadWriteTimeDelay = 1000000; //1 second time delay for read write cycle
CFlashSocketWriter* CFlashSocketWriter::NewL(MSocketNotifier& aSocketNotifier, RSocket& aSocket)
{
	CFlashSocketWriter* self = new (ELeave) CFlashSocketWriter(aSocketNotifier, aSocket);
	
	CleanupStack::PushL(self);

	self->ConstructL();
	
	CleanupStack::Pop(self);
	
	return self;
}

	
CFlashSocketWriter::CFlashSocketWriter(MSocketNotifier& aSocketNotifier, RSocket& aSocket)
: CActive(EPriorityStandard),
  iSocket(aSocket),
  iSocketNotifier(aSocketNotifier)
{
	TInt i;
	for (i=0;i<kMaxWriteBuffers;i++)
	{
		iWriteBuffers[i] = NULL;
	}
	iWriteBufferQueueFront = 0;
	iWriteBufferQueueRear = 0;
	iWriteBufferQueueCount = 0;
}


CFlashSocketWriter::~CFlashSocketWriter()
{
	Cancel();
	delete iTimer;

	TInt i;
	for (i=0;i<kMaxWriteBuffers;i++)
	{
		if (iWriteBuffers[i] != NULL)
		{
			delete iWriteBuffers[i];
		}
	}
}

void CFlashSocketWriter::DoCancel()
{	
	// Cancel asychronous write request
	iSocket.CancelWrite();
	iTimer->Cancel();
	iWriteStatus = EWaiting;
}

void CFlashSocketWriter::ConstructL()
{
	CActiveScheduler::Add(this);

	iTimer = CFlhTimerObject::NewL(CActive::EPriorityUserInput, *this);

	iWriteStatus = EWaiting;
}


void CFlashSocketWriter::RunL()
{
	iTimer->Cancel();
	
	// Active object request complete handler
	if (iStatus == KErrNone)
	{
		if (iWriteStatus == ESending)
		{
			// Data has been written to socket

			//delete the buffer and move the front pointer to next buffer in queue
			delete iWriteBuffers[iWriteBufferQueueFront];
			iWriteBuffers[iWriteBufferQueueFront] = NULL;

			iWriteBufferQueueFront = (iWriteBufferQueueFront + 1) % kMaxWriteBuffers;
			--iWriteBufferQueueCount;
			User::After(KMeanReadWriteTimeDelay);
			SendNextPacket();
		}
	}
	else 
	{
		// Error: pass it up to user interface
		iSocketNotifier.ReportErrorL(MSocketNotifier::EGeneralWriteError, iStatus.Int());
		iWriteStatus = EWaiting;
	}
}

void CFlashSocketWriter::IssueWriteL(const TDesC8& aData)
{
	if (aData.Size() > 0)
	{
		if ((iWriteStatus != EWaiting) && (iWriteStatus != ESending))
		{
			User::Leave(KErrNotReady);
		}

		if(iWriteBufferQueueCount >= kMaxWriteBuffers)
		{
			User::Leave(KErrOverflow);
		}

		// Make copy of data
		iWriteBuffers[iWriteBufferQueueRear] = aData.Alloc();
		
		iWriteBufferQueueRear = (iWriteBufferQueueRear + 1) % kMaxWriteBuffers;
		++iWriteBufferQueueCount;

		if (!IsActive())
		{
			SendNextPacket();
		}
	}
}

void CFlashSocketWriter::SendNextPacket()
{
	if((iWriteBufferQueueCount > 0) && (iWriteBuffers[iWriteBufferQueueFront] != NULL))
	{
		iWriteStatus = ESending;
		if (iWriteBuffers[iWriteBufferQueueFront]->Size() > 0)
		{
			iSocket.Send(*iWriteBuffers[iWriteBufferQueueFront], 0, iStatus); // Initiate actual write

			iTimer->Cancel();
			
			// Request timeout
			iTimer->After(KWriterTimeOut);
			
			SetActive();
			
		}
	}
	else
	{
		iWriteStatus = EWaiting;
		iSocketNotifier.WriteCompleted();
	}
}

void CFlashSocketWriter::DoRunL(const CActive* /*aActive*/)
{
	Cancel();
	iWriteStatus = EWaiting;
	iSocketNotifier.ReportErrorL(MSocketNotifier::ETimeOutOnWrite, KErrTimedOut);
}

TInt CFlashSocketWriter::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	return aError; // never should happen in this case as our RunL() cannot leave...
}
