/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the string utility functions.
*
*/


#include "platformbuild.h"

#include "platformdebug.h"
#include "platformtypes.h"
#include "saplayer_build.h"

#include "symbian_stringutils.h"
#include "saflash_consts.h"
#include "engine_defs.h"

// ConvertIntegerToString: Converts a signed integer to its ASCII representation
char* ConvertIntegerToString (int aValue, int aRadix, char* aBuffer, int aLength )
{
	char* pch = aBuffer + aLength;
	*--pch = 0;

	if (aRadix < 2 || aRadix > 36) {
		PLAYERASSERT( 0 );
		return pch;
	}

	if ( !aValue )
	{
		*--pch = '0';
		return pch;
	}

	int neg = aValue < 0;
	if ( neg )
	{
		aValue = -aValue;
	}

	while ( aValue )
	{
		int v = aValue % aRadix;
		aValue /= aRadix;

		*--pch = ( v <= 9 ) ? static_cast<char>(( v + '0' )) : static_cast<char>(( v - 10 + 'a' ));
	}

	if ( neg )
	{
		*--pch = '-';
	}

	return pch;
}

char* ConvertU32ToString ( U32 aValue, int aRadix, char* aBuffer, int aLength )
{
	char* pch = aBuffer + aLength;
	*--pch = 0;

	if (aRadix < 2 || aRadix > 36) {
		PLAYERASSERT( 0 );
		return pch;
	}

	if ( !aValue )
	{
		*--pch = '0';
		return pch;
	}

	while ( aValue )
	{
		U32 v = aValue % aRadix;
		aValue /= aRadix;

		*--pch = ( v <= 9 ) ? static_cast<char>(( v + '0' )) : static_cast<char>(( v - 10 + 'a' ));
	}

	return pch;
}

char* StrChr( const char* aStr, int aCh )
{
	while ( *aStr )
	{
		if ( *aStr == aCh )
			return (char*) aStr;	// we don't change but the caller might.
		aStr++;
	}
	return 0;
}

TBool StripPrefix(const TDesC& aInput, const TDesC& aPrefix, TPtrC& aStripped)
{
	TBool ret = EFalse;
	TInt pos = aInput.FindF(aPrefix);
	if (pos == 0)
	{
		if (aPrefix.Length() < aInput.Length() - 1) // check if anything after prefix
		{
			aStripped.Set(aInput.Mid(aPrefix.Length()));
		}
		else
		{
			aStripped.Set(KNullDesC); // empty descriptor
		}
		ret = ETrue;
	}
	return ret;
}

char* StripPrefix( const char* aStr, const char* aPre)
// If aStr begins with aPre, return the first char after in aStr
{
	if ((!aStr) || (!aPre))
		return 0;

	FOREVER	{
		// Map to uppercase
		unsigned char s = *aStr;
		if ( s >= 'a' && s <= 'z' )
			s -= 'a' - 'A';
		unsigned char p = *aPre;
		if ( p >= 'a' && p <= 'z' )
			p -= 'a' - 'A';

		// See if the characters are not equal or we hit the end of the strings
		if ( s != p || !s || !p ) 
			break;

		aPre++; aStr++;
	}
	//^^ return *aPre == 0 ? const_cast< char*>( aStr ) : 0;
	return *aPre == 0 ? (char*)aStr : 0;
}

TBool ComparePrefix(const TDesC& aStr, const TDesC& aPrefix)
{
	TPtrC str = aStr.Left(aPrefix.Length());
	if (str.CompareF(aPrefix) == 0)
	{
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}

TInt GetProtocolFromURL(const TDesC& aUrl)
{
	if (aUrl.Length() == 0)
	{
		return PROTOCOL_NONE;
	}
	else if (ComparePrefix(aUrl, KSchemeHTTP))
	{
		return PROTOCOL_HTTP;
	}
	else if (ComparePrefix(aUrl, KSchemeFile))
	{
		return PROTOCOL_FILE;
	}
	else if (ComparePrefix(aUrl, KSchemeNET))
	{
		return PROTOCOL_HTTP;
	}
	else if (ComparePrefix(aUrl, KSchemeSymbol))
	{
		return PROTOCOL_SYMBOL;
	}
	else if (ComparePrefix(aUrl, KSchemeRTSP))
	{
		return PROTOCOL_RTSP;
	}
	else if (ComparePrefix(aUrl, KSchemeReference))
	{
		return PROTOCOL_REFERENCE;
	}

	return PROTOCOL_NONE;
}

char* CreateStr(const char* aS)
{
	if (!aS) {
		return 0;
	}
	int iLen = FlashStrLen(aS) + 1;
	char* c = new char[iLen];
	if (c) {
		FlashMemCpy(c, aS, iLen);
	}
	return c;
}

bool ToU32 ( const char* aC, U32* aResult )
{
	if ( aC == NULL ) return false;
	
	U32 v = 0;
	bool result = false;

	while ( *aC >= '0' && *aC <= '9' ) 
	{
		result = true;
		v = v*10 + *aC-'0';
		aC++;
	}

	*aResult = v;
	return result;
}

U32 ToU32 ( const char* aC )
{
	U32 v = 0;
	if ( aC == NULL ) return 0;
	
	while ( *aC >= '0' && *aC <= '9' ) 
	{
		v = v*10 + *aC-'0';
		aC++;
	}

	return v;
}

TUint32 ToU32(const TDesC& aString)
{
	TUint32 num = 0;
	TInt len = aString.Length();
	TInt i;
	for (i=0;i<len;i++)
	{
		if ((aString[i] >= '0') && (aString[i] <= '9'))
		num = num * 10;
		num += aString[i] - '0';
	}
	return num;
}


bool ToInt(const char* aC, int* aResult)
{
	if ( aC == NULL ) return false;
	
	U32 v = 0;
	if ( *aC == '-' )
	{
		bool result = ToU32(aC+1, &v);
		*aResult = -(int)v;
		return result;
	}
	else
	{
		bool result = ToU32(aC, &v);
		*aResult = (int)v;
		return result;
	}
}

void ReplaceAll(TDes8& aDes8, char aFind, char aReplace)
{
	const TInt desLen=aDes8.Length();
	for(TInt i=0;i<desLen;i++)
	{
		if(aDes8[i]==aFind)
		{
			aDes8[i]=aReplace;
		}
	}
}

void ReplaceAll(TDes16& aDes16, TChar aFind, TChar aReplace)
{
	const TInt desLen=aDes16.Length();
	for(TInt i=0;i<desLen;i++)
	{
		if(aDes16[i]==(unsigned short)aFind)
		{
			aDes16[i]=(unsigned short)aReplace;
		}
	}
}

// designed not to leave so callers can wrap in User::LeaveIfNull if they want to...
char* ReAllocString(const char* aCurrentZtString, const char* aNewData, int aNewDataLen)
{ // use des's for len checking
	TPtrC8 currentPtr8(KNullDesC8);
	
	if(aCurrentZtString)
	{
		currentPtr8.Set((TUint8*)aCurrentZtString);
	}
	
	TPtrC8 appendPtr8((TUint8*)aNewData,aNewDataLen);
	const TInt newLen=currentPtr8.Length()+1+appendPtr8.Length();  // 1 for ZT...
	char* newStr=new char[newLen];
	if(newStr)
	{
		TPtr8 newPtr8((TUint8*)newStr,0,newLen);
		newPtr8.Append(currentPtr8);
		newPtr8.Append(appendPtr8);
		newPtr8.ZeroTerminate();
	}
	return newStr;
}

HBufC* URLEncodePostDataLC(TDesC8& aData)
{
	TInt elen = 0;
	TInt dlen = aData.Length();

	if (dlen <= 0)
	{
		return NULL;
	}

	TInt i;
	for (i=0;i<dlen;i++)
	{
		if (aData[i] <= 0x7F)
		{
			elen++;
		}
		else
		{
			elen += 3;
		}
	}

	HBufC* estr = HBufC::NewLC(elen);
	TPtr estrPtr = estr->Des();
	for (i=0;i<dlen;i++)
	{
		if (aData[i] <= 0x7F)
		{
			estrPtr.Append(aData[i]);
		}
		else
		{
			estrPtr.Append('%');
			estrPtr.AppendNumFixedWidth(aData[i], EHex, 2);
		}
	}

	return estr;
}
