/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Provides the CIADUpdateChecker implementation.
*
*
*/


//#ifdef SELF_IAD_UPDATE
#include "saplayer_build.h"

#include <featmgr.h>
#include <iaupdateobserver.h>
#include <iaupdate.h>
#include <iaupdateparameters.h> 
#include <iaupdateresult.h>
#include "symbian_updatecheck.h"

#include "flash_iad_update_callback.h"


#ifdef _DEBUG
#define DEBUG_IAD
#endif

#ifdef DEBUG_IAD

#include <flogger.h>
const TUint KMaxLogLength   = 512;

/*
 * Returns the time stamp in the format
 * hhmmssuuuuuu
 * where 
 * hh (hours)       : 00 to 24
 * mm (minutes)     : 00 to 59
 * ss (seconds)     : 00 to 59
 * uuuuuu (microseconds): 000000 to 999999
 */

TInt64 TimeStamp()
{
    TTime currentTime;
    currentTime.UniversalTime();

    TInt64 stamp = currentTime.DateTime().Hour();
    stamp *= 100;    // we have 24 hrs
    
    stamp += currentTime.DateTime().Minute();
    stamp *= 100;    // we have 60 minutes
    
    stamp += currentTime.DateTime().Second();
    stamp *= 1000000;    // we have 60 seconds
    
    stamp += currentTime.DateTime().MicroSecond();
    
    return stamp;
    
}
#endif

#ifdef DEBUG_IAD
void IADDebug(TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... )
    {
    if(aLogLevel < CUR_LOG_LEVEL)  
        return;

    TTime currentTime;
    currentTime.UniversalTime();
    
    TTime firstTime;
    TBool initialized = EFalse;
    if( !initialized )
    {
        firstTime.UniversalTime();
        initialized = true;
    }
    
    TTimeIntervalMicroSeconds timeStamp = currentTime.MicroSecondsFrom(firstTime);
    
    
    VA_LIST args;
    VA_START( args, aText );
    
    TBuf<KMaxLogLength> buf;
    buf.FormatList( aText, args );

    TBuf<128> theTimeStamp;
//    theTimeStamp.Format( _L("%d"), currentTime.DateTime().MicroSecond() ); //  timeStamp.Int64() );
//    theTimeStamp.Num( TimeStamp() ); //  timeStamp.Int64() );
    
    
    RFileLogger logger;
    
    TInt ret=logger.Connect();
    
    if (ret==KErrNone)
        {
        logger.SetDateAndTime( EFalse,EFalse );
        logger.CreateLog( _L("FlashLogs"), _L("IADLogFle.log"), EFileLoggingModeAppend );
//        logger.Write(theTimeStamp);
        logger.Write(buf);
        }

    logger.Close();

    VA_END( args );
    
    }

#else
void IADDebug(TUint8 /*aLogLevel*/,TRefByValue<const TDesC> /*aText*/, ... )
{
}

#endif
    

/**
 * Symbian constructor.
 *
 * @param aApplicationUID Uid describes either package UID or SID.
 *
 * @param aAppSearchCriteria When correct updates are searched,
 * this text is used as a criteria when checking the information 
 * that is attached to the updates.
 *
 * @param aCommandLineExecutable When the update operation finishes,
 * the content of this desciptor will be used as a command line executable.    								
 *
 * @param aCommandLineArguments When the update operation finishes,
 * the content of this desciptor will be used as command line arguments
 * with the command line executable.
 *
 * @param aShowProgress If ETrue, a wait dialog is shown when
     * CIAUpdate::CheckUpdates operation is started. If EFalse,
     * the dialog is not shown.
 * @return Pointer to created CIADUpdateChecker object
 *
 * @since S60 v3.2
 */
CIADUpdateChecker* CIADUpdateChecker::NewL(const TUid& aApplicationUID
    									, const TDesC& aAppSearchCriteria
    									,const TDesC& aCommandLineExecutable 
    									,const TDesC8& aCommandLineArguments
    									,TBool aShowProgress
    									,MIadUpdateCallback* aIADUpdateCompleteCallback
    								 )
{
	IADDebug(LOG_DEBUG,_L("In CUpdateChecker NewL"));
	CIADUpdateChecker* self = new(ELeave) CIADUpdateChecker(aApplicationUID
															,aAppSearchCriteria
															,aCommandLineExecutable
															,aCommandLineArguments
															,aShowProgress
															,aIADUpdateCompleteCallback);

	CleanupStack::PushL(self);
	self->ConstructL();
					
	CleanupStack::Pop(self);

	return self;	
}
   
/**
 * Destructor.
 */
CIADUpdateChecker::~CIADUpdateChecker()
{
	if(iIADUpdateCheck)
	{
		delete iIADUpdateCheck;
		iIADUpdateCheck = NULL;
	}
	
	if(iIADParam)
	{
		delete iIADParam;
		iIADParam =NULL;
	}

}

/**
 * C++ default constructor.
 *
 * 
 * @param aApplicationUID Uid describes either package UID or SID.
 *
 * @param aAppSearchCriteria When correct updates are searched,
 * this text is used as a criteria when checking the information 
 * that is attached to the updates.
 *
 * @param aCommandLineExecutable When the update operation finishes,
 * the content of this desciptor will be used as a command line executable.    								
 *
 * @param aCommandLineArguments When the update operation finishes,
 * the content of this desciptor will be used as command line arguments
 * with the command line executable.
 *
 * @param aShowProgress If ETrue, a wait dialog is shown when
     * CIAUpdate::CheckUpdates operation is started. If EFalse,
     * the dialog is not shown.
 * @return Pointer to created CIADUpdateChecker object
 */
CIADUpdateChecker::CIADUpdateChecker(const TUid& aApplicationUID
    								, const TDesC& aAppSearchCriteria
    								,const TDesC& aCommandLineExecutable 
    								,const TDesC8& aCommandLineArguments
    								,TBool aShowProgress
    								,MIadUpdateCallback* aIADUpdateCompleteCallback
    								):
    						iApplicationUID(aApplicationUID)
    					 ,iAppSearchCriteria(aAppSearchCriteria)
    					 ,iCommandLineExecutable(aCommandLineExecutable)
    					 ,iCommandLineArguments(aCommandLineArguments)
						,iShowProgress(aShowProgress)			
						,iIADUpdateCheck(NULL)
						,iIADParam(NULL)
						,iIADUpdateCompleteCallback(aIADUpdateCompleteCallback)
						,iIADUpdateStatus(KUpdateNotStarted)
												
{
	IADDebug(LOG_DEBUG,_L("In CUpdateChecker Constructor"));
}


/**
* By default Symbian 2nd phase constructor is private.
*/

void CIADUpdateChecker::ConstructL()
					
{
	IADDebug(LOG_DEBUG,_L("In CUpdateChecker ConstructL"));
	
}


TBool CIADUpdateChecker::IsIADFeatureSupportedL()
{
  TBool supported=EFalse;
  	
#ifdef FEATURE_MANAGER_CHECK

    TRAP_IGNORE(FeatureManager::InitializeLibL());
    supported = FeatureManager::FeatureSupported(KFeatureIdIAUpdate);
    FeatureManager::UnInitializeLib();

#endif // FEATURE_MANAGER_CHECK

   return supported;	
}


TBool CIADUpdateChecker::IsUpdateSupported()
{
	TBool supported = EFalse;
	TRAP_IGNORE(supported = IsIADFeatureSupportedL());
	return supported;
}
    	
void CIADUpdateChecker::CheckUpdateL()
{
	IADDebug(LOG_DEBUG,_L("In CheckUpdateL beginning"));

	// we receive the callback notifications.
	iIADUpdateCheck = CIAUpdate::NewL(*this);			

	if ( iIADUpdateCheck ) // IAUpdate is supported in the phone
       {
    	 	IADDebug(LOG_DEBUG,_L(" iIADUpdateCheck object created successfully"));   	      
	       if(!iIADParam)
	       	iIADParam = CIAUpdateParameters::NewL();
	       IADDebug(LOG_DEBUG,_L(" iIADParam object created successfully"));
	       // Search for updates to IAUpdateExampleClient, We know that the 
	       // SIS package uid is KIAParamUid, so let's use that as a 
	       // search criteria.
	       iIADParam->SetUid( iApplicationUID );
	       // Free-form search criteria is not supported in  
	       // IAUpdateExampleClient packages.
	       //iIADParam->SetSearchCriteriaL( KIAParamSearch );
	       // We want iaupdateexampleclient.exe to be started after update is 
	       // finished
	       if(iCommandLineExecutable != KNullDesC)
	       {
               IADDebug(LOG_DEBUG,iCommandLineExecutable );
	       		iIADParam->SetCommandLineExecutableL( iCommandLineExecutable );
	       		iIADParam->SetCommandLineArgumentsL( iCommandLineArguments );	
	       }
	       
	       // We don't want any wait dialog to be shown, let's keep everything 
	       // silent.
	       // EFalse is the default, so no need to call this.
	       if(iShowProgress)
	       	iIADParam->SetShowProgress( iShowProgress );
	       
	       // Check the updates
	       iIADUpdateCheck->CheckUpdates( *iIADParam );
	       
	       iIADUpdateStatus = CIADUpdateChecker::KUpdateInProgress;
	       
       }
       IADDebug(LOG_DEBUG,_L("In CheckUpdateL end"));
}


/** 
 * This callback function is called when the update checking operation has completed.
 *
 * @param aErrorCode The error code of the observed update operation.
 *                   KErrNone for successful completion, 
 *                   otherwise a system wide error code.
 * @param aAvailableUpdates Number of the updates that were found available.
 *
 * @since S60 v3.2
 */
void CIADUpdateChecker::CheckUpdatesComplete( TInt aErrorCode, TInt aAvailableUpdates )
{
    IADDebug(LOG_DEBUG,_L("In CheckUpdatesComplete begin"));

    if ( aErrorCode == KErrNone )
    {
       if ( aAvailableUpdates > 0 )
       {
            
            IADDebug(LOG_DEBUG,_L("No. of updates available: %d"), aAvailableUpdates);
        // There were some updates available. Let's ask if the user 
             //wants to update them.
             iIADUpdateCheck->UpdateQuery();            
       }
       else
       {
           iIADUpdateStatus = KUpdatesUnavailable; 
       }
        
    }
    IADDebug(LOG_DEBUG,_L("In CheckUpdatesComplete end"));
}


/** 
 * This callback function is called when an update operation has completed.
 * Even if multiple functions are provided to start different update operations,
 * this callback function is always called after an update operation has completed.
 *
 * @param aErrorCode The error code of the completed update operation.
 *                   KErrNone for successful completion, 
 *                   otherwise a system wide error code.
 * @param aResult Details about the completed update operation.
 *                Ownership is transferred.
 *
 * @since S60 v3.2
 */
void CIADUpdateChecker::UpdateComplete( TInt aErrorCode, CIAUpdateResult* aResult )
{
    IADDebug(LOG_DEBUG,_L("CIADUpdateChecker::UpdateComplete() ++ "));
    // called only when updated via browser
    // if updated through viewer, it would have been killed. hence this method wont be called.
    IADDebug(LOG_DEBUG,_L("Error code : %d"), aErrorCode );
    if ( aErrorCode == KErrNone )
       {
        IADDebug(LOG_DEBUG,_L("In Update complete begin"));
       // The update process that the user started from IAUpdate UI is now  
       // completed.
       // If the client application itself was updated in the update 
       // process, this callback is never called, since the client is 
       // not running anymore. 
       TInt successCount = aResult->SuccessCount();        
       IADDebug(LOG_DEBUG, _L("%d"), successCount );
       
       if( successCount > 0 )
       {
           // done with the update
           iIADUpdateStatus = CIADUpdateChecker::KUpdateCompleted;
    
           if( iIADUpdateCompleteCallback )
               iIADUpdateCompleteCallback->IADUpdateCallback( aErrorCode );
       }
       else
       {
           iIADUpdateStatus = CIADUpdateChecker::KUpdateFailed;
       }
       
       
       }
   
   // Ownership was transferred, so aResult must be deleted by the client
   delete aResult;  

   //now delete params created
   delete iIADParam;
   iIADParam = NULL;
   
   delete iIADUpdateCheck;
   iIADUpdateCheck = NULL;
   
   IADDebug(LOG_DEBUG,_L("CIADUpdateChecker::UpdateComplete() - "));
}


/** 
 * This callback function is called when an update query operation has completed.
 *
 * @param aErrorCode The error code of the observed query operation.
 *                   KErrNone for successful completion, 
 *                   otherwise a system wide error code.
 * @param aUpdateNow ETrue informs that an update operation should be started.
 *                   EFalse informs that there is no need to start an update
 *                   operation.
 * @since S60 v3.2
 */
void CIADUpdateChecker::UpdateQueryComplete( TInt aErrorCode, TBool aUpdateNow )
{
        IADDebug(LOG_DEBUG,_L("In UpdateQueryComplete begin"));
    if ( aErrorCode == KErrNone )
       {

            IADDebug(LOG_DEBUG,_L("Update now value: %d"), aUpdateNow);
            if ( aUpdateNow )
            {
               // User choosed to update now, so let's launch the IAUpdate UI.
               iIADUpdateCheck->ShowUpdates( *iIADParam );
            }
            else
            {
                //Dont do anything, timer is handled by IAD client
            
                // user selected "Later"
                iIADUpdateStatus = CIADUpdateChecker::KUpdateUserCancelled;
            }
            IADDebug(LOG_DEBUG,_L("In UpdateQueryComplete end"));
       }
}

CIADUpdateChecker::TIadUpdateStatus CIADUpdateChecker::UpdateStatus()
{
    return iIADUpdateStatus;
}
//#endif
