/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that manages 
*				 instances of CVideoObject and handles the Flash device video API.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformclib.h"
#include "saplayer_build.h"

#ifdef FEATURE_DEVICE_VIDEO

#include <collate.h>

#include "symbian_engine.h"
#include "symbian_videoengine.h"
#include "symbian_videoobject.h"
#include "saflash_consts.h"
#include "symbian_stringutils.h"

CVideoRequest* CVideoRequest::NewL(const FI_Video* aVideoRequest, const TInt aRequestType)
{
	CVideoRequest* r = new(ELeave) CVideoRequest(aRequestType);
	CleanupStack::PushL(r);
	r->ConstructL(aVideoRequest);
	CleanupStack::Pop(r);
	return r;
}

CVideoRequest* CVideoRequest::NewL(const TUint32 aVideoID, const TInt aRequestType)
{
	CVideoRequest* r = new(ELeave) CVideoRequest(aRequestType);
	r->iVideoID = aVideoID;
	return r;
}


CVideoRequest::CVideoRequest(TInt aRequestType)
{
	iRequestType = aRequestType;
}

void CVideoRequest::ConstructL(const FI_Video* aVideoRequest)
{
	iVideoID = aVideoRequest->videoID;
	if (aVideoRequest->playSrc && aVideoRequest->playSrc->textLen)
	{
		iPlaySrc = HBufC::NewL(aVideoRequest->playSrc->textLen);
		TPtrC src(aVideoRequest->playSrc->text.utf16String, aVideoRequest->playSrc->textLen);
		iPlaySrc->Des().Copy(src);
	}
	iVideoData.Set((TUint8*)aVideoRequest->videoData, aVideoRequest->videoDataSize);
}


CVideoRequest::~CVideoRequest()
{
	delete iPlaySrc;
}

/*
-----------------------------------------------------------------------------

  CVideoEngine* CVideoEngine::NewL()
  
	Description: Two-phased constructor.
	Comments   :
	
	  Return values: CVideoEngine object pointer
	  
-----------------------------------------------------------------------------
*/
CVideoEngine* CVideoEngine::NewL(CPlatformFlashEngine& aEngine, RFs& aFs) 
{
	CVideoEngine* self = CVideoEngine::NewLC(aEngine, aFs);
	CleanupStack::Pop(self);
    return self;
}

/*
-----------------------------------------------------------------------------

  CVideoEngine* CVideoEngine::NewLC()
  
	Description: Two-phased constructor.
	Comments   : Method leaves pointer to object in cleanup stack
	
	  Return values: CVideoEngine object pointer
	  
-----------------------------------------------------------------------------
*/
CVideoEngine* CVideoEngine::NewLC(CPlatformFlashEngine& aEngine, RFs& aFs)
{
    CVideoEngine* self = new (ELeave) CVideoEngine(aEngine, aFs);
    CleanupStack::PushL(self);
    self->ConstructL();
	return self;
}


/*
-----------------------------------------------------------------------------

  void CVideoEngine::ConstructL()
  
	Description: Second phase constructor.
	Comments   :
	
	  Return values: N/A
	  
-----------------------------------------------------------------------------
*/
void CVideoEngine::ConstructL()
{
	iAsyncObject = CFlhAsyncObject::NewL(CActive::EPriorityHigh, *this);
}

CVideoEngine::CVideoEngine(CPlatformFlashEngine& anEngine, RFs& aFs) 
	: iEngine(anEngine)
	, iFs(aFs)
	, iVideoList( _FOFF(CVideoObject, iLinkItem))	
	, iRequestList(_FOFF(CVideoRequest, iLinkItem))
{
	iVideoObjectIdIndex = 0;
}

CVideoEngine::~CVideoEngine()
{
#ifdef LOG_MESSAGE
	TRAP_IGNORE(iEngine.WriteLogMsgL("CVideoEngine::DeleteAllVideoObjects\n"));
#endif	// LOG_MESSAGE
	
	// Delete all Video objects
	DeleteAllVideoObjects();
	delete iAsyncObject;
}

void CVideoEngine::SetVideoRectL(TUint32 aVideoID, const FI_Rect* aLocation)
{
	CVideoObject* pVideo = FindVideoObject(aVideoID);
    if(pVideo)
    {
        pVideo->SetRectL(*aLocation);
    }
}

TSize CVideoEngine::GetVideoSize(TUint32 aVideoID)
{
    TSize size(0,0);
    CVideoObject* pVideo = FindVideoObject(aVideoID);
    if(pVideo)
    {
        size = pVideo->GetVideoSize();
    }
    return size;
}

void CVideoEngine::DeleteAllVideoObjects()
{
#ifdef LOG_MESSAGE
	TRAP_IGNORE(iEngine.WriteLogMsgL("CVideoEngine::DeleteAllVideoObjects\n",0));
#endif	// LOG_MESSAGE

    while(!iVideoList.IsEmpty()) 
    {
		CVideoObject* videoObject = iVideoList.First();
		iVideoList.Remove(*videoObject);
		delete videoObject;
	}
}

TBool CVideoEngine::HasActivePlayback()
    {
    TSglQueIter<CVideoObject> iter(iVideoList);
    CVideoObject* item = iter;

    if(item)
        return ETrue;
    
    return EFalse;
    }

CVideoObject* CVideoEngine::FindVideoObject(TUint32 aVideoID)
{
	TSglQueIter<CVideoObject> iter(iVideoList);
		
	CVideoObject* item = iter;

	while(item)
	{
		if(item->Id() == aVideoID)
		{
			break;
		}
		item = iter++;
	}
	return item;	
}

void CVideoEngine::CloseOtherVideos(TUint32 aID)
{
	TSglQueIter<CVideoObject> iter(iVideoList);
		
	CVideoObject* item = iter;

	while(item)
	{
		if(item->Id() != aID)
		{
			item->Close();
		}
		item = iter++;
	}
}


void CVideoEngine::DoRunL(const CActive* /*aActive*/)
{
    // Enhancement suggestion: We should only be processing one item in the
    // request list per call to DoRunL. Should process one, and if there are
    // more, call Start() to set this active again.
	while(!iRequestList.IsEmpty())
	{
		CVideoRequest* request = iRequestList.First();
		iRequestList.Remove(*request);
		CleanupStack::PushL(request);
		CVideoObject* pVideoObject = FindVideoObject(request->iVideoID);

		TBool paused = GetFlashEngine().IsPaused();

		if (pVideoObject)
		{
			switch(request->iRequestType)
			{
				case CVideoRequest::EPlayRequest:
				if (!paused)
				{
					// make sure we only play 1 video at a time
					CloseOtherVideos(request->iVideoID);
					iActiveVideoId = request->iVideoID;
					pVideoObject->PlayL(*request);
				}
				break;

				case CVideoRequest::EStopRequest:
				iActiveVideoId = 0;
				pVideoObject->StopL();
				break;

				case CVideoRequest::ECloseRequest:
				iActiveVideoId = 0;
				// tell engine that we aren't doing video anymore.
				iEngine.SetVideoRectL(NULL, EVideoNone);
    			pVideoObject->Close();
				break;
				
				case CVideoRequest::ECancelRequest:
				iActiveVideoId = 0;
				// tell engine that we aren't doing video anymore.
				iEngine.SetVideoRectL(NULL, EVideoNone);
				
				pVideoObject->CancelL();
				break;

				case CVideoRequest::EPauseRequest:
				pVideoObject->PauseL();
				break;

				case CVideoRequest::EResumeRequest:
				if (!paused)
				{
					pVideoObject->Resume();
				}

				case CVideoRequest::EMuteRequest:
				pVideoObject->MuteL(request->iIntegerArg ? ETrue : EFalse);
				break;

				case CVideoRequest::ELoopRequest:
				pVideoObject->SetLooping(request->iIntegerArg ? ETrue : EFalse);
				break;

				case CVideoRequest::ESeekRequest:
				if (!paused)
				{
					pVideoObject->SeekL(request->iIntegerArg);
				}
				break;

				case CVideoRequest::EDestroyRequest:
				pVideoObject->StopL();
				iVideoList.Remove(*pVideoObject);
				delete pVideoObject;
				// tell engine that we aren't doing video anymore.
				iEngine.SetVideoRectL(NULL, EVideoNone);
		    	break;
	    	}
		}

		CleanupStack::PopAndDestroy(request);
	}
}

TInt CVideoEngine::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	if((!iRequestList.IsEmpty()) && (!iAsyncObject->IsActive()))
	{
		iAsyncObject->Start();
	}
	// rborcic -- 2005/09/18 - Workaround for bug on 2.1 devices (see bug #141628).
	// We now leave with KErrNotReady if we are playing sound and try to start a video.
	// However, if we return KErrNotReady from this function, the user will see a system
	// error dialog (-18) displayed. We want to supress this dialog in this case, so
	// return KErrNone instead.
	if (aError == KErrNotReady)
	{
		return KErrNone;
	}
	return aError;
}

/************************************************************************/
/* Interface for SI functions                                           */
/************************************************************************/
TInt16 CVideoEngine::CreateVideoL(const FI_Rect* aLocation, TUint32* aRetVideoID)
{
	TInt16 retVal = 0;
	
#ifdef FEATURE_WINDOWLESS	
	if(iEngine.GetWMode()!= EWindow)
	{
		MFlashClientIf* flashClientIf = iEngine.GetClientIf();
		if(flashClientIf)
		{
			MUserNotiferIf* userNotifier = flashClientIf->GetUserNotifierIf();
					
			if(userNotifier)
			{
				userNotifier->NotifyClientError(KErrNotSupported, ETrue);

			}
			User::Leave(KErrNotSupported);
		}
	}
#endif //FEATURE_WINDOWLESS
	
	if ( aRetVideoID )
	{
		*aRetVideoID = 0;
	}

	// Create a new Video object with iPlayer and videoID
	CVideoObject* pVideoObject = new(ELeave) CVideoObject(*this);
	CleanupStack::PushL(pVideoObject);

	if (aLocation && aRetVideoID )
	{
		if ( pVideoObject->InitPlayerL(&iEngine, *aLocation, iEngine.GetVolume()) )
		{
			*aRetVideoID = ++iVideoObjectIdIndex;
			pVideoObject->SetId(*aRetVideoID);
			iVideoList.AddLast(*pVideoObject);
			retVal = 1;
		}
	}
    iVideoRect = *aLocation;
    CleanupStack::Pop(pVideoObject);

	return retVal;
}

TInt16 CVideoEngine::PlayVideoL(const FI_Video* aVideoInfo)
{
	if ( ! aVideoInfo )
	{
		return SI_GenericError;
	}

	CVideoRequest* request = CVideoRequest::NewL(aVideoInfo, CVideoRequest::EPlayRequest);	
	iRequestList.AddLast(*request);
	if (!iAsyncObject->IsActive())
	{
		iAsyncObject->Start();
	}
    iEngine.SetVideoRectL(&iVideoRect, EDeviceVideo);
	return SI_NoError;
}


void CVideoEngine::CloseVideo(TUint32 aVideoID)
{
	CVideoObject* pVideoObject = FindVideoObject(aVideoID);
    TRAP_IGNORE(iEngine.SetVideoRectL(NULL, EVideoNone));

	if (pVideoObject)
	{
		pVideoObject->Close();
	}
}

TInt16 CVideoEngine::CancelVideoL(TUint32 aVideoID)
{
	CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::ECancelRequest);
	iRequestList.AddLast(*request);
	if (!iAsyncObject->IsActive())
	{
		iAsyncObject->Start();
	}
	// Set active video to "none" immediately to prevent
	// extra stop requests from being queued when we play sound.
	iActiveVideoId = 0;
	
	return SI_NoError;
}


TInt16 CVideoEngine::StopVideoL(TUint32 aVideoID)
{
	CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::EStopRequest);
	iRequestList.AddLast(*request);
	if (!iAsyncObject->IsActive())
	{
		iAsyncObject->Start();
	}
	// Set active video to "none" immediately to prevent
	// extra stop requests from being queued when we play sound.
	iActiveVideoId = 0;

	return SI_NoError;
}


TInt16 CVideoEngine::PauseVideoL(TUint32 aVideoID)
{
	// Pause needs to be handled immediately -- do not add it to the queue.
	TInt16 retVal = SI_GenericError;
	CVideoObject* pVideoObject = FindVideoObject(aVideoID);

	if (pVideoObject)
	{
		if(pVideoObject->PauseL())
        {
            retVal = SI_NoError;
        }
        else
        {
            retVal = 2;
        }
	}
	return retVal;

}


TInt16 CVideoEngine::ResumeVideoL(TUint32 aVideoID)
{
	CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::EResumeRequest);
	iRequestList.AddLast(*request);
	if (!iAsyncObject->IsActive())
	{
		iAsyncObject->Start();
	}

	return SI_NoError;
}

_LIT(KLitSupport16,"mute seek loop\0");
TInt16 CVideoEngine::SupportedVideoMethods(FI_ReturnString* aMethods)
{
	aMethods->SetUTF16String(aMethods, (&KLitSupport16)->Ptr());
	return (TInt16)SI_NoError;
}


TInt16 CVideoEngine::ProcessVideoMethodL(TUint32 aVideoID, const FI_Method* aMethod, FI_ReturnString* aRetString)
{
	TInt16 retVal = SI_GenericError;

	if ( ! aMethod )
	{
		return 0;
	}

	FI_Method method = *aMethod;

	CVideoObject* pVideoObject = FindVideoObject(aVideoID);

	if (pVideoObject)
	{
		switch ( method.method )
		{
		case EVideoMethod_Mute:
			{
				if ( method.argc )
				{
					const TDesC* boolDes=&KFalseStrVal;
					if(pVideoObject->IsMuted())
					{
						boolDes=&KTrueStrVal;
					}

					aRetString->SetUTF16String(aRetString, boolDes->Ptr());
					TPtrC16 methodPtr(method.argv[0].text.utf16String);
					TCollationMethod collectionMethod;
					FlashMemSet( &collectionMethod, 0, sizeof (TCollationMethod) );
					collectionMethod.iFlags = TCollationMethod::EFoldCase;
					TBool bMute = (0 == methodPtr.CompareC(KTrueStrVal, 0, &collectionMethod));

					CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::EMuteRequest);
					request->iIntegerArg = bMute ? 1 : 0;
					iRequestList.AddLast(*request);
					if (!iAsyncObject->IsActive())
					{
						iAsyncObject->Start();
					}

					retVal = SI_NoError;
				}
			}
			break;

		case EVideoMethod_Seek:
			{
				TInt currentPos = pVideoObject->GetPositionL();
				TBuf<10> numBuf;

				numBuf.AppendNum(currentPos);

				aRetString->SetUTF16String(aRetString, numBuf.PtrZ());

				if ( method.argc )
				{
					TPtrC posStr(method.argv[0].text.utf16String, method.argv[0].textLen);
					TInt pos = ToU32(posStr);
					CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::ESeekRequest);
					request->iIntegerArg = pos;
					iRequestList.AddLast(*request);
					if (!iAsyncObject->IsActive())
					{
						iAsyncObject->Start();
					}
					retVal = SI_NoError;
				}
			}
			break;

		case EVideoMethod_Loop:
			{
				if ( method.argc )
				{
					const TDesC* boolDes=&KFalseStrVal;
					if(pVideoObject->IsLooping())
					{
						boolDes=&KTrueStrVal;
					}

					aRetString->SetUTF16String(aRetString, boolDes->Ptr());

					TPtrC16 loop(method.argv[0].text.utf16String);
					TCollationMethod collectionMethod;
					FlashMemSet( &collectionMethod, 0, sizeof (TCollationMethod) );
					collectionMethod.iFlags = TCollationMethod::EFoldCase;
					TBool bLoop = (0 == loop.CompareC(KTrueStrVal, 0, &collectionMethod));

					CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::ELoopRequest);
					request->iIntegerArg = bLoop ? 1 : 0;
					iRequestList.AddLast(*request);
					if (!iAsyncObject->IsActive())
					{
						iAsyncObject->Start();
					}
					retVal = SI_NoError;
				}
			}
			break;
		}
	}
	
	return retVal;
}

void CVideoEngine::DestroyVideoL(TUint32 aVideoID)
{
	CVideoRequest* request = CVideoRequest::NewL(aVideoID, CVideoRequest::EDestroyRequest);
	iRequestList.AddLast(*request);
	if (!iAsyncObject->IsActive())
	{
		iAsyncObject->Start();
	}
}

void CVideoEngine::NotifyNewVolumeLevelL(TInt aNewVolLevel)
{
	// Go through the list of videoObject and set all volumes

	TSglQueIter<CVideoObject> iter(iVideoList);
		
	CVideoObject* item = iter;

	while(item)
		{
		item->NotifyNewVolumeLevelL(aNewVolLevel);
		item = iter++;
		}
	
}

void CVideoEngine::NotifySizeChangedL()
	{
	// Go through the list of videoObject and set all rects
   
	TSglQueIter<CVideoObject> iter(iVideoList);
	CVideoObject* item = iter;

	while(item)
		{
		item->HandleSizeChangedL();
		item = iter++;
		}
		
	}    

/*
 * Host engine calls this function whenever we scroll
 */
void CVideoEngine::NotifyScrollL()
    {
    // we are scrolling... so notify all the video objects about it, so that they redraw themselves at correct location
    TSglQueIter<CVideoObject> iter(iVideoList);
    CVideoObject* item = iter;

    while(item)
        {
        item->HandleScrollL();
        item = iter++;
        }
    }

void CVideoEngine::SetVideoWindow(RWindow* aWindow)
    {
    // we are scrolling... so notify all the video objects about it, so that they redraw themselves at correct location
    TSglQueIter<CVideoObject> iter(iVideoList);
    CVideoObject* item = iter;

    while(item)
        {
        item->SetVideoWindow(aWindow);
        item = iter++;
        }
    }

#endif	// FEATURE_DEVICE_VIDEO
