/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:   
*
*/
#ifndef __ALFCOMPOSITIONCLIENT_H__
#define __ALFCOMPOSITIONCLIENT_H__

#include <e32base.h>
#include <w32std.h>
#include <e32hashtab.h>

class RAlfBridgerClient;
/*
 * CAlfRsSendBuffer creates in construction one CAlfCompositionClient. That client 
 * will wait for request completion. Request is completed, when an application communicaties 
 * to CAlfRenderStage via CAlfCompositionSource client. After request is completed, the 
 * CAlfCompositionClient (owned by CAlfRsSendBuffer) is again set active and it waits for 
 * the next message.
 */

/** Base classes for logistics **/
class CAlfCompositionClientBase: public CActive
	{
public:
    
    enum TAlfCompositionPermission
        {
        EAlfAllowChangeExtent           = 1,
        EAlfAllowChangeAlpha            = 2,
        EAlfAllowChangeOpacity          = 4,
        EAlfAllowChangeRotation         = 8,
        EAlfAllowChangeBackgroundAnim   = 16,
        
        EAlfAllowChangeAll               = 0xff9
        }; 
    
protected:
    CAlfCompositionClientBase(TInt aPriority = CActive::EPriorityHigh):CActive(aPriority)
        {
        CActiveScheduler::Add(this);
        }
        
    IMPORT_C ~CAlfCompositionClientBase();
    IMPORT_C void RequestEventL(TInt aBufferSize, RAlfBridgerClient* aClient = 0);        
    IMPORT_C TInt SendEvent(TInt aOp, const TAny* aEventData, TInt aEventSize);
    IMPORT_C void SetHandleL(TInt aHandle);
    
    // Virtuals
    IMPORT_C virtual void HandleEventL(TInt aEventType, TAny* aEventData);
    // from CActive
    IMPORT_C TInt RunError(TInt aError);

private: // from CActive
    void RunL();
    void DoCancel();

public: // for utils
    TInt Handle() const;
        
private: // data
    class CAlfCompositionClientBaseData;
    CAlfCompositionClientBaseData* iData;
    };

NONSHARABLE_CLASS(CAlfCompositionClientBase::CAlfCompositionClientBaseData):public CBase
    {
    public:
    static CAlfCompositionClientBaseData* NewL(TInt aBufferSize, RAlfBridgerClient* aClient);

    void ConstructL(TInt aBufferSize, RAlfBridgerClient* aClient);

    ~CAlfCompositionClientBaseData();

    RAlfBridgerClient* iClient;
    TBool iOwnsClient;
    HBufC8* iBuffer;
    TBool* iDeleted;
    TInt iBufferSize;
    TInt iHandle;
    };

class MAlfCompositionControl
    {
    public:
       	/** 
    * Enables / disables alpha on composition target 
    **/
    virtual TInt EnableAlpha(TBool aEnable = ETrue) = 0;
    
    /** 
    * Set constant opacity (0..255) multiplier that will be applied to 
    * composition target <in practice, only 256 different shades supported> 
    **/
    virtual TInt SetOpacity(TReal32 aOpacity) = 0;
    
    /** 
    * rotation of composition target
    * <in practice, only 90 degree steps supported, value given will be rounded to closest rotation> 
    **/
    virtual TInt SetRotation(TInt aRotationInDegrees) = 0;
        
    /** 
    * Defines extent of the composition target on a defined screen
    * Does not affect on hit testing, keyboard focus etc. but is just additional information for composition
    **/
    virtual TInt SetExtent(const TRect& aRect, TInt aScaleMode, TInt aScreen) = 0;

    /** 
    * Makes this composition target system wide background surface 
    **/
    virtual void SetIsBackgroundAnim(TBool ETrue) = 0;    
    };   

/** Interface for class that actually controls the composition singleton **/
class MAlfCompositionController: public MAlfCompositionControl
    {
public:
    virtual TInt CreateTargetL(TInt aTarget, TInt aClientHandle, TInt aGrpHandle ) = 0;    
    virtual void SetTargetL(TInt aTarget) = 0;    
    virtual TInt SetOrder(TInt aTop, TInt aBelow, TBool aCombineTargets) =0;
    virtual void DeleteTarget(TInt aTarget)=0;
    }; 

/** Wrapper class IPC <-> MAlfCompositionController **/
NONSHARABLE_CLASS(CAlfCompositionCntrlClient): public CAlfCompositionClientBase
    {
public:
    CAlfCompositionCntrlClient(MAlfCompositionController* aController):iController(aController){}
	IMPORT_C static CAlfCompositionCntrlClient* NewL(RAlfBridgerClient* aClient, MAlfCompositionController* aController);
    // From base
    void HandleEventL(TInt aEventType, TAny* aEventData);
    // from CActive
    TInt RunError(TInt aError);
private:
    MAlfCompositionController* iController;
    
    /*
     * 0, if not binded
     * not zero, if binded to a source. In this case, the iPermittedOperationFlags flags must be checked before issuing an operation
     */
    class TBondageStruct
        {
    public:
        TBondageStruct(TInt aTarget, TInt aPermissions) : iTarget(aTarget), iPermissions(aPermissions){};
        
        TInt iTarget;
        TInt iPermissions;
        };
    
    RHashMap<TInt,TBondageStruct> iBindToSource; 

    TInt32 iPermittedOperations;
    
    

    };

/** Client interface for composition fw information **/
class MAlfCompositionObserver
    {
public:
    /**
    * Called by framework when composition framework has completed a frame and thus can 
    * efficiently handle changes and/or new content
    **/    
    virtual void FrameReady(TInt aScreenNumber) = 0;

    /**
    * Notifications from FW
    **/    
    virtual void RunningLowOnGraphicsMemory() = 0; // running low on resources, clear cahces etc.
    virtual void CompositionTargetHidden() = 0; // target is no more visible on composition scene, 
                                                // free at least slack resources 
    };
    
/**
* Example1:
* // to set target semitransparent 
* CAlfCompositionSource* me = CAlfCompositionSource::NewL(iMyDrawableWindow);
* me->EnableAlpha(); 
* me->SetOpacity(0.5f);
* //...
* delete me; // deletion of client will restore state to original
*
* Example2:
* // to fade in for composition target
* class CMyClass: public CBase, private MAlfCompositionObserver
*     { 
*     CMyClass():iOpacity(0)
*       {
*       }
*     ConstructL()
*       {
*       // ...
*       iAlfCompositionSource = CAlfCompositionSource::NewL(iMyDrawableWindow);
*       iAlfCompositionSource->EnableAlpha(); 
*       iAlfCompositionSource->SetOpacity(iOpacity);
*       }
*     
*     void FrameReady(TInt aScreenNumber)
*       {
*       if (iOpacity < 1)
*           { 
*           iOpacity += 0.03; // just follow refresh linearly      
*           iAlfCompositionSource->SetOpacity(iOpacity);
*           }
*        else
*           { // fade in complete, no need to have composition cntrl anymore
*           delete iAlfCompositionSource;
*           iAlfCompositionSource = 0;
*           }       
*       }    
*
* Example 3:       
* // To grant other application / process a priviledged rights to modify composition target
* const TInt mySuperSecretKeyForHost1 = 0x000FAB10;
* const TInt flagsForHost1 = EAlfAllowChangeExtent | EAlfAllowChangeAlpha; // but not rotation  
*
* CAlfCompositionSource* me = CAlfCompositionSource::NewL(iMyDrawableWindow);
* TInt generatedKey = me->CreatePermissionToken(0x000FAB10, flagsForHost1);
* 
* PassGeneratedKeyToHost(generatedKey); // it already knows the other key..
*
* TBD, will the generates key work even after composition source destroys the
* CAlfCompositionSource instance
*      
**/
class CAlfCompositionSource: public CAlfCompositionClientBase, MAlfCompositionControl
	{
public:

	/** 
	* Constructor 
    **/
	IMPORT_C static CAlfCompositionSource* NewL(RWindow& aClientWindow);

   	/** 
    * Creates handle based on key
    * Flags define which kind of operations are allowed by host application
    * using the toke. Same token can be used multiple times for different clients.
    *
    **/
	IMPORT_C TInt CreatePermissionToken(TInt aKey, TInt aPermissionFlags);

   	/** 
    * Enables / disables alpha on composition target 
    **/
    IMPORT_C TInt EnableAlpha(TBool aEnable = ETrue);
    
    /** 
    * Set constant opacity 0.0 - 1.0f multiplier that will be applied to composition target
    * <in practice, only 256 different shades supported> 
    **/
    IMPORT_C TInt SetOpacity(TReal32 aOpacity);
    
    /** 
    * rotation of composition target
    * <in practice, only 90 degree steps supported, value given will be rounded to closest rotation> 
    **/
    IMPORT_C TInt SetRotation(TInt aRotationInDegrees);
    
    /** 
    * Defines the composition order. On success returns token
    * that can be used to create new CAlfCompositionSource object to
    * control union of theses two composition targets as single object 
    * for convenience
    *
    * Does not affect on hit testing, keyboard focus etc.
    **/
    IMPORT_C TInt SetZOrder(const CAlfCompositionClientBase& aNode, TBool aAbove, TInt aKey = -1); // todo magic
    
    /** 
    * Defines extent of the composition target on a defined screen
    * Does not affect on hit testing, keyboard focus etc. but is just additional information for composition
    **/
    IMPORT_C TInt SetExtent(const TRect& aRect, TInt aScaleMode, TInt aScreen);

    /** 
    * Adds composition observer for better synchronization
    **/
    IMPORT_C void AddCompositionObserverL(MAlfCompositionObserver& aObserver, TInt aScreenNumber = 0); // Todo: KMainScreen);

    /** 
    * Removes composition observer
    **/
    IMPORT_C void RemoveObserver(MAlfCompositionObserver& aObserver);

    /** 
    * Makes this composition target system wide background surface 
    * any other target that was previously used as bg anim will be discarded
    * < should this require some capability like WriteDeviceData ?>
    **/
    IMPORT_C void SetIsBackgroundAnim(TBool aIsBg);    
    
    /** 
    * Internal
    * For testing applications on extrenal screen 
    **/
    IMPORT_C void EnableKeyboard(TBool aEnable, TInt aScreen);

    IMPORT_C ~CAlfCompositionSource();

protected: // utils / baseclass
    IMPORT_C virtual void HandleEventL(TInt aEventType, TAny* aEventData);
    virtual void ConstructL(TInt aHandle1, TInt aHandle2);    
    TInt Handle();

protected: // data
    class CAlfCompositionSourceData;
    CAlfCompositionSourceData* iData;
    };

/**
Example1:
* // to set target from other app semitransparent 
* CAlfCompositionHost* me = CAlfCompositionHost::NewL(tokenFromSource, myCompositionKey);
* me->EnableAlpha(); 
* me->SetOpacity(0.5f);
* //...
**/
class CAlfCompositionHost: public CAlfCompositionSource
	{
public:
	/** 
	* Constructor for application that wishes modify 
	* composition information for some other process, see CreatePermissionToken()
    *
    * Leaves using KErrNotFound, if token is not valid
    * 
    * If the binded target layer is destroyed by its owner, any Set operations 
    * will have no effect.
    */
	IMPORT_C static CAlfCompositionHost* NewL(TInt aToken, TInt aKey);

	void ConstructL(TInt aToken, TInt aKey);
	
	
//private: // data
//    class CAlfCompositionHostData;
//    CAlfCompositionHostData* iData;
    };


// ================ Posting like SW interface ============================================
// 
// =======================================================================================

/** Inferface for the pixel buffer provider to provide information, frames and get events concerning context events **/
class MAlfBufferProvider
    {
public:
    
    enum TBufferFormat
        {
        ESourceFormatRGB_565 = 0,
        ESourceFormatXRGB_8888 = 1, 
        ESourceFormatARGB_8888 = 2,
        ESourceFormatARGB_8888_PRE = 3
        };

    struct TBufferCreationAttributes
        {
        /*
         * Buffer size
         */
        TInt iWidth;
        TInt iHeight;
        /**
         * iStride should be at least bytes * width.
         * iStride may still be adjusted by CAlfCompositionPixelSource for the target HW driver implementation
         * Client should always use iStride value available after CAlfCompositionPixelSource::ActivateL call.
         **/
        TInt iStride;
        
        TInt iAlignment;
        TBufferFormat iFormat;
        };

    /**
     * Called by framework when the next buffer is ready and available for the client to draw.
     * 
     * @return ETrue to continue drawing of frames. EFalse to pause drawing.
     *          After pausing drawing client needs to call CAlfCompositionPixelSource::ActivateL to
     *          continue drawing of frames 
     **/
    virtual TBool ProduceNewFrameL(const TRegion& aVisibleRegion, TUint8*& aBuffer) = 0;

    /**
     * Called by framework when new buffer is been created. Client should provide desired creation attributes.
     **/
    virtual TBufferCreationAttributes& BufferAttributes() = 0;
    
    /**
     * Notification from FW 
     **/
    virtual void ContextAboutToSuspend() =0;
    virtual void OnActivation() = 0;
    };

/**
* Example1:
* // to draw frames with CAlfCompositionPixelSource API
* class CMyClass: public CBase, public MAlfBufferProvider
*     { 
*     private:
*         CAlfCompositionPixelSource* iAlfCompositionPixelSource;
*         MAlfBufferProvider::TBufferCreationAttributes iAttributes;
*     public:
*     
*     CMyClass()
*       {
*       
*       }
*     ConstructL()
*       {
*       // ...
*       iAttributes.iWidth = width;
*       iAttributes.iHeight = height;
*       iAttributes.iFormat = MAlfBufferProvider::ESourceFormatARGB_8888;
*       iAttributes.iStride =  iAttributes.iWidth * 4;
*       iAttributes.iAlignment = 32;
*       
*       iAlfCompositionPixelSource = CAlfCompositionPixelSource::NewL(*this, iMyDrawableWindow);
*       iAlfCompositionPixelSource->iActivateL()
*       }
*     
*    TBool ProduceNewFrameL(const TRegion& aVisibleRegion, TUint8*& aBuffer)
*       {
*       // ...
*       // copy pixel data to aBuffer
*       TUint8* from = (TUint8*)bitmap->DataAddress();
*       TUint bytes = bitmap->SizeInPixels()*4;
*       for (TInt y = bitmap->SizeInPixels().iHeight; y > 0; y--)
*           {
*           Mem::Copy(aBuffer, from, bytes);
*           aBuffer += iAttributes.iStride;
*           from += bitmap->DataStride();
*           }
*       // ...
*       return ETrue; // to continue getting ProduceNewFrameL callbacks
*       };
*
*    MAlfBufferProvider::TBufferCreationAttributes& BufferAttributes() { return iAttributes };
*   
*    void ContextAboutToSuspend() {};
*    void OnActivation() {};
*    // ...
*    }
**/
class CAlfCompositionPixelSource: public CAlfCompositionSource
	{
public:
    /** 
     * Constructor 
     **/
    IMPORT_C static CAlfCompositionPixelSource* NewL(MAlfBufferProvider& aProvider, RWindow* aWindow = NULL);
    
    /**
     *  Activate drawing for the first time or continue drawing if it is being paused or suspended
     **/
	IMPORT_C void ActivateL();
    
	/**
	 * Suspend drawing 
	 **/
    IMPORT_C void Suspend();

    IMPORT_C ~CAlfCompositionPixelSource();

    /**
     * Internal
     * For drawing one frame
     **/
    TBool DrawFrameL(TRequestStatus& aStatus, TInt aBufferNumber = 0);

protected:

    /**
     * Internal
     * From base class 
     **/
    void HandleEventL(TInt aEventType, TAny* aEventData);                                          

private: // Internal / utils
    
    void ConstructL(MAlfBufferProvider& aProvider, RWindow* aWindow);
    void ConstructSurfaceL(MAlfBufferProvider::TBufferCreationAttributes& aCreationAttributes);
    void FreeSurface();
    TUint8* Buffer(TInt aBufferNumber);

    
private: // data
    
    class CAlfCompositionPixelSourceData;
    CAlfCompositionPixelSourceData* iData;
    };


#endif // #define __ALFCOMPOSITIONCLIENT_H__
