/*
* Copyright (c) 2002-2004 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  AO that completes on a change in the volume property.  Used for 
*                the new Central Repository Server.  Used in EKA2 builds only
*
*/

#include <eikapp.h>
#include <aknconsts.h>   
#include "VolumePropertyWatch.h"

#ifndef ENABLE_SF_CHANGES
#include <Flashlite_2_0PrivateCRKeys.h> //senRep keys
#else
#include "Flashlite_2_0PrivateCRKeys.h" 
#endif

#include <AknGlobalNote.h> 
#include "symbian_playerview.h"

// CONSTANTS
const TInt KFirstTimerExpiryInterval = 1; // Expire immediately
const TInt KTimerExpiryInterval = KAknStandardKeyboardRepeatRate;

// ----------------------------------------------------
// CVolumePropertyWatch::NewL
// Description: static constructor.
// Input:      aMinVolume : minimum volume value
//             aMaxVolume : maximum volume value
//            aObserver: link to the calling object
// Output: none
// Return: CVolumePropertyWatch object
// ----------------------------------------------------
CVolumePropertyWatch* CVolumePropertyWatch::NewL(    const TInt& aMinVolume,
                                                    const TInt& aMaxVolume,
                                                    MVolumeObserver* aObserver)
{
    CVolumePropertyWatch* me=new(ELeave) CVolumePropertyWatch(aMinVolume,aMaxVolume);
    CleanupStack::PushL(me);
    me->ConstructL(aObserver);
    CleanupStack::Pop(me);
    return me;
}

// ----------------------------------------------------
// CVolumePropertyWatch::CVolumePropertyWatch
// Description: constructor.
// Input:      aMinVolume : minimum volume value
//             aMaxVolume : maximum volume value
// Output: none
// Return: none
// ----------------------------------------------------
CVolumePropertyWatch::CVolumePropertyWatch(const TInt& aMinVolume, const TInt& aMaxVolume):
                                                iMinVolume(aMinVolume),
                                                iMaxVolume(aMaxVolume),
                                                iInterfaceSelector(NULL),
                                                iSelector(NULL),
                                                iCenRepos(NULL)
{
}

// ----------------------------------------------------
// CVolumePropertyWatch::ConstructL
// Description: second phase constructor.
// Input:  aObserver: link to the calling object
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumePropertyWatch::ConstructL(MVolumeObserver* aObserver)
{
    iObserver = aObserver;

    //
    // Open a connection to receive Volume Key events.
    iSelector = CRemConInterfaceSelector::NewL();
    iInterfaceSelector = CRemConCoreApiTarget::NewL(*iSelector,*this);
    iSelector->OpenTargetL();
    // Timer for implementing repeat
    iTimer = CPeriodic::NewL(CActive::EPriorityStandard);
    
    //
    // Open a connection to the Central Repository...
    TRAP_IGNORE(iCenRepos = CRepository::NewL(KCRUIdFlashliteViewer));
}

// ----------------------------------------------------
// CVolumePropertyWatch::~CVolumePropertyWatch
// Description: destructor.
// Input:  none
// Output: none
// ----------------------------------------------------
CVolumePropertyWatch::~CVolumePropertyWatch()
{
    if(iSelector)
    {
        delete iSelector;
        iInterfaceSelector=NULL;
        iSelector=NULL; //iSelector has been deleted by "delete iInterfaceSelector"
    }
    if(iTimer)
        {
        iTimer->Cancel();
        delete iTimer;
        }
    if(iCenRepos)
    {
        delete iCenRepos;
    }
}

// ----------------------------------------------------
// CVolumePropertyWatch::GetCurrentVolume
// Description: gets the current volume level from the 
//   central directory.
// Input:  none
// Output: none
// Return: volume level
// ----------------------------------------------------
TInt CVolumePropertyWatch::GetCurrentVolume()
{
    TInt currentVolume(0);
    //
    // Get the current volume
    if(iCenRepos)
        iCenRepos->Get(KFlashliteVwrVolume, currentVolume);
    
    #if defined(__WINS__) //offset required on the emulator
        currentVolume += WINS_DEFAULT_VOLUME;
    #endif
    
    return currentVolume;
}

// ----------------------------------------------------
// CVolumePropertyWatch::MrccatoCommand
// Description: A Volume key command has been received
//        (from MRemConCoreApiTargetObserver)
// Input:aOperationId The operation ID of the command
//         aButtonAct The button action associated with the command.
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumePropertyWatch::MrccatoCommand(TRemConCoreApiOperationId aOperationId, 
                             TRemConCoreApiButtonAction aButtonAct )
    {
    switch(aOperationId)
        {
        case ERemConCoreApiVolumeUp:
            {
            switch (aButtonAct)
                {
                case ERemConCoreApiButtonPress:
                    {
                     //Start Timer
                     if (!iTimer->IsActive())
                        {
                        iCommandId = EVolumeControlCmdVolumeUpByOne;
                        iTimer->Start(KFirstTimerExpiryInterval,
                                      KTimerExpiryInterval, 
                                      TCallBack(TimerCallback, this));
                        }
                    break;
                    }
                case ERemConCoreApiButtonRelease:
                    {
                    iTimer->Cancel();
                    break;
                    }
                case ERemConCoreApiButtonClick:
                    {
                    FilterAndSendCommand(EVolumeControlCmdVolumeUpByOne);
                    break;
                    }
                default:
                    {
                    break;
                    }
                }
            break;
            }
        case ERemConCoreApiVolumeDown:
            {
            switch (aButtonAct)
                {
                case ERemConCoreApiButtonPress:
                    {
                    //Start Timer
                    iCommandId = EVolumeControlCmdVolumeDownByOne;
                    if (!iTimer->IsActive())
                        {
                        iTimer->Start(KFirstTimerExpiryInterval,
                                      KTimerExpiryInterval, 
                                      TCallBack(TimerCallback, this));
                        }
                     break;
                    }
                case ERemConCoreApiButtonRelease:
                    {
                    iTimer->Cancel();
                    break;
                    }
                case ERemConCoreApiButtonClick:
                    {
                    FilterAndSendCommand(EVolumeControlCmdVolumeDownByOne);
                    break;
                    }
                default:
                    {
                    break;
                    }
                }
            break;
            }
        default:
            {
            break;
            }
        }        
    }

// -----------------------------------------------------------------------
// CVolumePropertyWatch::HandleRepeatEvent
// -----------------------------------------------------------------------
//
void CVolumePropertyWatch::HandleRepeatEvent()
    {
    FilterAndSendCommand(iCommandId);
    }

// -----------------------------------------------------------------------
// CVolumePropertyWatch::TimerCallback
// -----------------------------------------------------------------------
//
TInt CVolumePropertyWatch::TimerCallback(TAny* aPtr)
    {
    static_cast<CVolumePropertyWatch*>(aPtr)->HandleRepeatEvent();
    return KErrNone;
    } 

// -----------------------------------------------------------------------
// CVolumePropertyWatch::FilterAndSendCommand
// -----------------------------------------------------------------------
//
void CVolumePropertyWatch::FilterAndSendCommand(TInt aCommandId)
    {
    TInt currentVolume = 0;
    if (iCenRepos)
        {
        iCenRepos->Get(KFlashliteVwrVolume,currentVolume);    
        }    
    switch(aCommandId)
        {
        case EVolumeControlCmdVolumeUpByOne:
			// When muted, CenRepos sets currentVolume to a negative value.
        	// so, get currentVolume within the valid range.
        	if (currentVolume < iMinVolume) 
	    	{
	    		currentVolume = iMinVolume;	
	    	}
	    	else
	    	{
	    		currentVolume=(currentVolume<iMaxVolume)?++currentVolume:iMaxVolume;	
	    	}            
            break;
        case EVolumeControlCmdVolumeDownByOne:
        	currentVolume=(currentVolume>KVolumeMuteValue)?--currentVolume:KVolumeMuteValue;	        	            
            break;  
        default:
            break;      
        } 
    if (iCenRepos)    
        {
        iCenRepos->Set(KFlashliteVwrVolume,currentVolume);    
        }
    
#if defined(__WINS__) //offset required on the emulator
        currentVolume += WINS_DEFAULT_VOLUME;
#endif        
    iObserver->VolumeChanged(currentVolume);    
    }

// End of File

