/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class 
*
*/


#ifndef __XPS_MEDIA_STREAM_H
#define __XPS_MEDIA_STREAM_H

#include <rtpheader.h>
#include "helixdef.h"
#include "CXPSPacketSink.h"

// #define __DUMP_TO_FILE

class CFLRtpSource;




class TStreamPacketData
{
public:
    TUint8*          iFrameData;
    TUint            iDataLength;
    TUint            iTimeStamp;
    TBool            iEmbeddedVideo;
    TSize            iPadSize;
    TFrameType       iFrameType;
    TBool            iPacketConsumed;    
};

class TAudioInfo
{
public:
    TInt    iSampleRate;
    TInt    iSampleLength;
    TInt    iChannels;
};

class TAvcStreamInfo
{
public:
    TInt    iProfileInfo;
    TInt    iConstraintInfo;
    TInt    iLevelInfo;
};

class TStreamConfigData
{
public:
    TUint8* iConfigData;
    TInt    iConfigLength;    
};

enum TXPSStreamType
{
    EXPSUnkown,
    EXPSVideoStream,
    EXPSAudioStream
};








// -------------------------------------------------------------------------
// class CRTPPacket
//
// Represent the RTP packet used to wrap the media data.
// --------------------------------------------------------------------------
// 

class CRTPPacket : public CBase
{
public:
    CRTPPacket() 
    {
        iRTPPayLoadData = NULL;
        iPayloadDataLength = 0;
    }
    
    ~CRTPPacket()
    {
        delete[] iRTPPayLoadData;
    }
    
public:
    TRtpRecvHeader   iRTPRecvHdr;
    TUint8*          iRTPPayLoadData;
    TUint            iPayloadDataLength;
    TBool            iTSAdjusted;
};

// -------------------------------------------------------------------------
// class CXPSMediaStream
//
// Represent the packet stream buffering and supplying RTP packets to XPS.
// --------------------------------------------------------------------------
// 

class CXPSMediaStream : public CBase
{
public:
    /*----------------------------------------------------------------------- 
    State of this media stream object.
                                                  stop()
                               |---------------------------------------|
                               |                                       |
                               |                                       |
                start()        |       pause()              stop()     |
    (initial)------------->(started)------------>(paused)----------->(stopped)
                                    <------------         
								       resume()
     -----------------------------------------------------------------------
    */
    enum TStreamState
    {
        EStreamInitial, //Not ready
        EStreamStarted, //Start pushing packets to xps sink
        EStreamPaused,  //Does not push packets to xps sink
        EStreamStopped  //Done. Cannot be started again. To be destructed next.
    };
        

    /*-----------------------------------------------------------------------
    State of XPS sink.
		         KErrOverflow == Enque()
    (xps_ready)------------------------->(xps_overflow)
          |     <-------------------------
	      |	      RestorePacketSupply() 
          |
          |StreamEnd()
          |
          |
          v
    (xps_streamend)
    ------------------------------------------------------------------------
    */
    enum TXpsState
    {
        EXpsReady,     //xps sink is ready to take packets
        EXpsOverflow,  //xps sink queue has overflowed
        EXpsStreamEnd  //xps sink has been signalled that there are no more input packets.
    };


public:
    static CXPSMediaStream* NewLC(const TInt&     aStreamId, 
                                  CXPSPacketSink& aSink, 
                                  CFLRtpSource&  aStreamHandler);
    ~CXPSMediaStream();

	void ConstructL();
        
protected:
	CXPSMediaStream(const TInt& aStreamId, CXPSPacketSink& aSink, CFLRtpSource& aStreamHandler);

public:
    //Stream commands
           void             Start();
           void             Pause();    
           void             Resume();
           void             Reset();
           void             Stop();
           TStreamState     State();

           TInt             AddPacket(const TStreamPacketData& aPktData);
           void             RestorePacketSupply();
           void             StreamEnd();
           TBool            IsStreamEnded();
		   void             Restart();


           TInt             StreamId() const;

           void             SetStreamType(const TXPSStreamType& aStreamType);
           TXPSStreamType   StreamType() const;

           void             SetCodecType(const TCodecType& aCodecType);
           TCodecType       CodecType() const;

           TInt             SetStreamConfig(const TStreamConfigData& aStreamConfig);
           TUint8*          GetStreamConfig();

           void             SetAudioInfo(const TAudioInfo& aAudioInfo);
           void             GetAudioInfo(TAudioInfo& aAudioInfo);
  
    //Some custom APIs
           void             GetAvcStreamInfo(TAvcStreamInfo& aVideoInfo);
           void             SetAvcStreamInfo(const TAvcStreamInfo& aAvcInfo);    

           TUint            GetFirstPacketTimeStamp();
           TUint            CurrentTimestamp();
           TInt             PacketCount() const;
        
private:
           void             PushPacket();
           TRtpRecvHeader   InsertRtpHeader(const TUint& aTimeStamp);
           void             AdjustTimestamp(CRTPPacket* aPacket);
           TBool            TryXpsStreamEnd();            
           void             LogSummary();
    
#ifdef __DUMP_TO_FILE    
           void             DumpToFile(TInt aStreamId, const TUint8* aData, TUint aDataLength);
           void             DumpHeaderToFile(TInt aStreamId, TUint aDataLength, TRtpRecvHeader aHeader);
           void             LogFps(TInt aStreamId, TUint aFrameNum, TUint aTotalTime);
#endif
    
private:
    TInt                          iStreamId;
    TXPSStreamType                iStreamType;
    TCodecType                    iCodecType;
    TAudioInfo                    iAudioInfo;
    TAvcStreamInfo                iAvcStreamInfo;
    TStreamConfigData             iStreamConfigData;

    TStreamState                  iStreamState;
    TXpsState                     iXpsState;

    RPointerArray<CRTPPacket>     iPacketArray;  //Packet store pending to be pushed to xps
    TInt                          iSeqNumber;
    TUint                         iTimeDiff;
    TUint                         iPrevTS;
    TUint                         iSumOfDiff;
    TUint32                       iTimestamp;
    TReal                         iFrameRate;
    TBool                         iStreamEndSignalled;

    CXPSPacketSink&               iSink;
    CFLRtpSource&                 iStreamHandler;

	TBool					      iDoLog;
};

#endif // __XPS_MEDIA_STREAM_H
