/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class CFLPlatformDecoderInterface and other 
		 		 related utilities
*
*/


// -----------------------------------------------------------------
// NOTE: This MUST be the first include of every source file.
//		 It contains all the #define build parameters.

#include "platformbuild.h"		

// -----------------------------------------------------------------

#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include "commondef.h"

#include "flplatformdecoderinterface.h"
#include "symbian_engine.h"
#include "flplatformdecoder.h"
#include "symbian_plugin_interface.h"

#include "saplayer_build.h"

const TInt 		KMaxVideoPositioToBlt	 = 10000;

#ifdef __DEBUG_LOGS__

 #define __FILE_MODE_LOGGING

#include <flogger.h>

void DebugLog( TRefByValue<const TDesC> aText, ... )
    {    
    VA_LIST args;
    VA_START( args, aText );
    
    TBuf<512> buf;
    buf.FormatList( aText, args );

#ifdef __FILE_MODE_LOGGING

    RFileLogger logger;
    
    TInt ret=logger.Connect();
    
    if (ret==KErrNone)
        {
        logger.SetDateAndTime( EFalse,EFalse );
        logger.CreateLog( _L("HelixLogs"), _L("HelixCodec.log"), EFileLoggingModeAppend );       
        logger.Write(buf);
        }

    logger.Close();

#else

	RDebug::Print(buf); 

#endif

    VA_END( args );
    }
    
#else 

void DebugLog( TRefByValue<const TDesC> /*aText*/, ... )
    {  
    }

#endif

// -------------------------------------------------------------------------
// CFLPlatformDecoderInterface::CFLPlatformDecoderInterface(const TUserMode& aUserMode)
//
// Constructor
// --------------------------------------------------------------------------
// 

CFLPlatformDecoderInterface::CFLPlatformDecoderInterface(const TUserMode& aUserMode)
{
    iInstance = NULL;
    iInstanceFuncs = NULL;
    iUserMode = aUserMode;
    iHelixMode = EFalse;
}

// -------------------------------------------------------------------------
// TTimeIntervalMicroSeconds CFLPlatformDecoderInterface::GetVideoPosition()
//
// Gets the current played time (position) from the helix video player. 
// --------------------------------------------------------------------------
// 

TTimeIntervalMicroSeconds CFLPlatformDecoderInterface::GetVideoPosition()
{
		if(iHelixMode)
	 	{
			CFLPlatformDecoder*  decoderInstance = (CFLPlatformDecoder*) iInstance;
			if(decoderInstance)
				return decoderInstance->GetVideoPosition();
		}
		return KMaxVideoPositioToBlt;
}

// -------------------------------------------------------------------------
// CFLPlatformDecoderInterface* CFLPlatformDecoderInterface::NewL(const TUserMode& aUserMode)
//
// NewL static Constructor
// --------------------------------------------------------------------------
// 

CFLPlatformDecoderInterface* CFLPlatformDecoderInterface::NewL(const TUserMode& aUserMode)
{
    CFLPlatformDecoderInterface* self = new (ELeave) CFLPlatformDecoderInterface(aUserMode);
    return self;
}

// -------------------------------------------------------------------------
// CFLPlatformDecoderInterface::~CFLPlatformDecoderInterface() 
//
// Destructor
// --------------------------------------------------------------------------
// 
    
CFLPlatformDecoderInterface::~CFLPlatformDecoderInterface() 
{
    if(iHelixMode)
    {
    	if( iInstance )
    	{
        	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*) iInstance;
    		delete decoder;
    	}
    }
}

// -------------------------------------------------------------------------
// IVideoDecompressorInstance* 
// CFLPlatformDecoderInterface::CreateVideoDecompressor( MM_Object* aMMObj, U32 codecType )
//
// Creates the video decompressor.
// --------------------------------------------------------------------------
// 

IVideoDecompressorInstance* CFLPlatformDecoderInterface::CreateVideoDecompressorL( 
															MM_Object* aMMObj, 
															void * aAllocator, 
															U32 codecType) 
{
	LOG(***************************************************);
	
	CPlatformFlashEngine* hostEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    
    if( !hostEngine )
    {
    	return NULL;
    }
    
    iHelixMode = hostEngine->IsHelixFlvMode();
    
    iInstance = (IVideoDecompressorInstance*) CFLPlatformDecoder::NewL(aAllocator,
																		iUserMode, 
																		(TCodecType) codecType, 
																		*hostEngine);
    return iInstance;
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoderInterface::DispatchHXCommand()
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoderInterface::DispatchHXCommand(const THXVideoDecCommand& aCommand, 
											 const THXVDCmdArg& aArgInfo)
{
	if( !iInstance || !iHelixMode )
	{
		return;
	}
	
	CFLPlatformDecoder*  decoderInstance = (CFLPlatformDecoder*) iInstance;

	if( decoderInstance )
	{
		decoderInstance->DispatchHXCommand(aCommand, aArgInfo);
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoderInterface::MaxVolume()
// 
//
// Returns the maximum volume value of Hx Video player. 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoderInterface::MaxVolume() const
{
	if( !iInstance || !iHelixMode )
	{
		return KErrNotSupported;
	}
	
	CFLPlatformDecoder*  decoderInstance = (CFLPlatformDecoder*) iInstance;

	if( decoderInstance )
	{
		return decoderInstance->MaxVolume();
	}
	
	return KErrNotSupported;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoderInterface::GetSwfVolume()
// 
//
// Returns the current swf volume value
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoderInterface::GetSwfVolume() const
{
	if( !iInstance || !iHelixMode )
	{
		return KErrNotSupported;
	}
	
	CFLPlatformDecoder*  decoderInstance = (CFLPlatformDecoder*) iInstance;

	if( decoderInstance )
	{
		return decoderInstance->GetSwfVolume();
	}
	
	return KErrNotSupported;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoderInterface::
// DestroyVideoDecompressor( MM_Object * aMMObj, 
//							 IVideoDecompressorInstance * decompressorInstance )
//
// Destroys the video decompressor.
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoderInterface::DestroyVideoDecompressor( MM_Object * /*aMMObj*/, 
						   IVideoDecompressorInstance * /*decompressorInstance*/ )
{
	LOG(CFLPlatformDecoderInterface::DestroyVideoDecompressor() ++);
	
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*) iInstance;

    if( decoder )
    {
    	delete decoder;
    	iInstance = NULL;
    }
	LOG(CFLPlatformDecoderInterface::DestroyVideoDecompressor() --);    
}
 
// -------------------------------------------------------------------------
// IVideoDecompressorInstance*  CFLPlatformDecoderInterface::DecompresssorInstance()
//
// Returns the video decompressor instance.
// --------------------------------------------------------------------------
// 

IVideoDecompressorInstance*  CFLPlatformDecoderInterface::DecompresssorInstance()
{
    return iInstance;
}

// -------------------------------------------------------------------------
// TBool CFLPlatformDecoderInterface::IsComplete() const
//
// Tells whether playback is completed.  
// --------------------------------------------------------------------------
// 

TBool CFLPlatformDecoderInterface::IsComplete() const
{
	if( iHelixMode )
	{	
		CFLPlatformDecoder* decoder = (CFLPlatformDecoder*) iInstance;

	    if( decoder )
	    {
	    	return decoder->IsComplete();
	    }
	}  

	return EFalse;
}
